util_report_library_entry



proc reiterate_through_chunk {db which_chunk template} {
  set return_html ""
  switch $which_chunk {
    news {
      set news [most_recent_news $db]
      for {set i 0} {$i < [llength $news]} {incr i} {
        set item [lindex $news $i]
        set news_item_id [lindex $item 0]
        set title [lindex $item 1]
        set release_date [lindex $item 2]
        set body [lindex $item 3]
        set news_link "/news/item.tcl?scope=public&news_item_id=$news_item_id"
        set pretty_release_date [util_AnsiDatetoPrettyDate $release_date]
        append return_html [subst $template]
      }
    }
    threads {
      set bboard [most_active_threads $db]
      for {set i 0} {$i < [llength $bboard]} {incr i} {
        set article [lindex $bboard $i]
        set msg_id [lindex $article 0]
        set count [lindex $article 1]
        set topic [lindex $article 2]
        set one_line [lindex $article 3]
        set body [lindex $article 4]
        set msg_link "/bboard/q-and-a-fetch-msg.tcl?msg_id=$msg_id"
        append return_html [subst $template]
      }
    }
    calendar {
      set events [current_calendar_events $db]
      for {set i 0} {$i < [llength $events]} {incr i} {
        set event [lindex $events $i]
        set calendar_id [lindex $event 0]
        set category [lindex $event 1]
        set title [lindex $event 2]
        set body [lindex $event 3]
        set start_date [lindex $event 5]
        set end_date [lindex $event 6]
        set calendar_link "/calendar/item.tcl?scope=public&calendar_id=$calendar_id"
        set pretty_start_date [util_AnsiDatetoPrettyDate $start_date]
        set pretty_end_date [util_AnsiDatetoPrettyDate $end_date]
        append return_html [subst $template]
      }
    }
    recent_classifieds {
      set classifieds [most_recent_classifieds $db]
      for {set i 0} {$i < [llength $events]} {incr i} {
        set ad [lindex $classifieds $i]
        set classified_ad_id [lindex $ad 0]
        set domain [lindex $ad 1]
        set one_line [lindex $ad 2]
        set posted [lindex $ad 3]
        set manufacturer [lindex $ad 4]
        set classified_ad_link "/gc/view-one.tcl?classified_ad_id=$classified_ad_id"
        set pretty_posted [util_AnsiDatetoPrettyDate $posted]
        append return_html [subst $template]
      }
    }
    active_auctions {
      set auctions [most_active_classified_auctions $db]
      for {set i 0} {$i < [llength $auctions]} {incr i} {
        set ad [lindex $auctions $i]
        set classified_ad_id [lindex $ad 0]
        set domain [lindex $ad 1]
        set one_line [lindex $ad 2]
        set posted [lindex $ad 3]
        set manufacturer [lindex $ad 4]
        set count [lindex $ad 5]
        set classified_ad_link "/gc/view-one.tcl?classified_ad_id=$classified_ad_id"
        set pretty_posted [util_AnsiDatetoPrettyDate $posted]
        append return_html [subst $template]
      }
    }
  }
  return $return_html
}


proc most_recent_news {db} {
  set MemoizeTime [ad_parameter NewsCacheLength chunks 7200]
  return [util_memoize "most_recent_news_internal $db" $MemoizeTime]

}


proc most_recent_news_internal {db} {
  #returns a list of lists. the child lists will contain the news_id, title, release_date, body, 
  #and whether or not it's in html, in that order
  set news_items [list]
  set MaxItems [ad_parameter MaxNews chunks 3]
  set MaxLength [ad_parameter MaxNewsBodyLength chunks 120]
  set selection [ns_db select $db "SELECT news_item_id, title, body, html_p, release_date FROM news_items WHERE [db_sysdate] BETWEEN release_date AND expiration_date AND approval_state = 'approved' ORDER BY release_date DESC, creation_date DESC"]

  set CurItems 0
  while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set article [list $news_item_id $title $release_date]
    lappend article [string trim [string range $body 0 [expr [string first " " [string range $body $MaxLength end]] + [expr $MaxLength - 1]]]]
    lappend article $html_p
    lappend news_items $article
    incr CurItems
    if { $CurItems >= $MaxItems } {
      ns_db flush $db
      break
    }
  }

  return $news_items
}

proc most_active_threads {db} {
  set MemoizeTime [ad_parameter BBoardCacheLength chunks 7200]
  return [util_memoize "most_active_threads_internal $db" $MemoizeTime]
}

proc most_active_threads_internal {db} {
  #returns a list of lists. the child lists will contain the root_msg_id, total number of messages under that root,
  #the topic, the subject, and the first x characters of the body of the root msg in that order
  set db2 [ns_db gethandle subquery]
  set threads [list]
  set MaxThreads [ad_parameter MaxBboardThreads chunks 5]
  set MaxLength [ad_parameter MaxMsgBodyLength chunks 120]
  set MessageAge [ad_parameter CountOldMessages chunks 10]
  set selection [ns_db select $db "SELECT root_msg_id, count(*) as count from bboard WHERE posting_time >= ([db_sysdate] - $MessageAge)::datetime GROUP BY root_msg_id"]

  set CurThreads 0
  while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set sub_selection [ns_db 0or1row $db2 "SELECT topic, one_line, message as body FROM bboard, bboard_topics WHERE bboard.topic_id = bboard_topics.topic_id AND msg_id = '$root_msg_id'"]
    set_variables_after_subquery
    set TruncBody [string trim [string range $body 0 [expr [string first " " [string range $body $MaxLength end]] + [expr $MaxLength - 1]]]]
    set message [list $root_msg_id $count $topic $one_line $TruncBody]
    lappend threads $message
    incr CurThreads
    if { $CurThreads >= $MaxThreads } {
      ns_db flush $db
      break
    }
  }
  ns_db releasehandle $db2
  return $threads
}


proc current_calendar_events {db} {
  set MemoizeTime [ad_parameter EventsCacheLength 7200]
  return [util_memoize "current_calendar_events_internal $db" $MemoizeTime]
}

proc current_calendar_events_internal {db} {
  #returns a list of lists. the child lists will contain calendar_id, category, title, body, 
  #whether the body is in html, start date, end date - in that order.
  set calendar [list]
  set MaxEvents [ad_parameter MaxEvents chunks 5]
  set MaxLength [ad_parameter MaxEventBodyLength chunks 120]
  set selection [ns_db select $db "SELECT calendar_id, category, title, body, html_p, start_date, end_date FROM calendar, calendar_categories WHERE calendar.category_id = calendar_categories.category_id AND approved_p = 't' AND [db_sysdate] <= end_date ORDER BY start_date, creation_date"]
 
  set CurEvents 0
  while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set TruncBody [string trim [string range $body 0 [expr [string first " " [string range $body $MaxLength end]] + [expr $MaxLength - 1]]]]
    set event [list $calendar_id $category $title $TruncBody $html_p $start_date $end_date]
    lappend calendar $event
    incr CurEvents
    if { $CurEvents >= $MaxEvents } {
      ns_db  flush $db
      break
    }
  }
  return $calendar
}

proc most_recent_classifieds {db} {
  set MemoizeTime [ad_parameter ClassifiedsCacheLength 7200]
  return [util_memoize "most_recent_classifieds_internal $db" $MemoizeTime]
}

proc most_active_classified_auctions {db} {
  set MemoizeTime [ad_parameter AuctionsCacheLength 7200]
  return [util_memoize "most_active_classified_auctions_internal $db" $MemoizeTime]
}


proc most_recent_classifieds_internal {db} {
  #returns a list of lists. the child lists will contain classified_ad_id, domain, one_line, posted, and
  #manufacturer(which seems to store price) - in that order
  set classifieds [list]
  set MaxAds [ad_parameter MaxAds chunks 5]
  set selection [ns_db select $db "SELECT classified_ad_id, one_line, manufacturer, posted, domain from classified_ads, ad_domains where classified_ads.domain_id = ad_domains.domain_id AND (sysdate() <= expires or expires is null) order by posted desc"]

  set CurAds 0
  while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set ad [list $classified_ad_id $domain $one_line $posted $manufacturer]
    lappend classifieds $ad
    incr CurAds
    if { $CurAds >= $MaxAds } {
      ns_db flush $db
      break
    }
  }
  return $classifieds
}

proc most_active_classified_auctions_internal {db} {
  #returns a list of lists. the child lists will contain classified_ad_id, domain, one_line, posted, 
  #manufacturer(which seems to store price), and number of recent bids - in that order
  set db2 [ns_db gethandle subquery]
  set auctions [list]
  set MaxAuctions [ad_parameter MaxAuctions chunks 5]
  set BidAge [ad_parameter CountOldBids chunks 10]
  set selection [ns_db select $db "SELECT ca.classified_ad_id, count(*) as count FROM classified_ads ca, classified_auction_bids cab WHERE ca.classified_ad_id = cab.classified_ad_id AND bid_time >= ([db_sysdate] - $BidAge)::datetime GROUP BY ca.classified_ad_id ORDER BY count DESC"]

  set CurAuctions 0
  while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set sub_selection [ns_db 0or1row $db2 "SELECT domain, one_line, posted, manufacturer FROM classified_ads ca, ad_domains ad WHERE ca.domain_id = ad.domain_id AND classified_ad_id = $classified_ad_id"]
    set_variables_after_subquery
    set ad [list $classified_ad_id $domain $one_line $posted $manufacturer $count]
    lappend auctions $ad
    incr CurAuctions
    if { $CurAuctions >= $MaxAuctions } {
      ns_db flush $db
      break
    }
  }
  ns_db releasehandle $db2
  return $auctions
} 

util_report_successful_library_load
