
#
# This is a cybercash stub that simulates the cybercash system for testing purposes
#
# written by Ben Adida (ben@adida.net)

# Schedule enabling of the stub at a later point
# so that ad_parameter can load first.
# ns_schedule_proc -once 1 ccstub_enable_stub

# This enables the stub by registering the proc
# cc_send_to_server_21 if the parameter in the .ini
# file says to do so

if {[ad_parameter EnabledP ccstub]} {
    ns_log Notice "Cybercash stub enabled"
    proc cc_send_to_server_21 {command in_data out_data} {
	stub_cc_send_to_server_21 $command $in_data $out_data
    }
}


# The actual cybercash stub
# This is very simplistic for now and doesn't ever simulate
# cybercash failures. Eventually, it should.
proc stub_cc_send_to_server_21 {command in_data out_data} {
    set db [ns_db gethandle subquery]

    set_variables_after_query_not_selection $in_data
    set order_id [ns_set get $in_data order-id]
    set card_number [ns_set get $in_data card-number]
    set card_exp [ns_set get $in_data card-exp]

    switch $command {
	mauthonly {
	    regexp {([a-zA-Z]+) ([0-9]+)} $amount all currency amount_num

	    ns_db dml $db "insert into cybercash_stub_transactions
	    (order_id, card_number, card_exp, currency, amount_auth)
	    values
	    ('$order_id', '$card_number', '$card_exp', '$currency', $amount_num)"

	    # Return full success with AVS
	    ns_set put $out_data MStatus success
	    ns_set put $out_data "avs-code" A
	    ns_set put $out_data "amount" $amount_num
	}
	
	postauth {
	    regexp {([a-zA-Z]+) ([0-9]+)} $amount all currency amount_num

	    # A restrictive update statement to ensure that we're marking
	    # an order that exists and hasn't been marked yet.
	    # We also check the currency to make sure it still matches,
	    # and that we're marking only as much as we reserved.
	    ns_db dml $db "update cybercash_stub_transactions
	    set postauth_p='t', amount_mark=$amount_num where order_id='$order_id' and postauth_p='f' 
	    and currency='$currency' and amount_auth <= $amount_num"

	    # Success if there was one row updated (or more than one,
	    # but that won't happen because of the primary key restriction on order_id
	    set success [ns_pg ntuples $db]
	   
	    # Send the success back
	    if {$success} {
		ns_set put $out_data MStatus success
	    } else {
		ns_set put $out_data MStatus failure
		ns_set put $out_data MErrMsg "No row updated. This means you're doing something wrong."
	    }
	}

	return {
	    regexp {([a-zA-Z]+) ([0-9]+)} $amount all currency amount_num
	    
	    # A restrictive update statement to ensure that we're returning
	    # an order that exists and has been marked.
	    # We also can't refund more than was marked.
	    ns_db dml $db "update cybercash_stub_transactions
	    set return_p='t', amount_refund= $amount_num where postauth_p='t' and return_p='f'
	    and currency='$currency' and amount_mark <= $amount_num"

	    # Success if there was one row updated (or more than one,
	    # but that won't happen because of the primary key restriction on order_id
	    set success [ns_pg ntuples $db]
	   
	    # Send the success back
	    if {$success} {
		ns_set put $out_data MStatus success
	    } else {
		ns_set put $out_data MStatus failure
		ns_set put $out_data MErrMsg "No row updated. This means you're doing something wrong."
	    }
	}

	void {
	    # Not sure what to do here
	}

	retry {
	    # This should never happen with this stub
	    # as long as the query option works well
	}

	query {
	    set check 0

	    switch ${txn-type} {
		auth {
		    set check [database_to_tcl_string $db "select count(*) from cybercash_stub_transactions where order_id='$order_id'"]
		}
		
		marked {
		    set check [database_to_tcl_string $db "select count(*) from cybercash_stub_transactions where order_id='$order_id' and postauth_p='t'"]
		}

		settled {
		    set check [database_to_tcl_string $db "select count(*) from cybercash_stub_transactions where order_id='$order_id' and postauth_p='t'"]
		}
	    }

	    if {$check} {
		ns_set put $out_data MStatus success
	    } else {
		ns_set put $out_data MStatus failure
	    }
	}
    }

    ns_db releasehandle $db
}