#
# Extra utilities by Ben Adida
# 

# a variable is in the form:
# {varname "Pretty Var Name" var_type optional_stuff1 optional_stuff2}

proc html_var_varname {var} {
    return [lindex $var 0]
}

proc html_var_prettyname {var} {
    return [lindex $var 1]
}

proc html_var_type {var} {
    return [lindex $var 2]
}

proc html_var_optional {var n} {
    return [lindex $var [expr 3 + $n]]
}

proc make_html_form {method action list_of_vars} {
    set return_string "<FORM METHOD=$method ACTION=$action>
    <table noborder>
    "
    
    foreach var $list_of_vars {
	set this_var_string "<tr>
	<td valign=top>[html_var_prettyname $var]</td>"
	
	set varname [html_var_varname $var]

	switch [html_var_type $var] {
	    {text} {
		append this_var_string "<td><INPUT TYPE=text name=$varname></td>\n"
	    }

	    {textarea} {
		append this_var_string "<td><TEXTAREA name=$varname COLS=[html_var_optional $var 0] ROWS=[html_var_optional $var 1] WRAP=[html_var_optional $var 2]></TEXTAREA></td>"
	    }

	    {date} {
		append this_var_string "<td>[ns_entrywidgetbytype $varname date]</td>\n"
	    }

	    {hidden} {
		set this_var_string "<INPUT TYPE=hidden name=$varname value=\"[html_var_optional $var 0]\">"
	    }
	    
	    {select} {
		append this_var_string "<td><SELECT name=$varname>"
		foreach option [html_var_optional $var 0] {
		    set val [lindex $option 0]

		    # if selected
		    if {$val == [html_var_optional $var 1]} {
			set extra_html "SELECTED"
		    } else {
			set extra_html ""
		    }

		    if {[llength $option] == 2} {
			set prettyval [lindex $option 1]
		    } else {
			set prettyval $val
		    }
			
		    append this_var_string "<OPTION $extra_html VALUE=\"$val\"> $prettyval\n"
		}
		append this_var_string "</SELECT></td>"
	    }
	    
	}
	
	append this_var_string "</tr>\n"

	append return_string $this_var_string

    }	

    append return_string "</table><INPUT TYPE=submit value=go></FORM>\n"
    
    return $return_string
    
}

proc make_html_select {varname list_of_vals {selected ""}} {
    set return_html "<SELECT NAME=$varname>\n"

    foreach val $list_of_vals {
	if {[lindex $val 0] == $selected} {
	    set selected_html "SELECTED"
	} else {
	    set selected_html ""
	}

	if {[llength $val] == 1} {
	    append return_html "<OPTION $selected_html VALUE=\"$val\"> $val\n"
	} else {
	    append return_html "<OPTION $selected_html VALUE=\"[lindex $val 0]\"> [lindex $val 1]\n"
	}
    }

    append return_html "</SELECT>\n"

    return $return_html
}

proc make_html_checkboxes {varname list_of_vals {selected ""} {separation "<br>"}} {
    set return_html ""

    foreach val $list_of_vals {
	if {[lsearch $selected [lindex $val 0]] != -1} {
	    set selected_html "CHECKED"
	} else {
	    set selected_html ""
	}

	if {[llength $val] == 1} {
	    append return_html "<INPUT TYPE=CHECKBOX $selected_html NAME=$varname VALUE=\"$val\"> $val\n"
	} else {
	    append return_html "<INPUT TYPE=CHECKBOX  $selected_html NAME=$varname VALUE=\"[lindex $val 0]\"> [lindex $val 1]\n"
	}

	append return_html "$separation\n"
    }

    return $return_html
}
