
# A system for generalizing Notifications
# Ben Adida (ben@mit.edu)

proc notification_system_name {} {
    return "Notification"
}

proc notification_url_stub {} {
    return "/notification"
}

proc notification_sender_email {} {
    return "notification@adida.net"
}

proc notification_header {title} {
    return [ad_header $title]
}

proc notification_footer {} {
    return [ad_footer]
}

proc notification_list_of_prefs {} {
    return [list none instant hourly daily weekly]
}

proc notification_user_is_interested_p {user_id class_id item_id {db {}}} {
    check_db_var

    set check [database_to_tcl_string $db "select count(*) from user_notification_interest_map where notification_class_id=$class_id and user_id=$user_id and item_id=$item_id"]

    cleanup_db_var

    return $check
}

proc notification_set_user_interest {user_id class_id item_id {db {}}} {
    check_db_var

    # This user might already have expressed interest
    catch {ns_db dml $db "insert into user_notification_interest_map (notification_class_id, user_id, item_id) values ($class_id, $user_id, $item_id)"} errmsg

    cleanup_db_var
}

proc notification_remove_user_interest {user_id class_id item_id {db {}}} {
    check_db_var

    ns_db dml $db "delete from user_notification_interest_map where notification_class_id=$class_id and user_id=$user_id and item_id=$item_id"

    cleanup_db_var
}

proc notification_notify_users {class_id item_id message {db {}}} {
    check_db_var

    # Call PL/SQL
    database_to_tcl_string $db "select user_add_notification($class_id, $item_id, '[DoubleApos $message]')"

    cleanup_db_var
}


proc notification_send_one_email {to from subject body} {
    ns_sendmail $to $from $subject "$body

----------------------------------------------------------------------------------
This email is being sent to you by [notification_system_name] 
at [ad_url], because you
requested updates on bugs and features.

If you would like to change your notification status, please visit:
[ad_url][notification_url_stub]/prefs.tcl
"
}
   
## Scheduled Procs for notification
proc user_send_notifications {notification_interval} {
    set db [ns_db gethandle subquery]

    ns_db dml $db "begin transaction"

    # Lock the table
    ns_db dml $db "lock table user_notifications"

    # Select all the notifications for this particular interval
    set data_to_send [database_to_tcl_list_list $db "
    select users.user_id, email, notification_content, notification_class
    from user_notifications, user_notification_prefs, users, user_notification_classes where
    user_notification_classes.notification_class_id= user_notification_prefs.notification_class_id and
    user_notification_prefs.notification_class_id= user_notifications.notification_class_id and
    users.user_id= user_notifications.user_id and 
    user_notifications.user_id= user_notification_prefs.user_id
    and (notification_content is not null and notification_content != '')
    and notification_pref= '$notification_interval'"]

    # Close the transaction
    ns_db dml $db "end transaction"

    ns_db releasehandle $db

    # Send out the emails!
    foreach email $data_to_send {
	notification_send_one_email [lindex $email 1] [notification_sender_email] "[lindex $email 3] Update" [lindex $email 2]
    }

    set db [ns_db gethandle subquery]

    # Clear things out
    ns_db dml $db "
    update user_notifications set notification_content=''
    where user_id in 
    (select user_id from user_notification_prefs
    where notification_pref='$notification_interval')"

    ns_db releasehandle $db
}

proc user_remove_notifications {interval} {
    set db [ns_db gethandle subquery]
    
    ns_db dml $db "update user_notifications set notification_content='' where user_id in (select user_id from user_notification_prefs where notification_pref='$interval')"

    ns_db releasehandle $db
}

ns_share -init {set user_notification_scheduled_p 0} user_notification_scheduled_p

if {!$user_notification_scheduled_p} {
    set user_notification_scheduled_p 1

    for {set i 0} {$i<24} {incr i} {
	ns_schedule_daily -thread $i 0 user_send_notifications hourly
    }

    ns_schedule_daily -thread 23 15 user_send_notifications daily

    ns_schedule_weekly -thread 0 23 30 user_send_notifications weekly

    ns_schedule_daily -thread 0 30 user_remove_notifications none
}
