
proc sdm_url_stub {} {
    return [ad_parameter UrlStub sdm]
}

proc sdm_software_root {} {
    return [ad_parameter SoftwareRoot sdm]
}

proc sdm_version {} {
    return [ad_parameter SdmVersion sdm]
}

proc sdm_notification_sender_email {} {
    return [ad_parameter NotificationSenderEmail sdm]
}

# FOR NAVIGATION

proc sdm_home_context_bar_item {} {
    return [list [sdm_url_stub] "SDM"]
}

proc sdm_module_context_bar_item {module_id module_name} {
    return [list [sdm_url_stub]/one-module.tcl?[export_url_vars module_id] $module_name]
}

proc sdm_package_context_bar_item {package_id package_name} {
    return [list [sdm_url_stub]/one-package.tcl?[export_url_vars package_id] $package_name]
}

proc sdm_release_context_bar_item {release_id release_name} {
    return [list [sdm_url_stub]/one-release.tcl?[export_url_vars release_id] $release_name]
}

proc sdm_package_release_context_bar_item {release_id release_name} {
    return [list [sdm_url_stub]/one-package-release.tcl?[export_url_vars release_id] $release_name]
}

# other stuff

proc sdm_header {title} {
    set user_id [ad_verify_and_get_user_id]

    set return_url [ns_conn url]?[ns_conn query]
    if {$user_id == 0} {
	set extra_stuff "
	<table cellpadding=4 align=right>
	<tr>
	<td bgcolor=lightgrey>SDM v[sdm_version]<br><a href=/register/index.tcl?[export_url_vars return_url]>log in</a></td>
	</tr>
	</table>"
    } else {
	set db [ns_db gethandle subquery]
	set email [database_to_tcl_string $db "select email from users where user_id=$user_id"]
	ns_db releasehandle $db
	set extra_stuff "
	<table cellpadding=4 align=right>
	<tr>
	<td align=center bgcolor=lightgrey>SDM v[sdm_version]<br>$email<br>
	<font size=-1><a href=[sdm_url_stub]/pvt/prefs.tcl>Prefs</a> | <a href=/register/logout.tcl?[export_url_vars return_url]>log out</a>
	</td>
	</tr>
	</table>"
    }
    return "[ad_header $title]$extra_stuff"
}

proc sdm_footer {} {
    return "[ad_footer]"
}

proc sdm_system_name {} {
    return [ad_parameter SystemName sdm]
}

proc sdm_default_severity {} {
    return [ad_parameter DefaultSeverity sdm]
}

proc sdm_list_of_baf_types {} {
    return [list bug feature]
}

proc sdm_list_of_severities {} {
    return [ad_parameter ListOfSeverities sdm]
}

proc sdm_list_of_notification_prefs {} {
    return [list none hourly daily weekly]
}

proc sdm_closed_baf_status_id {} {
    set db [ns_db gethandle subquery]
    set return_val [database_to_tcl_string $db "select baf_status_id from baf_status where baf_status='closed'"]
    ns_db releasehandle $db
    return $return_val
}

proc sdm_reopened_baf_status_id {} {
    set db [ns_db gethandle subquery]
    set return_val [database_to_tcl_string $db "select baf_status_id from baf_status where baf_status='reopened'"]
    ns_db releasehandle $db
    return $return_val
}

ns_share -init {set ad_sdm_security_filters_installed 0} ad_sdm_security_filters_installed

if {!$ad_sdm_security_filters_installed} {
    set ad_sdm_security_filters_installed 1
    # Bounce people out of the /sdm/pvt directory if they're not logged in
    ns_register_filter preauth GET /sdm/pvt/* ad_verify_identity
    ns_register_filter preauth POST /sdm/pvt/* ad_verify_identity
}

# The following procedures just call their PL/SQL equivalents.
# An extra layer of abstraction is added:
# - in case we stop doing this in PL/SQL
# - so that the programmer is discouraged from performing deadlock-risky
# queries cause Oracle 8.0.5 sucks so bad.

proc sdm_check_admin {} {
    set user_id [ad_verify_and_get_user_id]
    
    ## HACK TO LET BEN SEE EVERYTHING FOR NOW
    if {$user_id == 4} {
	return 1
    } else {
	return 0
    }
}

proc sdm_user_can_add_packages {db user_id} {
    return [ad_administrator_p $db $user_id]
}

proc user_can_see_package_p {db user_id package_id} {
    if {[sdm_check_admin]} {
	return 1
    }

    if {[database_to_tcl_string $db "select user_can_see_package_p($user_id, $package_id) from dual"]== "t"} {
	return 1
    } else {
	return 0
    }
}

proc user_can_see_module_p {db user_id module_id} {
    if {[sdm_check_admin]} {
	return 1
    }

    if {[database_to_tcl_string $db "select user_can_see_module_p($user_id, $module_id) from dual"]== "t"} {
	return 1
    } else {
	return 0
    }
}

proc user_can_edit_package_p {db user_id package_id} {
    if {[sdm_check_admin]} {
	return 1
    }

    if {[database_to_tcl_string $db "select user_can_edit_package_p($user_id, $package_id) from dual"]== "t"} {
	return 1
    } else {
	return 0
    }
}

proc user_can_edit_module_p {db user_id module_id} {
    if {[sdm_check_admin]} {
	return 1
    }

    if {[database_to_tcl_string $db "select user_can_edit_module_p($user_id, $module_id) from dual"]== "t"} {
	return 1
    } else {
	return 0
    }
}

proc user_can_edit_baf_p {db user_id baf_id} {
    if {[database_to_tcl_string $db "select user_can_edit_baf_p($user_id, $baf_id) from dual"] == "t"} {
	return 1
    } else {
	return 0
    }
}

# Return an error if access problem
proc sdm_return_access_complaint {} {
    ns_return 500 text/html "[sdm_header "Access Violation"]
    <h2>Access Violation</h2>
    <hr><p>
    You are not allowed to view this information, either because you are not logged on, or if you are, because this information is private and you are not authorized.
    <p>
    [sdm_footer]
"
}

# This expects a SQL query which will select user_id, first_names, last_name,
# and email
proc make_user_select_html {db sql_query {name user_id}} {
    set selection [ns_db select $db $sql_query]

    set return_html "<SELECT NAME=$name>\n"

    while {[ns_db getrow $db $selection]} {
	set_variables_after_query

	append return_html "<OPTION VALUE=$user_id> $last_name, $first_names ($email)\n"
    }

    append return_html "</SELECT>"

    return $return_html
}

#
# Simple user information management
##
proc set_simple_user_information {db} {
    uplevel {
	set user_id [ad_verify_and_get_user_id]

	set user_logged_on_p 0

	if {$user_id > 0} {
	    set selection [ns_db 1row $db "select first_names, last_name, email from users where user_id=$user_id"]
	    set_variables_after_query
	    set user_logged_on_p 1
	}
    }
}

#
# Simple user information management
##
proc set_simple_module_information {db module_id} {
    uplevel {
	set selection [ns_db 0or1row $db "select module_name, packages.package_id as package_id, package_name, packages.private_p as package_private_p, modules.private_p as module_private_p from modules,packages where modules.module_id=$module_id and modules.package_id=packages.package_id"]
	
	if {$selection != ""} {
	    set_variables_after_query
	}
    }
}

#
# Simple user information management
##
proc set_simple_package_information {db package_id} {
    uplevel {
	set selection [ns_db 1row $db "select package_name, private_p as package_private_p from packages where package_id=$package_id"]
	set_variables_after_query
    }
}


##
## Location of software
proc sdm_software_check_dir {package_id release_id {module_id ""}} {
    # This will create the right directory
    if {[catch {ns_mkdir "[sdm_software_root]/package_$package_id"} errmsg]} {}
    if {[catch {ns_mkdir "[sdm_software_root]/package_$package_id/release_$release_id"} errmsg]} {}

    if {$module_id != ""} {
	if {[catch {ns_mkdir "[sdm_software_root]/package_$package_id/release_$release_id/module_$module_id"} errmsg]} {}
    }
}

proc sdm_software_dir {package_id release_id {module_id ""}} {
    if {$module_id != ""} {
	return "[sdm_software_root]/package_$package_id/release_$release_id/module_$module_id"
    } else {
	return "[sdm_software_root]/package_$package_id/release_$release_id"
    }
}

proc sdm_release_filename {release_id original_filename} {
    return "$original_filename"
}

proc sdm_cleanup_filename {filename} {
    set last_forward_slash [string last "/" $filename]
    set last_back_slash [string last "\\" $filename]

    if {$last_forward_slash > $last_back_slash} {
	set last_slash $last_forward_slash
    } else {
	set last_slash $last_back_slash
    }

    return [string range $filename [expr $last_slash + 1] end]
}

## We look for download requests of the following type:
## /sdm/download/3/2/12/filename
## where 3 is the package id, 2 is the module id, 12 is the release id,
## and filename is the original uploaded filename.

ns_register_proc GET /sdm/download/* sdm_download_release
ns_register_proc GET /sdm/download-package/* sdm_download_package_release

proc sdm_code_license {} {
    set file [open "/web/aol-dev/www/license.html" r]
    set contents [read $file]
    close $file

    return $contents
}

proc sdm_download_package_release {ignore} {
    set url_stub [ns_conn url]

    if {![regexp {/sdm/download-package/([^/]*)/([^/]*)/([^/]*)} $url_stub all package_id release_id filename]} {
	ns_return 500 text/html "Bad URL"
	return
    }

    set db [ns_db gethandle]

    set_simple_user_information $db

    if {![user_can_see_package_p $db $user_id $package_id]} {
	sdm_return_access_complaint
	return
    }

    set_simple_package_information $db $package_id

    set filename_stub [database_to_tcl_string $db "select release_filename from package_releases where package_id=$package_id and release_id=$release_id"]

    ns_db releasehandle $db

    ns_returnfile 200 [ns_guesstype $filename_stub] [sdm_software_dir $package_id $release_id]/$filename_stub

    set ip_address [ns_conn peeraddr]

    set db [ns_db gethandle]
    
    ns_db dml $db "insert into package_release_downloads (package_id, release_id, download_date, ip_address) VALUES ($package_id, $release_id, sysdate(), '$ip_address')"

    ns_db releasehandle $db

    ns_returnredirect "/sdm/download-thank-you.tcl?[export_url_vars package_id release_id]"
}


##
## The CVS abstraction
## 

proc sdm_load_package {package_id dir {release_id ""}} {
    set db [ns_db gethandle subquery]
    set selection [ns_db 1row $db "select package_name, cvs_path, cvs_server, cvs_username, cvs_password from packages_with_cvs_data where package_id=$package_id"]
    set_variables_after_query

    if {$release_id != ""} {
	set selection [ns_db 1row $db "select release_name(major_version,minor_version,patch_version,beta_version) as release_name from package_releases where package_id=$package_id and release_id=$release_id"]
	set_variables_after_query

	regsub -all "\." $release_name "-" release_name
	set release_name "v$release_name"
    } else {
	set release_name ""
    }

    cd $dir
    cvs_checkout $package_name $cvs_path $release_name $cvs_server $cvs_username $cvs_password

    return $package_name
}

proc sdm_load_source_package {package_id {release_id ""}} {
    set dir_name [sdm_load_package $package_id [sdm_source_root] $release_id]
    return $dir_name
}

proc sdm_load_download_package {package_id release_id} {
    sdm_software_check_dir $package_id $release_id
    set absolute_dir [sdm_software_dir $package_id $release_id]
    set code_dir [sdm_load_package $package_id $absolute_dir $release_id]

    set relative_filename [tar_and_gz $absolute_dir $code_dir]
    set release_filename [sdm_release_filename $release_id $relative_filename]
    
    exec mv $absolute_dir/$relative_filename $absolute_dir/$release_filename

    return $release_filename
}

## Tar and GZ
## This returns the name of the one file that is the 
## tar'ed and gz'ed file.
proc tar_and_gz {absolute_dir code_dir} {
    cd $absolute_dir

    if {[catch {exec /usr/local/bin/tar cf $absolute_dir/${code_dir}.tar $code_dir} errmsg]} {
    }

    if {[catch {exec /usr/local/bin/gzip $absolute_dir/${code_dir}.tar} errmsg]} {
    }

    return ${code_dir}.tar.gz
}


##
## Bug Rating
##

proc sdm_user_has_rated_baf_p {user_id baf_id {db {}}} {
    if {$user_id == ""} {
	return 0
    }

    check_db_var

    set check [database_to_tcl_string $db "select count(*) from baf_ratings where user_id=$user_id and baf_id=$baf_id"]

    cleanup_db_var

    if {$check > 0} {
	return 1
    } else {
	return 0
    }
}

proc sdm_rate_baf {user_id baf_id rating {db {}}} {
    check_db_var

    if {[catch {
	ns_db dml $db "insert into baf_ratings (user_id, baf_id, rating, rating_date)
	values ($user_id, $baf_id, $rating, sysdate())"

	cleanup_db_var
    } errmsg]} {
	return 0
    } else {
	return 1
    }
}

proc sdm_baf_rating {baf_id {db {}}} {
    check_db_var

    set rating [database_to_tcl_string $db "select avg(rating) from baf_ratings where baf_id=$baf_id"]
    
    cleanup_db_var

    return $rating
}

## Interest Mapping

proc sdm_baf_n_interested_users {baf_id {db {}}} {
    check_db_var

    set n_interested [database_to_tcl_string $db "select count(*) from user_baf_interest_map where baf_id=$baf_id"]

    cleanup_db_var

    return $n_interested
}

proc sdm_baf_user_is_interested_p {baf_id user_id {db {}}} {
    check_db_var

    set check [database_to_tcl_string $db "select count(*) from user_baf_interest_map where baf_id= $baf_id and user_id=$user_id"]

    cleanup_db_var
    
    return $check
}

proc sdm_baf_user_is_interested {baf_id user_id {db {}}} {
    check_db_var

    ns_db dml $db "insert into user_baf_interest_map (user_id, baf_id) values ($user_id, $baf_id)"

    cleanup_db_var
}


proc sdm_baf_user_is_not_interested {baf_id user_id {db {}}} {
    check_db_var

    ns_db dml $db "delete from user_baf_interest_map where user_id=$user_id and baf_id=$baf_id"

    cleanup_db_var
}

## Package interests

proc sdm_package_n_interested_users {package_id {db {}}} {
    check_db_var

    set n_interested [database_to_tcl_string $db "select count(*) from user_package_interest_map where package_id=$package_id"]

    cleanup_db_var

    return $n_interested
}

proc sdm_package_user_is_interested_p {package_id user_id {db {}}} {
    check_db_var

    set check [database_to_tcl_string $db "select count(*) from user_package_interest_map where package_id= $package_id and user_id=$user_id"]

    cleanup_db_var
    
    return $check
}

proc sdm_package_user_is_interested {package_id user_id {db {}}} {
    check_db_var

    ns_db dml $db "insert into user_package_interest_map (user_id, package_id) values ($user_id, $package_id)"

    cleanup_db_var
}


proc sdm_package_user_is_not_interested {package_id user_id {db {}}} {
    check_db_var

    ns_db dml $db "delete from user_package_interest_map where user_id=$user_id and package_id=$package_id"

    cleanup_db_var
}


## Notification Code

proc sdm_notify_users_baf {baf_id notification {db {}}} {
    check_db_var

    # Call the Pl/SQL
    database_to_tcl_string $db "select baf_edit_notification($baf_id, '[DoubleApos $notification]')"
    
    cleanup_db_var
}
    
proc sdm_notify_users_package {package_id notification {db {}}} {
    check_db_var

    # Call the PL/SQL
    database_to_tcl_string $db "select package_edit_notification($package_id, '[DoubleApos $notification]')"

    cleanup_db_var
}
 
proc sdm_send_one_email {to from subject body} {
    ns_sendmail $to $from $subject "$body

----------------------------------------------------------------------------------
This email is being sent to you by [sdm_system_name] 
at [ad_url], because you
requested updates on bugs and features.

If you would like to change your notification status, please visit:
[ad_url][sdm_url_stub]/pvt/prefs.tcl
"
}
   
## Scheduled Procs for notification
proc sdm_send_notifications {notification_interval} {
    set db [ns_db gethandle subquery]

    ns_db dml $db "begin transaction"

    # Lock the table
    ns_db dml $db "lock table sdm_notifications"

    # Select the BAF notifications
    set data_to_send [database_to_tcl_list_list $db "
    select users.user_id, email, baf_notifications
    from sdm_notifications, sdm_notification_prefs, users where 
    users.user_id= sdm_notifications.user_id and
    sdm_notifications.user_id= sdm_notification_prefs.user_id 
    and (baf_notifications is not null and baf_notifications != '') 
    and baf_pref='$notification_interval'"]
    
    # Clear things out
    ns_db dml $db "
    update sdm_notifications set baf_notifications=''
    where user_id in 
    (select user_id from sdm_notification_prefs
    where baf_pref='$notification_interval')"

    # Close the transaction
    ns_db dml $db "end transaction"

    ns_db releasehandle $db

    # Send out the emails!
    foreach email $data_to_send {
	sdm_send_one_email [lindex $email 1] [sdm_notification_sender_email] "Bug/Feature Update" [lindex $email 2]
    }

    # now the package
    set db [ns_db gethandle subquery]

    ns_db dml $db "begin transaction"

    # Lock the table
    ns_db dml $db "lock table sdm_notifications"

    # Select the PACKAGE notifications
    set data_to_send [database_to_tcl_list_list $db "
    select users.user_id, email, package_notifications
    from sdm_notifications, sdm_notification_prefs, users where 
    users.user_id= sdm_notifications.user_id and
    sdm_notifications.user_id= sdm_notification_prefs.user_id 
    and (package_notifications is not null and package_notifications != '') 
    and package_pref='$notification_interval'"]
    
    # Clear things out
    ns_db dml $db "
    update sdm_notifications set package_notifications=''
    where user_id in 
    (select user_id from sdm_notification_prefs
    where package_pref='$notification_interval')"

    # Close the transaction
    ns_db dml $db "end transaction"

    ns_db releasehandle $db

    # Send out the emails!
    foreach email $data_to_send {
	sdm_send_one_email [lindex $email 1] [sdm_notification_sender_email] "Package Update" [lindex $email 2]
    }
}

proc sdm_remove_notifications {interval} {
    set db [ns_db gethandle subquery]
    
    ns_db dml $db "update sdm_notifications set baf_notifications='' where user_id in (select user_id from sdm_notification_prefs where baf_pref='$interval')"

    ns_db dml $db "update sdm_notifications set package_notifications='' where user_id in (select user_id from sdm_notification_prefs where package_pref='$interval')"

    ns_db releasehandle $db
}

ns_share -init {set sdm_notification_scheduled_p 0} sdm_notification_scheduled_p

if {!$sdm_notification_scheduled_p} {
    set sdm_notification_scheduled_p 1

    for {set i 0} {$i<24} {incr i} {
	ns_schedule_daily -thread $i 0 sdm_send_notifications hourly
    }

    ns_schedule_daily -thread 23 15 sdm_send_notifications daily

    ns_schedule_weekly -thread 0 23 30 sdm_send_notifications weekly

    ns_schedule_daily -thread 0 30 sdm_remove_notifications none
}

proc sdm_util_strip_leading_slash {stuff} {
    if {[string range $stuff 0 0] == "/"} {
	return [string range $stuff 1 end]
    } else {
	return $stuff
    }
}

# Managing Patches
# for source code
proc sdm_patch_file_path {} {
    return "[ad_parameter PathToACS]/data/sdm-patches"
}

proc sdm_one_package_patch_file_path {package_id} {
    set path [sdm_patch_file_path]/$package_id

    # make sure the directory exists
    if {![file exists $path]} {
	ns_mkdir $path
    }

    return $path
}

proc sdm_one_patch_file_full_path {package_id patch_id filename} {
    return [sdm_one_package_patch_file_path $package_id]/${patch_id}-${filename}
}