##
## This implements the SDM to todo link
##
## ben@adida.net

ns_schedule_proc -once 1 sdm_todoplugin_register

# The proc to register the SDM todo plugin
proc sdm_todoplugin_register {} {
    set data [ns_set new -persist sdm_todoplugin_data]
    ns_set put $data GET_PERSONAL_CATEGORIES sdm_todoplugin_get_personal_categories
    ns_set put $data GET_OTHER_CATEGORIES sdm_todoplugin_get_other_categories
    ns_set put $data ITEM_CHANGE_ALLOWED_P 0
    ns_set put $data MARK_COMPLETED_ALLOWED_P 1
    ns_set put $data MARK_COMPLETED sdm_todoplugin_mark_completed
    ns_set put $data ONE_ITEM_URL [sdm_url_stub]/one-baf.tcl
    ns_set put $data ONE_ITEM_ID_KEY baf_id
    ns_set put $data ITEM_LIST_SQL sdm_todoplugin_item_list_sql

    # actually register
    todo_plugin_register SDM $data
}

# Getting the personal categories
proc sdm_todoplugin_get_personal_categories {db user_id} {
    set categories [database_to_tcl_list_list $db "
    select package_id, package_name from packages where package_id in
    (select package_id from bugs_and_features where completion is null and baf_id in
    (select baf_id from baf_assignments where user_id=$user_id))"]

    return $categories
}

# Getting other categories
proc sdm_todoplugin_get_other_categories {db user_id} {
    return [list]
}

# When an item is done
# This just redirects to the right location
# and returns a flag saying that no more information should be returned
proc sdm_todoplugin_mark_completed {db user_id todo_id {completion_date {}}} {
    ns_returnredirect [sdm_url_stub]/pvt/baf-close.tcl?baf_id=$todo_id

    # A boolean of true means everything has been done and the
    # calling page should do nothing more.
    return 1
}


# This should return the SQL
# to select the stuff out of the database
proc sdm_todoplugin_item_list_sql {db user_id category_id date_predicates date_prettifier n_days_to_show_completed_items completion_status order_by} {
    
    set due_date "sdm_baf_due_date(bafs.baf_id)"

    set list_of_predicates [list]

    if {$date_predicates != ""} {
	lappend list_of_predicates [subst $date_predicates]
    }

    if {$category_id != ""} {
	lappend list_of_predicates "packages.package_id= $category_id"
    }

    if {$completion_status == "open"} {
	lappend list_of_predicates "completion is null"
    } 

    if {$completion_status == "closed"} {
	lappend list_of_predicates "completion is not null"
    }

    lappend list_of_predicates "packages.package_id= bafs.package_id"
    lappend list_of_predicates "baf_assignments.baf_id= bafs.baf_id"
    lappend list_of_predicates "baf_assignments.user_id= $user_id"

    set sql "
    select 
    'SDM' as plugin, packages.package_id as category_id, packages.package_name as category,
    round(6-baf_rating(bafs.baf_id)/2) as priority, ${date_prettifier}($due_date) as due_date,
    todo_interval_min($due_date) as raw_due_date,
    ''::varchar as assigned_by,
    bafs.baf_id as todo_id, baf_type::text || ': '::text || bafs.description::text as item_details, sdm_completion_date(bafs.baf_id) as completion_date
    from
    bugs_and_features bafs, packages, baf_assignments
    where 
    [join $list_of_predicates " and "]"

    return $sql
}
    