# sql-book-defs.tcl,v 1.5.4.1 2000/02/03 09:17:47 ron Exp
# definitions for philg's SQL book (http://photo.net/sql/ ) 

# we don't want to link directly to Oracle docs because 

# 1) these will change over time as new versions of Oracle are released
# 2) we might or might not get permissions to keep Oracle docs on our server
# 3) Oracle might put its docs on its own Web server and keep them stable

# usage:  we assign a token to every concept, e.g., "create_table" and then 
# link to /sql/ref/create_table which will redirect to the appropriate spot

util_report_library_entry

ns_register_proc GET /sql/ref/ sql_redirect 

proc sql_redirect {ignore} {
    if { ![regexp {/sql/ref/(.+)$} [ns_conn url] match token] } {
	# we couldn't find the token
	ad_return_error "Reference Error" "A SQL reference is supposed to look like
<pre><code>
[ns_conn location]/sql/ref/***token***
</pre></code>

This request looked like
<pre><code>
[ns_conn request]
</pre></code>
[ad_footer]"
        return
    }
    # we have a token, now let's look it up
    set destination [sql_redirect_lookup_token $token]
    if [empty_string_p $destination] {
	ad_return_error "Token Lookup Failure" "We couldn't find \"$token\" in our little database of reference materials.  Sorry."
    } else {
	ns_returnredirect $destination
    }
}

proc sql_redirect_lookup_token {token} {
    set token_db(create_materialized_view) "http://oradoc.photo.net/ora81/DOC/server.815/a67779/ch4f.htm#24503"
    set token_db(concepts) "http://oradoc.photo.net/ora81/DOC/server.815/a67781/toc.htm"
    set token_db(utilities) "http://oradoc.photo.net/ora81/DOC/server.815/a67792/toc.htm"
    set token_db(net8) "http://oradoc.photo.net/ora81/DOC/network.815/a67440/toc.htm"
    set token_db(copycommand) "http://oradoc.photo.net/ora81/DOC/server.815/a66736/ch63.htm#307"
    set token_db(export) "http://oradoc.photo.net/ora81/DOC/server.815/a67792/ch01.htm#2531"
    set token_db(import) "http://oradoc.photo.net/ora81/DOC/server.815/a67792/ch02.htm#34966"
    set token_db(jobs) "http://oradoc.photo.net/ora81/DOC/server.815/a67772/jobq.htm#750"
    set token_db(oraclelanguages) "http://oradoc.photo.net/ora81/DOC/server.815/a67789/appa.htm#956235"
    set token_db(intermediatext) "http://oradoc.photo.net/ora81/DOC/inter.815/a67843/toc.htm"
    set token_db(connectby) "http://oradoc.photo.net/ora81/DOC/server.815/a67779/ch4l.htm#13496"
    set token_db(plsql) "http://oradoc.photo.net/ora81/DOC/server.815/a67842/toc.htm"
    set token_db(using_procedures_and_packages) "http://oradoc.photo.net/ora81/DOC/server.815/a68003/01_10pck.htm#562"
    set token_db(java_stored_procedures) "http://oradoc.photo.net/ora81/DOC/java.815/a64686/toc.htm"
    set token_db(addmonths) "http://oradoc.photo.net/ora81/DOC/server.815/a67779/function.htm#1025480"
    set token_db(createsequence) "http://oradoc.photo.net/ora81/DOC/server.815/a67779/ch4g.htm#3055"
    set token_db(usingsequences) "http://oradoc.photo.net/ora81/DOC/server.815/a67779/ch2.htm#27457"
    set token_db(transactions) "http://oradoc.photo.net/ora81/DOC/server.815/a67781/c15trans.htm#222"
    set token_db(dbms_lob) "http://oradoc.photo.net/ora81/DOC/server.815/a68001/dbms_lob.htm#998100"
    set token_db(concepts_date_datatype) "http://oradoc.photo.net/ora81/DOC/server.815/a67781/c10datyp.htm#796"
    set token_db(using_date_datatype) "http://oradoc.photo.net/ora81/DOC/server.815/a68003/01_04blt.htm#426706"
    set token_db(date_functions) "http://oradoc.photo.net/ora81/DOC/server.815/a67779/function.htm#1024474"
    set token_db(date_format) "http://oradoc.photo.net/ora81/DOC/server.815/a67779/ch2.htm#34926"
    set token_db(create_index) "http://oradoc.photo.net/ora81/DOC/server.815/a67779/ch4f.htm#39410"
    set token_db(composite_indices) "http://oradoc.photo.net/ora81/DOC/server.815/a67775/ch6_acce.htm#2174"
    set token_db(rollup) "http://oradoc.photo.net/ora81/DOC/server.815/a68003/rollup_c.htm#31838"
    set token_db(limits) "http://oradoc.photo.net/ora81/DOC/server.815/a67790/ch4.htm#4762"

    if [info exists token_db($token)] {
	return $token_db($token)
    } else {
	return ""
    }
}

# let's add stuff for diary here also 

ns_register_adptag "diary_header" "/diary_header" diary_header

proc diary_title {} {
    return "Diary of a Startup"
}

proc diary_header {page_title tagset} {
    return "
<html>
<head>
<title>$page_title</title>
</head>
<body bgcolor=white text=black>

<h2>$page_title</h2>

part of <a href=\"index.adp\">[diary_title]</a> 
by Philip Greenspun

<hr>
"
}

ns_register_adptag "diary_footer" "/diary_footer" diary_footer

proc diary_footer {} {
    return "
<hr>
Return to <a href=\"index.adp\">Table of Contents</a>
<br>
<br>
<address><a href=\"mailto:philg@mit.edu\">philg@mit.edu</a></address>
</body>
</html>
"
}

util_report_successful_library_load
