
##
## Team Definitions
##

proc check_db_var {} {
    uplevel {
	if {$db == ""} {
	    set db [ns_db gethandle subquery]
	    set this_is_a_subquery 1
	}
    }
}

proc cleanup_db_var {} {
    uplevel {
	if {[info exists this_is_a_subquery]} {
	    ns_db releasehandle $db
	}
    }
}

proc team_header {title} {
    return [ad_header $title]
}

proc team_footer {} {
    return [ad_footer]
}

proc team_url_stub {} {
    return "/team"
}

proc team_system_name {} {
    return "Team Management"
}

proc team_user_group_type {} {
    return "team"
}

proc team_roles {} {
    return {external internal}
}

proc team_actions {} {
    return {view_project edit_project fix_issue close_issue}
}

proc team_permissions {} {
    return {{view_project external} {view_project internal} {edit_project internal} {fix_issue internal}}
}

# Find a project's team
proc team_project_group_id {project_id {db {}}} {
    check_db_var

    return [database_to_tcl_string_or_null $db "select team_id from ticket_project_teams where project_id=$project_id"]
}

# Find a team by name
proc team_get_user_group_id {team_name {db {}}} {
    check_db_var

    set user_group_id [database_to_tcl_string_or_null $db "select user_group_id from user_groups where group_type='[team_user_group_type]' and lower(group_name)=lower('[DoubleApos $team_name]')"]

    return $user_group_id
}

# Create a new team
proc team_new {team_name {db {}}} {
    check_db_var

    set team_id [ad_user_group_add $db [team_user_group_type] [DoubleApos $team_name] "t" "f" "wait" "t"]

    # Add the roles we want
    foreach role [team_roles] {
	ad_user_group_role_add $db $team_id $role
    }

    # Add the actions
    foreach action [team_actions] {
	ad_user_group_action_add $db $team_id $action
    }

    # Add the matching
    foreach permission [team_permissions] {
	ad_user_group_action_role_map $db $team_id [lindex $permission 0] [lindex $permission 1]
    }

    return $team_id
}

# Add a user to a team
proc team_add_user {team_id user_id role {db {}}} {
    check_db_var

    # Special case if the person is already a member of the group
    if {[ad_user_group_member $db $team_id $user_id]} {
	team_user_change_role $team_id $user_id $role $db
	return
    }

    ad_user_group_user_add $db $user_id $role $team_id

    set team_name [database_to_tcl_string $db "select group_name from user_groups where group_id=$team_id"]
    set full_name [database_to_tcl_string $db "select email from users where user_id=$user_id"]

    # Add a todo list for that person
    set new_list_id [todo_list_add $user_id "$team_name tasks" $db]

    # Enable access to that list for that group
    todo_list_allow_user_group $new_list_id $team_id $db
}

proc team_user_change_role {team_id user_id new_role {db {}}} {
    check_db_var

    # Blech, hack it baby, change the role directly in the table.
    # First delete all the mappings, then readd it.
    ns_db dml $db "delete from user_group_map where group_id=$team_id and user_id=$user_id"
    ad_user_group_user_add $db $user_id $new_role $team_id
}    

proc team_remove_user {team_id user_id {db {}}} {
    check_db_var

    ns_db dml $db "begin transaction"

    # Delete the mapping
    ns_db dml $db "delete from user_group_map where group_id=$team_id and user_id=$user_id"

    set list_id [database_to_tcl_string $db "select todo_lists.list_id as list_id from todo_lists, todo_list_user_group_map where user_id=$user_id and user_group_id=$team_id and todo_lists.list_id= todo_list_user_group_map.list_id"]

    # Delete the todo items
    ns_db dml $db "delete from todo_items where list_id=$list_id"

    # Delete the mapping
    ns_db dml $db "delete from todo_list_user_group_map where list_id=$list_id"

    # Delete the todo list
    ns_db dml $db "delete from todo_lists where list_id=$list_id"

    ns_db dml $db "end transaction"
}

# Check if a user is a member of a team
proc team_user_is_member_p {team_id user_id {db {}}} {
    check_db_var

    return [ad_user_group_member $db $team_id $user_id]
}

# Check if this is a team admin
proc team_user_is_admin_p {team_id user_id {db {}}} {
    check_db_var
    
    return [ad_user_group_authorized_admin $user_id $team_id $db]
}
    
# Adding a user to a project
# returns a list of user_id, todo_list_id
# where the todo_list_id is the list_id of the shared todo_list
proc team_add_user_by_email {db team_id email role} {
    set email [string trim $email]
    set QQemail [DoubleApos $email]
    set project_name [database_to_tcl_string $db "select title from ticket_projects where project_id=(select project_id from ticket_project_teams where team_id=$team_id)"]
    
    # Check if there's a user_id
    set email_user_id [database_to_tcl_string_or_null $db "select user_id from users where upper(email)=upper('$QQemail')"]

    # Create user if necessary
    if {$email_user_id == ""} {
	set email_user_id [database_to_tcl_string $db "select user_id_sequence.nextval from dual"]
	ns_db dml $db "insert into users (user_id, first_names, last_name, email, converted_p, password, user_state) VALUES ($email_user_id, '$QQemail', '$QQemail', '$QQemail', 't', '$QQemail', 'authorized')"

	foreach table_name [ad_parameter_all_values_as_list RequiredUserTable] {
	    if {[database_to_tcl_string $db "select count(*) from $table_name where user_id=$email_user_id"] == 0} {
		ns_db dml $db "insert into $table_name (user_id) values ($email_user_id)"
	    }
	}

    } else {
	# user exists
    }

    team_add_user $team_id $email_user_id $role $db
}


