##
## This implements the Ticket to todo link
##
## ben@adida.net

ns_schedule_proc -once 1 ticket_todoplugin_register

# The proc to register the Ticket todo plugin
proc ticket_todoplugin_register {} {
    set data [ns_set new -persist ticket_todoplugin_data]
    ns_set put $data GET_PERSONAL_CATEGORIES ticket_todoplugin_get_personal_categories
    ns_set put $data GET_OTHER_CATEGORIES ticket_todoplugin_get_other_categories
    ns_set put $data ITEM_CHANGE_ALLOWED_P 0
    ns_set put $data MARK_COMPLETED_ALLOWED_P 1
    ns_set put $data MARK_COMPLETED ticket_todoplugin_mark_completed
    ns_set put $data MARK_UNCOMPLETED ticket_todoplugin_mark_uncompleted
    ns_set put $data ONE_ITEM_URL [ticket_url_stub]/issue-view.tcl
    ns_set put $data ONE_ITEM_ID_KEY msg_id
    ns_set put $data ITEM_LIST_SQL ticket_todoplugin_item_list_sql

    # actually register
    todo_plugin_register Ticket $data
}

# Getting the personal categories
proc ticket_todoplugin_get_personal_categories {db user_id} {
    set categories [database_to_tcl_list_list $db "
    select project_id, title from ticket_projects where ticket_user_can_see_project_p($user_id, project_id)='t'"]

    return $categories
}

# Getting other categories
proc ticket_todoplugin_get_other_categories {db user_id} {
    return [list]
}

# When an item is done
# This just redirects to the right location
# and returns a flag saying that no more information should be returned
proc ticket_todoplugin_mark_completed {db user_id todo_id {completion_date {}}} {
    ns_returnredirect [ticket_url_stub]/issue-fix.tcl?msg_id=$todo_id

    # A boolean of true means everything has been done and the
    # calling page should do nothing more.
    return 1
}

proc ticket_todoplugin_mark_uncompleted {db user_id todo_id {completion_date {}}} {

    ns_db dml $db "update ticket_issues set status='open', fixed_release_id=NULL, fix_date=NULL where msg_id=$todo_id and ticket_user_can_edit_project_p($user_id, project_id)='t'"

    # A boolean of true means everything has been done and the
    # calling page should do nothing more.
    return 0
}


# This should return the SQL
# to select the stuff out of the database
proc ticket_todoplugin_item_list_sql {db user_id category_id date_predicates date_prettifier n_days_to_show_completed_items completion_status order_by} {
    
    set due_date "deadline"

    set list_of_predicates [list]

    if {$date_predicates != ""} {
	lappend list_of_predicates [subst $date_predicates]
    }

    if {$category_id != ""} {
	lappend list_of_predicates "ticket_projects.project_id=$category_id"
    }

    if {$completion_status == "open"} {
	lappend list_of_predicates "fix_date is null"
    } 

    if {$completion_status == "closed"} {
	lappend list_of_predicates "fix_date is not null"
    }

    lappend list_of_predicates "ticket_projects.project_id= ticket_issues.project_id"
    lappend list_of_predicates "ticket_issues.msg_id= ticket_issue_assignments.msg_id"
    lappend list_of_predicates "ticket_issue_assignments.user_id= $user_id"
    lappend list_of_predicates "ticket_issues.close_date is null"

    set sql "
    select 
    'Ticket' as plugin, ticket_projects.project_id as category_id, ticket_projects.title as category,
    1 as priority, ${date_prettifier}($due_date) as due_date, todo_interval_min($due_date) as raw_due_date,
    ''::varchar as assigned_by,
    ticket_issues.msg_id as todo_id, ticket_issues.one_line as item_details, ticket_issues.fix_date as completion_date
    from
    ticket_issues, ticket_issue_assignments, ticket_projects
    where 
    [join $list_of_predicates " and "]"

    return $sql
}
    

