
proc todo_system_name {} {
    return [ad_parameter SystemName todo]
}

proc todo_url_stub {} {
    return [ad_parameter UrlStub todo]
}

proc todo_administrator {} {
    return [ad_parameter Administrator todo]
}

proc todo_header {title} {
    set return_url [ns_conn url]?[ns_conn query]
    
    set return_string "[ad_header $title]"

    # Check if items to delete
    set db [ns_db gethandle subquery]
    set count_check [database_to_tcl_string $db "select count(*) from deleted_todo_items where [ad_get_user_id]=(select user_id from todo_lists where list_id=deleted_todo_items.list_id)"]
    ns_db releasehandle $db

    if {$count_check > 0} {
	set extra_right_link "<a href=deleted-items.tcl>deleted items</a><br>[help_link]"
    } else {
	set extra_right_link "[help_link]"
    }


    if {[ns_conn url] != "/todo/preferences.tcl"} {
	append return_string "
    <table align=right noborder>
    <tr><td><a href=/todo/preferences.tcl?[export_url_vars return_url]>edit preferences</a><br>
$extra_right_link</td></tr>
    </table>"
    }

    return $return_string
}

proc_doc todo_help_header { title } { Returns a help header for Todo. } {
    return "[ad_header $title]
<h2>$title</h2>
<a href=\"javascript:history.back()\">Return to Todo</a>
<hr>
"
}

proc todo_footer {} {
    return "[ad_footer]"
}

#
# Simple user information management
##
proc set_simple_user_information {db} {
    uplevel {
	set user_id [ad_verify_and_get_user_id]

	set user_logged_on_p 0

	if {$user_id > 0} {
	    set selection [ns_db 1row $db "select first_names, last_name, email from users where user_id=$user_id"]
	    set_variables_after_query
	    set user_logged_on_p 1
	}
    }
}


ns_share -init {set ad_todo_security_filters_installed 0} ad_todo_security_filters_installed

if {!$ad_todo_security_filters_installed} {
    set ad_todo_security_filters_installed 1
    # Bounce people out of the /sdm/pvt directory if they're not logged in
    ns_register_filter preauth GET /todo/* ad_verify_identity
    ns_register_filter preauth POST /todo/* ad_verify_identity
}


proc user_can_access_list_p {db user_id list_id} {
    set check [database_to_tcl_string $db "select user_can_access_list_p($user_id,$list_id) from dual"]

    if {$check == "t"} {
	return 1
    } else {
	return 0
    }
}

proc todo_return_access_complaint {} {
    ns_return 500 text/html "[todo_header "Access Violation"]
    <h2>Access Violation</h2>
    <hr><p>
    You are not allowed to view this information, either because you are not logged on, or if you are, because this information is private and you are not authorized.
    <p>
    [todo_footer]
"
}

proc todo_set_user_preferences {db user_id} {
    uplevel {
	set selection [ns_db 0or1row $db "select old_item_expire, COALESCE(time_offset,0) as time_offset, COALESCE(personal_midnight,0) as personal_midnight, sort_by, notification_interval, separate_completed_items_p from todo_user_preferences where user_id=$user_id"]

	if {$selection == "" } {
	    ns_db dml $db "insert into todo_user_preferences (user_id) VALUES ($user_id)"
	    todo_set_user_preferences $db $user_id
	}

	set_variables_after_query

	# Why was this here? (BMA)
	# set sort_by [subst $sort_by]
    }
}

proc todo_daily_sweep {} {
    set db [ns_db gethandle]

    # The -0.01 is for slight inconsistencies in times that the proc runs
    set user_ids [database_to_tcl_list $db "select user_id from todo_user_preferences where notification_interval>0 and (sysdate()>=(last_notification+notification_interval-0.01) or last_notification is NULL)"]

    foreach user_id $user_ids {
	ns_log Notice "doing email for user_id $user_id"

	set notification_interval [database_to_tcl_string $db "select notification_interval from todo_user_preferences where user_id=$user_id"]

	set selection [ns_db select $db "select list_name, item_id, first_names as assigned_by_first_names, todo_lists.list_id as list_id, completion_date, pretty_relative_date(todo_items.due_date, 0) as pretty_relative_due_date, todo_days_from_pretty(pretty_relative_date(todo_items.due_date,0)) as n_days, substr(item_details,0,60) as short_item_details, item_details, priority from todo_items, todo_lists, users where todo_items.assigned_by=users.user_id(+) and todo_items.list_id= todo_lists.list_id and (same_day_p(todo_items.due_date,sysdate(), 0)='t' or todo_items.due_date<(sysdate()+$notification_interval)) and todo_lists.user_id=$user_id and (completion_date is NULL)"]

	set things_to_do_text ""

	while {[ns_db getrow $db $selection]} {
	    set_variables_after_query

	    append things_to_do_text "- ($priority) $pretty_relative_due_date, for $list_name: $item_details\n"
	}

	set email [database_to_tcl_string $db "select email from users where user_id=$user_id"]
	
	if {[catch {
	    if {[string length $things_to_do_text] > 0} {
		ns_log Notice "sending the email"
		ns_sendmail "$email" "ben@openforce.net" "Things to do" "
		In the next $notification_interval day(s), you have the following things to do
\n
$things_to_do_text
\n\n
-ToDo Manager"
}
} errmsg]} {
    ns_log Notice "error: $errmsg"
} else {
    ns_db dml $db "update todo_user_preferences set last_notification=sysdate() where user_id=$user_id"
}
}

ns_db releasehandle $db

}



## A procedure to output things consistently
## for list of todos
proc todo_one_item_form {submit_tag} {
    set db [ns_db gethandle subquery]

    set list_of_dates [database_to_tcl_list_list $db "select days,pretty_date from relative_date_pretty_text where days>=0 or days is NULL order by days"]

    set string "
<tr>
<td>[make_html_select priority {1 2 3 4 5}]</td>
<td>[make_html_select n_days $list_of_dates]</td>
<td><INPUT TYPE=text name=item_details size=50></td>
<td><INPUT TYPE=submit value=$submit_tag></td>
</tr>
"

ns_db releasehandle $db

return $string

}

# The list of headers for the front page
proc todo_delete_list_of_headers {} {
    return {
	{priority Priority}
	{list_name List}
	{short_details Details}
	{deletion_date {Deletion Date}}
    }
}

proc todo_general_list_of_headers {} {
    return {
	{priority Priority}
	{list_name List}
	{item_details Details}
	{assigned_by_first_names {Assigned by}}
    }
}

proc todo_general_plugin_list_of_headers {} {
    return {
	{priority Priority}
	{category Category}
	{item_details Details}
	{assigned_by {Assigned by}}
    }
}

# List of specific stuff
proc todo_specific_list_of_headers {} {
    return {
	{priority Priority}
	{pretty_relative_due_date {Due Date}}
	{item_details Details}
	{assigned_by_first_names {Assigned by}}
    }
}

proc todo_plugin_list_of_headers {} {
    return {
	{priority Priority}
	{due_date {Due Date}}
	{item_details Details}
	{assigned_by {Assigned By}}
    }
}

## The table headers
proc todo_list_html_header {list_of_headers} {
    set return_html "<tr bgcolor=lightgrey>\n"
    foreach header $list_of_headers {
	append return_html "<th align=left>[lindex $header 1]</th>"
    }
    append return_html "<th>Actions</th></tr>\n"
    
    return $return_html
}


# The tabs for open or closed items
proc todo_openclosed_tabs {view_open_items} {
    if {$view_open_items == ""} {
	return ""
    }
    
    set current_url [ns_conn url]?[export_ns_set_vars url view_open_items [ns_getform]]

    set return_html "<table border=0 cellpadding=4 cellspacing=0><tr>"

    if {$view_open_items == "f"} {
	append return_html "<th bgcolor=#bbbbbb><a href=${current_url}&view_open_items=t>Open</a></th><th bgcolor=#dddddd>Closed</th>"
    } else {
	append return_html "<th bgcolor=#dddddd>Open</th><th bgcolor=#bbbbbb><a href=${current_url}&view_open_items=f>Closed</a></th>"
    }

    append return_html "</tr></table>"

    return $return_html
}

# The list_of_headers is a list of list
# with first the sql name, second the pretty name.
proc todo_list_html {db sql current_url list_of_headers {extra_rows {}} {item_id_to_edit {}} {view_open_items ""}} {

    if {[string first "?" $current_url] == -1} {
	set current_url "$current_url?"
    }

    set return_url "$current_url"

    ## Build the header stuff
    #
    set return_header ""
    set always_header ""

    if {$view_open_items != ""} {
	append always_header "<table noborder cellpadding=3><tr>"
	if {$view_open_items == "t"} {
	    regsub " todo_items " $sql " open_todo_items todo_items " sql

	    append always_header "<th bgcolor=#dddddd>Open</th><th bgcolor=#bbbbbb><a href=${current_url}&view_open_items=f>Closed</a></th>"
	} else {
	    regsub " todo_items " $sql " closed_todo_items todo_items " sql
	    
	    append always_header "<th bgcolor=#bbbbbb><a href=${current_url}&view_open_items=t>Open</a></th><th bgcolor=#dddddd>Closed</th>"
	}
	append always_header "</tr></table>\n"
    } else {
	# Fix so we only see the undeleted ones
	regsub " todo_items " $sql " viewable_todo_items todo_items " sql
    }

    append return_header "<table noborder>\n<tr>\n"
    foreach header $list_of_headers {
	append return_header "<th align=left>[lindex $header 1]</th>"
    }
    append return_header "<th>Actions</th></tr>\n"

    set return_string ""

    ## Do the query
    #
    set selection [ns_db select $db $sql]
    
    while {[ns_db getrow $db $selection]} {
	set_variables_after_query

	if {$item_id_to_edit == $item_id} {
	    append return_string "<FORM METHOD=POST action=item-edit.tcl>
	    [export_form_vars item_id return_url]
	    [bt_mergepiece [todo_one_item_form "edit"] $selection]
	    </FORM>\n"
	    continue
	}
	
	set one_row "<tr>\n"
	
	# In case it's completed
	if {$completion_date == ""} {
	    set pre_item ""
	    set post_item ""
	    set actions_html "<td><a href=item-completed.tcl?[export_url_vars item_id return_url]><img align=middle height=15 border=0 src=unchecked.gif></a>  <a href=${current_url}&item_id_to_edit=$item_id>edit</a></td>"
	} else {
	    set pre_item "<strike>"
	    set post_item "</strike>"
	    set return_url $current_url
	    set actions_html "<td><nobr><a href=item-uncompleted.tcl?[export_url_vars item_id return_url]><img align=middle height=15 border=0 src=checked.gif></a> <a href=item-delete.tcl?[export_url_vars return_url item_id]><img align=middle height=15 border=0 src=delete.gif></a></nobr></td>"
	}

	# If item is deleted, totally change the action
	if {[info exists deletion_date] && $deletion_date!=""} {
	    set return_url $current_url
	    set actions_html "<td><a href=item-undelete.tcl?[export_url_vars item_id return_url]>undelete</a></td>"
	}
	
	foreach header $list_of_headers {
	    # Special case for list name
	    if {[lindex $header 0] == "list_name"} {
		append one_row "<td>$pre_item<a href=one-list.tcl?[export_url_vars list_id]>$list_name</a>$post_item</td>"
		continue
	    }

	    # Special case for item details
	    if {[lindex $header 0] == "item_details"} {
		if {[string length $item_details] > 100} {
		    append one_row "<td>$pre_item [string range $item_details 0 100]... (<a target=detail_window href=one-item.tcl?[export_url_vars item_id]>more</a>) $post_item</td>"
		    continue
		}
	    }

	    if {[lindex $header 0] == "priority"} {
		append one_row "<td>$pre_item<strong>($priority)</strong>$post_item</td>"
		continue
	    }
	    
	    append one_row "<td>$pre_item[set [lindex $header 0]]$post_item</td>"
	}
	
	append one_row "$actions_html</tr>\n"

	append return_string $one_row
    }
    
    if {$item_id_to_edit == ""} {
	append return_string "$extra_rows"
    } 

    if {$return_string == ""} {
	return "$always_header\n no items"
    } else {
	set return_string "$always_header\n $return_header\n$return_string"
    }

    append return_string "</table>"

    if {$item_id_to_edit != ""} {
	append return_string "<a href=$current_url>cancel edit</a>\n"
    }
    
    return $return_string
}


# A procedure to send email if necessary
proc todo_email {db item_id event} {
    set assigner_user_id [database_to_tcl_string $db "select assigned_by from todo_items where item_id=$item_id"]
    set assignee_user_id [database_to_tcl_string $db "select user_id from todo_lists where list_id= (select list_id from todo_items where item_id=$item_id)"]

    # No notification if you did something yourself

    if {$assigner_user_id == ""} {
	return
    }
    if {$assignee_user_id == $assigner_user_id} {
	return
    }

    set selection [ns_db 1row $db "select item_details as details, due_date from todo_items where item_id= $item_id"]
    set_variables_after_query

    # Here we want to set the 
    # from_email, from_first_names, from_last_name
    # to_email, to_first_names, to_last_name
    if {$event == "assigned"} {
	set from_user_id $assigner_user_id
	set to_user_id $assignee_user_id

	set subject "new todo item"
	set action "has assigned you"
    }

    if {$event == "completed"} {
	set from_user_id $assignee_user_id
	set to_user_id $assigner_user_id

	set subject "item you assigned is completed"
	set action "has completed"
    }

    if {$event == "uncompleted"} {
	set from_user_id $assignee_user_id
	set to_user_id $assigner_user_id

	set subject "item you assigned is NO LONGER completed"
	set action "has NOT completed"
    }

    set selection [ns_db 1row $db "select email as from_email, first_names as from_first_names, last_name as from_last_name from users where user_id=$from_user_id"]
    set_variables_after_query

    set selection [ns_db 1row $db "select email as to_email, first_names as to_first_names, last_name as to_last_name from users where user_id=$to_user_id"]
    set_variables_after_query

    # Send the email
    if {[catch {
	ns_sendmail $to_email $from_email "$subject" "
$from_first_names $from_last_name $action item #$item_id:

$details
"
} errmsg]} {
    ns_log Warning "Email could not be sent in the todo manager because: $errmsg"
}
}


## A proc to get the view_open_items var
proc todo_view_open_items {separate_completed_items_p} {
    if {$separate_completed_items_p == "f"} {
	return ""
    }

    set view_open_items [ns_queryget "view_open_items"]

    if {$view_open_items == ""} {
	set view_open_items "t"
    }

    return $view_open_items
}

#
# A procedure to really delete old deleted items.
#
proc todo_delete_old_items {} {
    set db [ns_db gethandle subquery]
    
    ns_db dml $db "delete from deleted_todo_items where (deletion_date+ timespan_days(2))<sysdate()"

    ns_db releasehandle $db
}

# More things
proc todo_n_days_from_pretty {due_date} {
    return [util_memoize "todo_n_days_from_pretty_no_memoize \"$due_date\""]
}

proc todo_n_days_from_pretty_no_memoize {due_date} {
    set db [ns_db gethandle subquery]
    set result [database_to_tcl_string $db "select todo_days_from_pretty('$due_date') from dual"]
    ns_db releasehandle $db

    return $result
}

ns_share -init {set ad_todo_procs_scheduled 0} ad_todo_procs_scheduled

if {!$ad_todo_procs_scheduled} {
    set ad_todo_procs_scheduled 1
    # Schedule this every day at 7am EST
    ns_schedule_daily 7 0 todo_daily_sweep
    ns_schedule_daily 8 0 todo_delete_old_items
}

