##
## This is where things get crazy recursive: the todo module as a plugin to itself.
## This will help to standardize things tremendously
##
## written by Ben Adida (ben@adida.net)

ns_schedule_proc -once 1 todo_todoplugin_register

# The proc to register the TODO todo plugin
proc todo_todoplugin_register {} {
    set data [ns_set new -persist todo_todoplugin_data]
    ns_set put $data GET_PERSONAL_CATEGORIES todo_todoplugin_get_personal_categories
    ns_set put $data GET_OTHER_CATEGORIES todo_todoplugin_get_other_categories
    ns_set put $data ITEM_CHANGE_ALLOWED_P 1
    ns_set put $data ITEM_CHANGE todo_todoplugin_item_change
    ns_set put $data MARK_COMPLETED_ALLOWED_P 1
    ns_set put $data MARK_COMPLETED todo_todoplugin_mark_completed
    ns_set put $data MARK_UNCOMPLETED todo_todoplugin_mark_uncompleted
    ns_set put $data ONE_ITEM_URL [todo_url_stub]/one-item.tcl
    ns_set put $data ONE_ITEM_ID_KEY item_id
    ns_set put $data ITEM_LIST_SQL todo_todoplugin_item_list_sql
    ns_set put $data CATEGORY_FOOTER_HTML todo_todoplugin_category_footer_html
    ns_set put $data ITEM_ADD_ALLOWED_P 1
    ns_set put $data ITEM_ADD todo_todoplugin_item_add
    ns_set put $data ITEM_DELETE todo_todoplugin_item_delete

    # actually register
    todo_plugin_register "todo" $data
}

# Getting the personal categories
proc todo_todoplugin_get_personal_categories {db user_id} {
    set categories [database_to_tcl_list_list $db "
    select todo_lists.list_id, todo_lists.list_name from todo_lists where user_id= $user_id"]

    return $categories
}

# Getting other categories
proc todo_todoplugin_get_other_categories {db user_id} {
    set categories [database_to_tcl_list_list $db "
    select todo_lists.list_id, case when last_name=email then ' ' || email || ' - ' || todo_lists.list_name else last_name || ', ' || first_names || ' - ' || todo_lists.list_name end as list_name from todo_lists, users where todo_lists.user_id!= $user_id and todo_lists.user_id= users.user_id and user_can_access_list_p($user_id, list_id)='t'
    order by list_name
    "]

    return $categories
}

# When an item is changed
# Do the edit
proc todo_todoplugin_item_change {db user_id todo_id new_date new_priority new_item_details} {
    ns_db dml $db "update todo_items set 
    priority= $new_priority,
    due_date= $new_date,
    item_details= '[DoubleApos $new_item_details]'
    where
    item_id= $todo_id
    and user_can_access_list_p($user_id, list_id)='t'"

    # If we return 0, the calling page will keep going, which is what we want
    return 0
}
    

# When an item is done
# This does the right thing
proc todo_todoplugin_mark_completed {db user_id todo_id {completion_date {}}} {
    ns_db dml $db "update todo_items set completion_date=sysdate() where item_id=$todo_id and user_can_access_list_p($user_id, list_id)='t'"

    todo_email $db $todo_id "completed"

    # We're done, and we let the proc do the redirect
    return 0
}

proc todo_todoplugin_mark_uncompleted {db user_id todo_id {completion_date {}}} {
    ns_db dml $db "update todo_items set completion_date=NULL where item_id=$todo_id and user_can_access_list_p($user_id, list_id)='t'"

    todo_email $db $todo_id "uncompleted"

    # We're done, and we let the proc do the redirect
    return 0
}


# This should return the SQL
# to select the stuff out of the database
proc todo_todoplugin_item_list_sql {db user_id category_id date_predicates date_prettifier n_days_to_show_completed_items completion_status order_by} {
    
    set due_date "todo_items.due_date"

    set list_of_predicates [list]

    if {$date_predicates != ""} {
	# We substitue var names
	lappend list_of_predicates [subst $date_predicates]
    }

    # If we have no specified list here, we only show the ones owned
    # by the user, otherwise it could get very messy (BMA)
    if {$category_id != ""} {
	lappend list_of_predicates "todo_lists.list_id=$category_id"
	lappend list_of_predicates "user_can_access_list_p($user_id, todo_lists.list_id)='t'"
    } else {
	lappend list_of_predicates "todo_lists.user_id= $user_id"
    }    

    if {$completion_status == "open"} {
	lappend list_of_predicates "todo_items.completion_date is null"
    } else {
	lappend list_of_predicates "(todo_items.completion_date is null OR todo_items.completion_date + timespan_days($n_days_to_show_completed_items) >= sysdate())"
    }
	

    if {$completion_status == "closed"} {
	lappend list_of_predicates "todo_items.completion_date is not null"
    }

    lappend list_of_predicates "todo_items.list_id= todo_lists.list_id"
    lappend list_of_predicates "todo_items.deletion_date is null"

    set sql "
    select 
    'todo' as plugin, todo_lists.list_id as category_id, todo_lists.list_name as category,
    priority, ${date_prettifier}(todo_items.due_date) as due_date, todo_interval_min(todo_items.due_date) as raw_due_date,
    user_full_name(todo_items.assigned_by) as assigned_by,
    todo_items.item_id as todo_id, item_details, completion_date
    from
    todo_lists, todo_items
    where 
    [join $list_of_predicates " and "]
    $order_by"

    return $sql
}
    
proc todo_todoplugin_category_footer_html {db user_id category_id} {
    set return_html ""

    set list_id $category_id

    append return_html "<p>
    You can also <a href=[todo_url_stub]/new-item.tcl?[export_url_vars list_id]>add a detailed item</a>.<p>"

    # ownership check
    set check [database_to_tcl_string $db "select count(*) from todo_lists where user_id=$user_id and list_id=$list_id"]

    if {$check} {
	append return_html "<p> 
	Collaboration:
	<ul>
	<li> <a href=[todo_url_stub]/new-permission.tcl?[export_url_vars list_id]>Add a collaborator</a>.
	<p>
	"

	set selection [ns_db select $db "select user_id, first_names, last_name from users where user_id in (select user_id from todo_list_user_map where list_id=$list_id)"]

	while {[ns_db getrow $db $selection]} {
	    set_variables_after_query

	    append return_html "<li> \[ <A href=delete-permission.tcl?[export_url_vars list_id user_id]>remove</a> \] $first_names $last_name\n"
	}

	append return_html "<p>"

	set selection [ns_db select $db "select group_name, group_id from user_groups where group_id in (select user_group_id from todo_list_user_group_map where list_id=$list_id)"]

	while {[ns_db getrow $db $selection]} {
	    set_variables_after_query

	    append return_html "<li> Group: $group_name\n"
	}

	append return_html "</ul>
	<p>
	You may choose to <a href=[todo_url_stub]/list-delete.tcl?[export_url_vars list_id]>delete</a> this list.<p>
	"
    } else {
	set selection [ns_db 1row $db "select first_names, last_name, email from users where user_id=(select user_id from todo_lists where list_id=$list_id)"]
	set_variables_after_query

	append return_html "<p>This list belongs to $first_names $last_name ($email).<p>"
    }

    return $return_html
}

# Adding something
proc todo_todoplugin_item_add {db user_id category_id due_date priority item_details} {

    set todo_id [database_to_tcl_string $db "select nextval('todo_item_id_sequence')"]

    if {[database_to_tcl_string $db "select user_can_access_list_p($user_id, $category_id)"] == "t"} {
	ns_db dml $db "insert into todo_items (list_id, item_id, creation_date, due_date, priority, item_details, assigned_by) values ($category_id, $todo_id, sysdate(), $due_date, $priority, '[DoubleApos $item_details]', $user_id)"
    }

    todo_email $db $todo_id "assigned"

    # return false to ensure that the redirect happens
    return 0
}

# Removing something
proc todo_todoplugin_item_delete {db user_id category_id todo_id} {
    ns_db dml $db "update todo_items set deletion_date=sysdate() where item_id=$todo_id and user_can_access_list_p($user_id, $category_id)='t' and list_id=$category_id"

    return 0
}