<%= [openacs_header "OpenACS 4.x Guidlines"] %>
<%= [openacs_menu]%> <p>
<h2>OpenACS 4.0 Guidelines</h2>
by <a href=/shared/community-member.tcl?user_id=>Don Baccus</a> and <a href=/shared/community-member.tcl?user_id=4>Ben Adida</a>.
<hr><p>

<h2>Simple Issues</h2>

We faced most of these issues at some point during the process of developing OpenACS 3.x. It's now time to standardize these and do everything correctly in 4.x!


<h3>Sequences</h3>
Use Dan Wickstrom's sequence view hack. The idea is to structure a view around a sequence so that the <tt>foo_seq.nextval</tt> construct can work.
<p>
<pre>
create sequence user_id_seq;
create view user_id_sequence as select nextval('user_id_seq') as nextval;
</pre>

<h3>Empty Strings</h3>
Be stricter, not Oracle'ish. Default all varchars to empty string not null.
<p>
<pre>
create table foo (
       some_text varchar(100) default '' not null
);
</pre>

<h3>Sysdate</h3> 

No using sysdate in the data model. This is pretty important given
that we don't really want to have the basic data-model depend on a
PL/SQL procedure. In general, avoid user-defined functions in the data
model. Otherwise, the sysdate() function works fine.

<h3>Case/Decode, NVL/Coalesce</h3>
These issues are solved, but let's just remind everyone how they should be solved<p>

In Oracle, a decode statement looks like this:
<pre>
select sum(decode(available_p,'t', 1,0)) as n_available from foo;
</pre>
<p>
In PG, this looks like this:
<pre>
select sum(case when available_p='t' then 1 else 0 end) as n_available from foo;
</pre>

<p>

In Oracle, an NVL statement looks like:
<pre>
select user_id, nvl(screen_name, '(no screen name)') from users;
</pre>
<p>
In PG, it looks like:
<pre>
select user_id, coalesce(screen_name, '(no screen name)') from users;
</pre>
<p>
Note that coalesce is more powerful than NVL in that you can list any
number of arguments, and the first non-null one will be returned. With
NVL, you have to nest a number of NVL statements to achieve the same
effect.

<h3>Outer joins</h3>
conversion examples by Don.


<h2>Dispatching New Queries</h2>

This is a pretty involved topic which will be handled in a separate document. We note one important thing:
we can easily identify which page we are in using <tt>[ns_conn
url]</tt> and the Tcl procedure we are in using <tt>ns_info</tt>. This
means that the unique tagging scheme used in ACS 4.0 will work
perfectly fine to replace simple SQL queries using a different source.

<h2>Data Model Changes</h2>

We need to standardize ways in which a data model is modified from
Oracle to Postgres (7.1 or later):

<ul>
<li> Char(1) becomes boolean
<li> date becomes timestamp
<li> lztext gone, we use text
<li> varchar(4000) becomes text
<li> varchar < 1000 remains varchar
<li> clob becomes text
<li> blobs are moved to filesystem level persistent object stuff (content repository).
<li> move to Postgres object extensions (declare indexes and triggers
on all child tables). This may still be somewhat controversial. We
will discuss this on the bboards, right <a href=/bboard>here</a>.
<li> <tt>comment on</tt> is supported, which means all the commenting
can be kept precisely as is.
</ul>

<h2>PL/pgSQL</h2>
<ul>
<li> function names when packages are involved: <tt>package__function</tt> (two underbars)
<li> 16 parameters enough? We will see.
<li> default params: function overloading for common cases.
</ul>

<h2>Things for PG/GB to do</h2>
<ul>
<li> up the identifier length to 64.
<li> up the # of PL/pgSQL parameters to 32.
</ul>

<h2>Questions for aD</h2>
<ul>
<li> what's with the disabled constraints, followed by index creation, followed by constraint reenablement?
<li> what is the security_inherit_p attribute, and why no explicit
attribute created for it? <i>Answer: that is used by the AFS-like
permission scheme, where a security context is inherited or not</i>.
</ul>

<h2>Random Issues</h2>
<ul>
<li> Email done within Oracle?? No way - a new notifications module
<li> acs_object_context_index may not be necessary? Create views on top of pg_inherits?
<li> Adding a content type that is a persistent object in the FS, that other systems rely on.
</ul>

<h2>Immediate Next Steps</h2>
<ul>
<li> Who's available to do work?
<li> Putting together a full doc of all stuff discussed by Don and Ben <b>done!</b>
<li> Pushing PG and GB on a couple of issues <b>this cannot happen for
7.1. Maybe 7.2</b>
<li> Putting a CVS solution/repository <b>begun on openacs.org</b>
<li> DB layer - fetching queries from somewhere else <b>initial design
in progress</b>
<li> DB bind variables <b>work in progress</b>
<li> APM 
<li> Porting Kernel
<li> a Tcl API for storing large files
</ul>
<p>

<%= [openacs_footer] %>