# post-edit-2.tcl,v 3.0 2000/02/06 03:36:13 ron Exp
# File:     /calendar/admin/post-edit-2.tcl
# Date:     1998-11-18
# Contact:  philg@mit.edu, ahmeds@arsdigita.com
# Purpose:  edits one calendar item
#
# Note: if page is accessed through /groups pages then group_id and group_vars_set are already set up in 
#       the environment by the ug_serve_section. group_vars_set contains group related variables (group_id, 
#       group_name, group_short_name, group_admin_email, group_public_url, group_admin_url, group_public_root_url,
#       group_admin_root_url, group_type_url_p, group_context_bar_list and group_navbar_list)

if {[ad_read_only_p]} {
    ad_return_read_only_maintenance_message
    return
}

set user_id [ad_get_user_id]
if { $user_id == 0 } {
    ns_returnredirect "/register/index.tcl?[export_url_scope_vars]"
    return
}

set_the_usual_form_variables 0
# category_id, calendar_id, title, body, AOLserver ns_db magic vars that can be 
# kludged together to form release_date and expiration_date
# maybe scope, maybe scope related variables (user_id, group_id, on_which_group, on_what_id)

ad_scope_error_check
set db [ns_db gethandle]
ad_scope_authorize $db $scope admin group_admin none

set exception_count 0
set exception_text ""

if [catch  { ns_dbformvalue [ns_conn form] start_date date start_date
             ns_dbformvalue [ns_conn form] end_date date end_date } errmsg] {
    incr exception_count
    append exception_text "<li>Please make sure your dates are valid."

} else {

    # we assume that the event ends at the very end of the end_date
    # we have to do the bogus 1000* and then rounding because of Stupid Oracle
    # driver truncation errors (doesn't like big fractions)
    set expire_laterthan_future_p [database_to_tcl_string $db "select date_part('epoch', '$end_date  11:59:59'::datetime -'$start_date'::datetime)"]
    if {$expire_laterthan_future_p <= 0} {
	incr exception_count
	append exception_text "<li>Please make sure the end date is later than the start date."
    }
}

# now start_date and end_date are set

if { ![info exists title] || $title == ""} {
    incr exception_count
    append exception_text "<li>Please enter a title."
}
if { ![info exists body] || $body == "" } {
    incr exception_count
    append exception_text "<li>Please enter the full story."
}

if { [ad_parameter InternationalP] && ![info exists country_code] } {
    incr exception_count
    append exception_text "<li>Please select the country"
} else { 
    set country_code "" 
}

if { [ad_parameter SomeAmericanReadersP] && ![info exists usps_abbrev] } {
    incr exception_count
    append exception_text "<li>Please select the state"
} else {  
    set usps_abbrev ""  
}

if ![info exists zip_code] {
    set zip_code ""
}

if { [info exists event_email] && ![empty_string_p $event_email] && ![philg_email_valid_p $event_email] } {
    incr exception_count
    append exception_text "<li>The event contact email address that you typed doesn't look right to us.  Examples of valid email addresses are 
<ul>
<li>Alice1234@aol.com
<li>joe_smith@hp.com
<li>pierre@inria.fr
</ul>
"
}

if { [info exists event_url] && ![philg_url_valid_p $event_url] } {
    set event_url ""
}

if {$exception_count > 0} { 
    ad_scope_return_complaint $exception_count $exception_text $db
    return
}


if [catch { ns_db dml $db "update calendar
set category_id = $category_id, title='$QQtitle', 
body='$QQbody', html_p='$html_p',
start_date = '$start_date', end_date = '$end_date  11:59:59'::datetime,
expiration_date= '$end_date  11:59:59'::datetime + [ad_parameter DaysFromEndToExpiration calendar 3],
event_url = [ns_dbquotevalue $event_url text], 
event_email = [ns_dbquotevalue $event_email text], 
country_code = [ns_dbquotevalue $country_code text], 
usps_abbrev = [ns_dbquotevalue $usps_abbrev text], 
zip_code = [ns_dbquotevalue $zip_code text]
where calendar_id = $calendar_id"
} errmsg] {
    # update failed; let's see if it was because of duplicate submission
    ns_log Error "/calendar/admin/post-edit-2.tcl choked:  $errmsg"
    ad_scope_return_error "Update Failed" "The Database did not like what you typed.  This is probably a bug in our code.  Here's what the database said:
<blockquote>
<pre>
$errmsg
</pre>
</blockquote>
" $db
    return
}


ns_returnredirect "item.tcl?[export_url_scope_vars calendar_id]"

