-- /www/doc/sql/intranet.sql
--
-- A complete revision of June 1999 by dvr@arsdigita.com
--
-- mbryzek@arsdigita.com, January 2000
--
-- intranet.sql,v 3.4.2.6 2000/03/17 21:58:51 ron Exp


--DRB: intranet apparently has been almost entirely rewritten.

-- What states can our customers be in?
create sequence im_customer_status_seq start 1;
create table im_customer_status (
	customer_status_id	integer primary key,
	customer_status		varchar(100) not null unique,
	display_order		integer default 1
);


-- we store simple information about a customer
-- all contact information goes in the address book
create table im_customers (
	group_id 	 	integer primary key references user_groups,
	deleted_p        	char(1) default('f') constraint im_customers_deleted_p check(deleted_p in ('t','f')),
	customer_status_id      integer references im_customer_status,
	primary_contact_id	integer references address_book,
	note			varchar(4000),
	-- keep track of when status is changed
        status_modification_date datetime,
	-- and what the old status was
        old_customer_status_id  integer references im_customer_status
);

-- What are the different project types that we support
create sequence im_project_types_id_seq start 1;
create table im_project_types (
	project_type_id		integer primary key,
	project_type		varchar(200) not null unique,
	display_order		integer default 1
);

-- In what states can our projects be?
create sequence im_project_status_id_seq start 1;
create table im_project_status (
	project_status_id	integer primary key,
	project_status		varchar(100) not null unique,
	display_order		integer default 1
);


-- What types of urls do we ask for when creating a new project
-- and in what order?
create sequence im_url_types_type_id_seq start 1;
create table im_url_types (
	url_type_id		integer not null primary key,
	url_type		varchar(200) not null unique,
	-- we need a little bit of meta data to know how to ask 
	-- the user to populate this field
	to_ask			varchar(1000) not null,
	-- if we put this information into a table, what is the 
	-- header for this type of url?
	to_display		varchar(100) not null,
	display_order		integer default 1
);
	

-----------------------------------------------------------
-- Projects
--
-- 1) Each project can have any number of sub-projects
--
-- 2) Each (sub)project can be billed either hourly or
-- monthly, but not both. If part of a contract is being
-- billed hourly, create a top-level project specifying
-- the monthly fee and a subproject for the work being done 
-- hourly.

create table im_projects (
    -- each project is a subgroup of the group Projects of group_type intranet
    -- the name of the project is the group_name
    group_id                integer primary key references user_groups,
    parent_id               integer references im_projects,
    customer_id             integer references im_customers,
    project_type_id	    integer not null references im_project_types,
    project_status_id	    integer not null references im_project_status,
    description             varchar(4000),
    -- fees
    fee_setup               float8,
    fee_hosting_monthly     float8,	
    fee_monthly             float8,
    bill_hourly_p           char(1) default 't' check (bill_hourly_p in ('t','f')),
    start_date              datetime,
    end_date		    datetime,
    -- make sure the end date is after the start date
    -- constraint im_projects_date_const check( end_date - start_date >= 0::timespan ),	
    note                    varchar(4000),
    project_lead_id	    integer references users,
    supervisor_id	    integer references users,
    ticket_project_id       integer references ticket_projects
);
create index im_project_parent_id_idx on im_projects(parent_id);


-- we store all urls and their types
create table im_project_url_map (
	group_id		integer not null references im_projects,
	url_type_id		integer not null references im_url_types,
	url			varchar(250),
	-- each project can have exactly one type of url
	primary key (group_id, url_type_id)
);
-- We need to create an index on url_type_id if we ever want to ask
-- "What are all the staff servers?"
create index im_proj_url_url_proj_idx on im_project_url_map(url_type_id, group_id);

-- What states can our customers be in?

create table im_employee_info (
	user_id             integer primary key references users,
	job_title           varchar(200),
	job_description     varchar(4000),
	-- is this person an official team leader?
	team_leader_p	    char(1) 
		constraint im_employee_team_lead_con check (team_leader_p in ('t','f')),
	-- can this person lead projects?
	project_lead_p      char(1) 
		constraint im_employee_project_lead_con check (project_lead_p in ('t','f')),
	-- percent of a full time person this person works
	percentage	    integer,
	supervisor_id       integer references users,
	group_manages       varchar(100),
	current_information   varchar(4000),
	--- send email if their information is too old
	last_modified       datetime default current_timestamp not null,
	ss_number           varchar(20),
	salary              float8,
	salary_period       varchar(12) default 'month' 
	      constraint im_employee_salary_period_con check (salary_period in ('hour','day','week','month','year')),
	--- W2 information
	dependant_p         char(1) 
		constraint im_employee_dependant_p_con check (dependant_p in ('t','f')),
	only_job_p          char(1) 
		constraint im_employee_only_job_p_con check (only_job_p in ('t','f')),
	married_p           char(1) 
		constraint im_employee_married_p_con check (married_p in ('t','f')),
	dependants          integer default 0,
	head_of_household_p char(1) 
		constraint im_employee_head_of_house_con check (head_of_household_p in ('t','f')),
	birthdate           datetime,
	skills              varchar(2000),
	first_experience    datetime,	
	years_experience    float8,
	educational_history varchar(4000),
	last_degree_completed    varchar(100),
	resume			 text,
	resume_html_p		 char(1) 
		constraint im_employee_resume_html_p_con check (resume_html_p in ('t','f')),
	start_date          datetime,
	received_offer_letter_p	char(1) 
		constraint im_employee_recv_offer_con check(received_offer_letter_p in ('t','f')),
	returned_offer_letter_p char(1) 
		constraint im_employee_return_offer_con check(returned_offer_letter_p in ('t','f')),
	-- did s/he sign the confidentiality agreement?
	signed_confidentiality_p char(1) 
		constraint im_employee_conf_p_con check(signed_confidentiality_p  in ('t','f')),   
	most_recent_review  datetime,
	most_recent_review_in_folder_p char(1) 
		constraint im_employee_recent_review_con check(most_recent_review_in_folder_p in ('t','f')),
	featured_employee_approved_p char(1) 
		constraint featured_employee_p_con check(featured_employee_approved_p in ('t','f')),
	featured_employee_approved_by integer references users,
	-- SCC: somebody eradicate the clob!
	featured_employee_blurb varchar(4000),
	featured_employee_blurb_html_p char(1) default 'f'
		constraint featured_emp_blurb_html_p_con check (featured_employee_blurb_html_p in ('t','f')),
	referred_by             integer references users
);

-- next two views were created to ease porting of outer joins.  The views allow
-- constructs such as info.* to be retained.  See /intranet/users/view.tcl for
-- an example of their usage.

-- drop view im_employee_info_null;

create view im_employee_info_null as 
select 
''::integer as user_id
,''::varchar as job_title                      
,''::varchar as job_description                
,''::char as team_leader_p                  
,''::char as project_lead_p                 
,''::integer as percentage                     
,''::integer as supervisor_id                  
,''::varchar as group_manages                  
,''::varchar as current_information            
,''::varchar as last_modified                  
,''::varchar as ss_number                      
,''::float as salary8                         
,''::varchar as salary_period                  
,''::char as dependant_p                    
,''::char as only_job_p                     
,''::char as married_p                      
,''::integer as dependants                     
,''::char as head_of_household_p            
,''::varchar as birthdate                      
,''::varchar as skills                         
,''::varchar as first_experience               
,''::float as years_experience8               
,''::varchar as educational_history            
,''::varchar as last_degree_completed          
,''::text as resume                         
,''::char as resume_html_p                  
,''::varchar as start_date                     
,''::char as received_offer_letter_p        
,''::char as returned_offer_letter_p        
,''::char as signed_confidentiality_p       
,''::varchar as most_recent_review             
,''::char as most_recent_review_in_folder_p 
,''::char as featured_employee_approved_p   
,''::integer as featured_employee_approved_by  
,''::varchar as featured_employee_blurb        
,''::char as featured_employee_blurb_html_p 
,''::integer as referred_by;


-- drop view im_employee_info_view;

create view im_employee_info_view as 
select 
user_id
,job_title                      
,job_description                
,team_leader_p                  
,project_lead_p                 
,percentage                     
,supervisor_id                  
,group_manages                  
,current_information            
,to_char(last_modified,'YYYY-MM-DD') as last_modified
,ss_number                      
,salary                        
,salary_period                  
,dependant_p                    
,only_job_p                     
,married_p                      
,dependants                     
,head_of_household_p            
,to_char(birthdate,'YYYY-MM-DD') as birthdate
,skills                         
,to_char(first_experience,'YYYY-MM-DD') as first_experience               
,years_experience               
,educational_history            
,last_degree_completed          
,resume                         
,resume_html_p                  
,to_char(start_date,'YYYY-MM-DD') as start_date                     
,received_offer_letter_p        
,returned_offer_letter_p        
,signed_confidentiality_p       
,to_char(most_recent_review,'YYYY-MM-DD') as most_recent_review             
,most_recent_review_in_folder_p 
,featured_employee_approved_p   
,featured_employee_approved_by  
,featured_employee_blurb        
,featured_employee_blurb_html_p 
,referred_by from im_employee_info;



-- logic depends on this being null instead of 0 which seems to be the default
-- for the null string
create function im_projects_tr() returns opaque as '
begin
         if new.parent_id = '''' then
            new.parent_id := null;
         end if;
  return new;
end;
' language 'plpgsql';

create trigger im_projects_tr
before insert or update on im_projects
for each row execute procedure im_projects_tr();


create function im_info_tr() returns opaque as '
begin
         if new.supervisor_id = '''' then
            new.supervisor_id := null;
         end if;
  return new;
end;
' language 'plpgsql';

create trigger im_info_tr
before insert or update on im_employee_info
for each row execute procedure im_info_tr();


--- We record logged hours in the im_hours table.

create table im_hours (
    user_id          integer not null references users,
    on_what_id       integer not null,
    on_which_table   varchar(50),
    note             varchar(4000),
    day              datetime,
    hours            float8,
    billing_rate     float8,
    primary key(user_id, on_which_table, on_what_id, day)
);
create index im_hours_table_id_idx on im_hours(on_which_table, on_what_id);

create view im_hours_null as select
NULL::integer as user_id,
NULL::integer as on_what_id,
NULL::char as on_which_table,
NULL::char as note,
NULL::datetime as day,
NULL::float8 as hours,
NULL::float8 as billing_rate;

-- Offices - linked to user groups
create table im_offices (
	group_id               	integer primary key references user_groups,
	phone                   varchar(50),
	fax                     varchar(50),
	address_line1           varchar(80),
	address_line2           varchar(80),
	address_city            varchar(80),
	address_state           varchar(80),
	address_postal_code     varchar(80),
	address_country_code    char(2) references country_codes(iso),
	contact_person_id       integer references users,
	landlord                varchar(4000),
	--- who supplies the security service, the code for
	--- the door, etc.
	security                varchar(4000),
	note                    varchar(4000)
);


-- configure projects to have general comments
insert into table_acs_properties 
(table_name, section_name, user_url_stub, admin_url_stub) 
values ('im_projects','intranet','/admin/intranet/projects/view.tcl?group_id=','/admin/intranet/projects/view.tcl?group_id=');


create function salary_divisor(varchar) returns float8 as '
declare
	type_salary	alias for $1;
begin
	if type_salary = ''year'' then
	   return 12;
	else
	   return 1;
	end if;
end;
' language 'plpgsql';

create view im_monthly_salaries
as
select user_id, salary/salary_divisor(salary_period) as salary
from im_employee_info
where salary_period in ('year','month');

create function salary_multiplier(varchar) returns float8 as '
declare
	type_salary	alias for $1;
begin
	if type_salary = ''month'' then
	   return 12;
	else
	   return 1;
	end if;
end;
' language 'plpgsql';

create view im_yearly_salaries
as
select user_id, round(salary * salary_multiplier(salary_period),2) as salary
from im_employee_info
where salary_period in ('month','year');



-- We base our allocations, employee count, etc. around
-- a fundamental unit or block.
-- im_start_blocks record the dates these blocks
-- will start for this system.

create table im_start_blocks (
	start_block   datetime not null primary key,
	note          varchar(4000)
);


create table im_employee_percentage_time ( 
	start_block 	datetime references im_start_blocks, 
	user_id 	integer references users, 
	percentage_time integer, 
	note 		varchar(4000), 
	primary key (start_block, user_id) 
); 


-- tracks the money coming into a contract over time

create sequence im_project_payment_id_seq start 10000;

create table im_project_payments (
	payment_id   integer not null primary key,
	group_id	integer references im_projects,
	start_block	datetime references im_start_blocks,
	fee             float8,
	-- setup, monthly, monthly_hosting, hourly, stock, other
        fee_type        varchar(50),
	paid_p  	char(1) default 'f' check (paid_p in ('t','f')),
	due_date	      datetime,
	received_date	      datetime,
	note		      varchar(4000),
	last_modified           datetime not null,
 	last_modifying_user     integer not null references users,
	modified_ip_address     varchar(20) not null
);


create table im_project_payments_audit (
	payment_id   integer,
	group_id	integer references im_projects,
	start_block	datetime references im_start_blocks,
	fee             float8,
	-- setup, monthly, monthly_hosting, hourly, stock, other
        fee_type        varchar(50),
	paid_p  	char(1) default 'f' check (paid_p in ('t','f')),
	due_date	      datetime,
	received_date	      datetime,
	note		      varchar(4000),
	last_modified           datetime not null,
 	last_modifying_user     integer not null references users,
	modified_ip_address     varchar(20) not null,
	delete_p                char(1) default 'f' check (delete_p in ('t','f'))
);

create index im_proj_payments_aud_id_idx on im_project_payments_audit(payment_id);

create function im_project_payments_audit_tr() returns opaque as '
begin
                  insert into im_project_payments_audit (
                  payment_id,group_id, start_block, fee,
	          fee_type, paid_p, due_date, received_date, note, 
                  last_modified,
                  last_modifying_user, modified_ip_address
                  ) values (
                  old.payment_id, old.group_id, old.start_block, old.fee,
        	  old.fee_type, old.paid_p, old.due_date, 
                  old.received_date, 
		  old.note, old.last_modified,
                  old.last_modifying_user, old.modified_ip_address
                  );
                  return new;
end;
' language 'plpgsql';


create trigger im_project_payments_audit_tr
          before update or delete on im_project_payments
          for each row execute procedure im_project_payments_audit_tr();


--- im_allocations is used to do predictions and tracking based on
--- percentage of time/project. 

-- im_allocations does not have a separate audit
-- table because we want to take a snapshot of allocation 
-- at a chosed times.


create sequence im_allocations_id_seq;

create table im_allocations (
	--- allocation_id is not the primary key becase
	--- an allocation may be over several blocks of
	---  time.  We store a row per block.
	---  To answer the question "what is the allocation for
	--- this time block, query the most recent allocation
	--- for either that allocation_id or user_id.
	allocation_id    integer not null,
	group_id       integer not null references im_projects,
	-- this may be null because we will rows we need to store
	-- rows that are currently not allocated (future hire or
	-- decision is not made)
	user_id		 integer  references users,
	-- Allocations are divided up into blocks of time.
	-- Valid dates for start_block must be separated
	-- by the block unit.  For example, if your block unit
	-- was a week, valid start_block dates may be "Sundays"
	-- If the start_blocks don't align, reports get very difficult.
	start_block      datetime references im_start_blocks,
	percentage_time	 integer not null,
	note		 varchar(1000),
	last_modified           datetime not null,
        last_modifying_user     integer not null references users,
        modified_ip_address     varchar(20) not null
);
create index im_all_alloc_id_group_id_idx on im_allocations(allocation_id);
create index im_all_group_id_group_id_idx on im_allocations(group_id);
create index im_all_group_id_user_id_idx on im_allocations(user_id);
create index im_all_group_id_last_mod_idx on im_allocations(last_modified);


create table im_allocations_audit (
	allocation_id    integer not null,
	group_id       integer not null references im_projects,
	user_id		 integer  references users,
	-- Allocations are divided up into blocks of time.
	-- Valid dates for start_block must be separated
	-- by the block unit.  For example, if your block unit
	-- was a week, valid start_block dates may be "Sundays"
	-- If the start_blocks don't align, reports get very difficult.
	start_block      datetime references im_start_blocks,
	percentage_time	 integer not null,
	note		 varchar(1000),
	last_modified           datetime not null,
        last_modifying_user     integer not null references users,
        modified_ip_address     varchar(20) not null
);


--- we will put a row into the im_allocations_audit table if
--- a) another row is added with the same allocation_id and start_block
--- b) another row is added with the same user_id, group_id and start_block

create function im_allocations_audit_tr() returns opaque as '
begin
        insert into im_allocations_audit (
        allocation_id, group_id, user_id,  start_block, percentage_time,note, last_modified, last_modifying_user, modified_ip_address
 	) values (
        old.allocation_id, old.group_id, old.user_id,  old.start_block, old.percentage_time,old.note, old.last_modified, old.last_modifying_user, old.modified_ip_address);
  return new;
end;
' language 'plpgsql';


create trigger im_allocations_audit_tr
before update or delete on im_allocations
for each row execute procedure im_allocations_audit_tr();



create function get_start_week(datetime) returns datetime as '
declare
	v_start_date    alias for $1;
        dow             integer;
BEGIN
        dow := date_part(''dow'',v_start_date);
        return (v_start_date - dow)::datetime;
end;
' language 'plpgsql';


-- calculate the monthly fee for a given start_block and end_block


-- drop function im_projects_monthly_fee(integer,datetime,datetime);

-- you might be wondering why not use sum(fee) instead?
-- I couldn't get aggregates to work inside these functions with
-- postgres 6.5.3, so I hacked these up to at least get the same 
-- functionality.
create function im_projects_monthly_fee(integer,datetime,datetime) 
returns float8 as '
declare
	v_group_id      alias for $1;
	v_start_block   alias for $2;
	v_end_block     alias for $3;
        monthly_fee     float8;
        par_rec         record;
BEGIN
        monthly_fee = 0.0;
        for par_rec in select *
        from im_project_payments
        where im_project_payments.group_id = v_group_id
         and start_block >= v_start_block
         and start_block <= v_end_block
         and fee_type <> ''setup''
         and fee_type <> ''stock''
        loop
          monthly_fee := monthly_fee + par_rec.fee;
        end loop;
        return monthly_fee;
end;
' language 'plpgsql';


-- calulate the setup fee for a given start_block and end_block

-- drop function im_projects_setup_fee(integer,datetime,datetime);

-- you might be wondering why not use sum(fee) instead?
-- I couldn't get aggregates to work inside these functions with
-- postgres 6.5.3, so I hacked these up to at least get the same 
-- functionality.
create function im_projects_setup_fee(integer,datetime,datetime) 
returns float8 as '
declare
	v_group_id      alias for $1;
	v_start_block   alias for $2;
	v_end_block     alias for $3;
        monthly_fee     float8;
        par_rec         record;
BEGIN
        monthly_fee := 0.0;
        for par_rec in select * 
        from im_project_payments
        where im_project_payments.group_id = v_group_id
        and start_block >= v_start_block
        and start_block <= v_end_block
        and fee_type = ''setup''
        loop
          monthly_fee := monthly_fee + par_rec.fee;
        end loop;
        return monthly_fee;
end;
' language 'plpgsql';


-- calulate the stock for a given start_block and end_block

-- you might be wondering why not use sum(fee) instead?
-- I couldn't get aggregates to work inside these functions with
-- postgres 6.5.3, so I hacked these up to at least get the same 
-- functionality.

-- drop function im_projects_stock_fee(integer,datetime,datetime);
create function im_projects_stock_fee(integer,datetime,datetime) 
returns float8 as '
declare
	v_group_id      alias for $1;
	v_start_block   alias for $2;
	v_end_block     alias for $3;
        stock_fee      	float8;
        par_rec         record;
BEGIN
        stock_fee := 0.0;
        for par_rec in select *
        from im_project_payments
        where im_project_payments.group_id = v_group_id
        and start_block >= v_start_block
        and start_block <= v_end_block
        and fee_type = ''stock''
        loop
          stock_fee := stock_fee + par_rec.fee;
        end loop;
        return stock_fee;
end;
' language 'plpgsql';



create sequence im_partner_types_seq start 1;
create table im_partner_types (
	partner_type_id	        integer primary key,
	partner_type		varchar(100) not null unique,
	display_order		integer default 1
);

-- In what states can our projects be?
create sequence im_partner_status_id_seq start 1;
create table im_partner_status (
	partner_status_id	integer primary key,
	partner_status		varchar(100) not null unique,
	display_order		integer default 1
);


-- we store simple information about a customer
-- all contact information goes in the address book
create table im_partners (
	group_id 	 	integer primary key references user_groups,
	deleted_p        	char(1) default('f') constraint im_partners_deleted_p check(deleted_p in ('t','f')),
	partner_type_id         integer references im_partner_types,
	partner_status_id       integer references im_partner_status,
	primary_contact_id	integer references users,
	url			varchar(200),
	note			varchar(4000)
);


-- The various procedures. Note that user-groups don't really
-- work here because of the meta data we 
-- store (supervisor/certifier/long note)

create sequence im_procedures_procedure_id_seq start 1;

create table im_procedures (
    procedure_id            integer not null primary key,
    name                    varchar(200) not null,
    note                    varchar(4000),
    creation_date           datetime not null,
    creation_user           integer not null references users,
    last_modified           datetime,
    last_modifying_user     integer references users
);


-- Users certified to do a certain procedure

create table im_procedure_users (
    procedure_id        integer not null references im_procedures,
    user_id             integer not null references users,
    note                varchar(400),
    certifying_user     integer not null references users,
    certifying_date     datetime not null,
    primary key(procedure_id, user_id)
);

-- Occasions the procedure was done by a junior person,
-- under the supervision of a certified person

create sequence im_proc_event_id_seq;

create table im_procedure_events (
    event_id            integer not null primary key,
    procedure_id        integer not null references im_procedures,
    -- the person who did the procedure
    user_id             integer not null references users,
    -- the certified user who supervised
    supervising_user    integer not null references users,
    event_date          datetime not null,
    note                varchar(1000)
);



-- Now the pls definitions

-- Some helper functions to make our queries easier to read
create function im_proj_type_from_id(integer) returns varchar as '
declare
	v_project_type_id alias for $1;
  v_project_type    varchar;
BEGIN
  select project_type into v_project_type from im_project_types where project_type_id = v_project_type_id;
  return v_project_type;
END;
' language 'plpgsql';


create function im_proj_status_from_id(integer) returns varchar as '
declare
	v_project_status_id alias for $1;
  v_project_status    varchar;
BEGIN
  select project_status into v_project_status from im_project_status where project_status_id = v_project_status_id;
  return v_project_status;
END;
' language 'plpgsql';

-- couldn't find out what this statement does. commented out for now.
-- exception when others then null;
create function im_proj_url_from_type(integer,varchar) returns varchar as '
declare
	v_group_id      alias for $1;
	v_url_type      alias for $2;
        v_url 		varchar;
BEGIN
  begin
        select url into v_url 
          from im_url_types, im_project_url_map
         where group_id = v_group_id
           and im_url_types.url_type_id = im_project_url_map.url_type_id
           and url_type = v_url_type;
  end;
  return v_url;
END;
' language 'plpgsql';

create function im_cust_status_from_id(integer) returns varchar as '
declare
	v_customer_status_id alias for $1;
  v_customer_status    varchar;
BEGIN
  select customer_status into v_customer_status from im_customer_status where customer_status_id = v_customer_status_id;
  return v_customer_status;
END;
' language 'plpgsql';


--- Define an administration group for the Intranet

select administration_group_add('Intranet Administration', 'intranet', 'intranet', '', 'f', '/admin/intranet/');


-- Create a group type of intranet. Does nothing if the group type is already defined


create function create_group_type_intranet() returns bool as '
declare
 n_system_group_types	integer;
begin
 select count(*) into n_system_group_types from user_group_types where group_type = ''intranet'';
 if n_system_group_types = 0 then 
   -- create the group type
   insert into user_group_types
     (group_type, pretty_name, pretty_plural, approval_policy, default_new_member_policy, group_module_administration)
   values
     (''intranet'', ''Intranet'', ''Intranet Groups'', ''closed'', ''closed'', ''full'');

 end if;
 return ''t'';
end;
' language 'plpgsql';

select create_group_type_intranet();

-- stuff we need for the Org Chart
-- Oracle will pop a cap in our bitch ass if do CONNECT BY queries 
-- on im_users without these indices

create index im_employee_info_idx1 on im_employee_info(user_id, supervisor_id);
create index im_employee_info_idx2 on im_employee_info(supervisor_id, user_id);

-- you can't do a JOIN with a CONNECT BY so we need a PL/SQL proc to
-- pull out user's name from user_id

create function im_name_from_user_id(integer) returns varchar as '
declare
	v_user_id	alias for $1;
	v_full_name	varchar(8000);
BEGIN
  select first_names || '' '' || last_name into v_full_name 
   from users 
   where user_id = v_user_id;
  return v_full_name;
end;
' language 'plpgsql';


-- couldn't find a place where this was actually being used, so I didn't 
-- convert it.
create function im_supervises_p(integer,integer) returns varchar as '
declare
	v_supervisor_id alias for $1;
	v_user_id	alias for $2;
	v_exists_p	char;
BEGIN
  select (case when count(*) = 0 then ''f'' else ''t'' end) into v_exists_p
   from im_employee_info
   where user_id = v_user_id
   and level > 1
   start with user_id = v_supervisor_id
   connect by supervisor_id = PRIOR user_id;
   return v_exists_p;
end;
' language 'plpgsql';


-- drop function im_projects_gen_key(integer);
create function im_projects_gen_key(integer) returns varchar as '
declare
        id      alias for $1;
        str     varchar;
        len     integer;
begin
        str := ''000000'' || text(id);
        len := char_length(str);
        return substr(str,len - 5);
end;
' language 'plpgsql';


create function im_projects_connect_by(integer) returns varchar as '
declare
	id              alias for $1;
        parentid        integer;
BEGIN
        select into parentid parent_id from im_projects where id = group_id;
	IF parentid is null or parentid = ''''
	THEN 
		return im_projects_gen_key(id);
	ELSE
		return im_projects_connect_by(parentid) || ''/'' || im_projects_gen_key(id);
	END IF;	
END;
' language 'plpgsql';


create function im_projects_level(integer,integer) returns integer as '
declare
	id              alias for $1;
        count           alias for $2;
	parentid	integer;
BEGIN
        select into parentid parent_id
                from im_projects
		where group_id = id;

	IF parentid is null or parentid = ''''
	THEN 
		return count;
	ELSE
		return im_projects_level(parentid,count+1);
	END IF;	
END;
' language 'plpgsql';


create view im_projects_view as 
select p.*, im_projects_level(p.group_id,1) as level,
       im_projects_connect_by(p.group_id) as connect_by_key
from im_projects p;


-- set sql "select user_group_name_from_id(group_id) as project_name,
--                 group_id as project_id, level
--            from im_projects p
--           where exists (select 1 
--                           from user_group_map ugm 
--                        where p.group_id=ugm.group_id and ugm.user_id=$user_id)
--      connect by prior group_id=parent_id
--      start with parent_id is null"

-- select user_group_name_from_id(group_id) as project_name, 
--                group_id as project_id, level
--           from im_projects_view p
--          where exists (select 1 from user_group_map ugm 
--                         where p.group_id=ugm.group_id 
--                           and ugm.user_id=$user_id)
--       order by connect_by_key
--
--


create function im_org_chart_connect_by(integer,integer) returns varchar as '
declare
	id              alias for $1;
        start           alias for $2;
        parentid        integer;
BEGIN
        select into parentid supervisor_id from im_employee_info 
        where id = user_id;

	IF parentid is null or parentid = start or parentid = ''''
	THEN 
		return im_projects_gen_key(id);
	ELSE
		return im_org_chart_connect_by(parentid,start) || ''/'' || im_projects_gen_key(id);
	END IF;	
END;
' language 'plpgsql';


create function im_org_chart_level(integer,integer,integer) 
returns integer as '
declare
	id              alias for $1;
        count           alias for $2;
        start           alias for $3;
	parentid	integer;
BEGIN
        select into parentid supervisor_id
                from im_employee_info
		where user_id = id;

	IF parentid is null or parentid = start or parentid = ''''
	THEN 
		return count;
	ELSE
		return im_org_chart_level(parentid,count+1,start);
	END IF;	
END;
' language 'plpgsql';


--  SCC: somebody check the plpgsql...
-- drop function im_project_ticket_project_id(integer);
create function im_project_ticket_project_id(integer)
returns integer as '
declare
	v_group_id	alias for $1;
	v_project_id	ticket_projects.project_id%TYPE;
BEGIN
  BEGIN
    select project_id into v_project_id from ticket_projects where customer_id = v_group_id;

  END;
  if v_project_id is null then
     return 0;
  else
     return v_project_id;
  end if;
END;
' language 'plpgsql';



-- a bunch of stuff for outer joins (BMA)
create function im_employee_salary (integer)
returns numeric as '
DECLARE
	v_user_id alias for $1;
BEGIN
	return salary from im_monthly_salaries where user_id= v_user_id;
END;
' language 'plpgsql';

create function im_employee_n_years_experience (integer)
returns numeric as '
DECLARE
	v_user_id alias for $1;
	v_experience numeric;
BEGIN
	select (date_part(''year'',age(sysdate(),info.first_experience)) + date_part(''month'',age(sysdate(),info.first_experience))/12)::numeric into v_experience from im_employee_info info where user_id= v_user_id;
	if v_experience is null
	then return 0::numeric;
	else return v_experience;
	end if;
END;
' language 'plpgsql';

create function im_employee_referals(integer) returns integer as '
declare
        id      alias for $1;
        cnt     integer;
begin
        select into cnt count(1)
          from im_employee_info
         where referred_by = id;
         return cnt;
end;
' language 'plpgsql';


-- drop function mod_days(integer);
create function mod_days(integer) returns integer as '
declare
        day     alias for $1;
begin
        if day >= 7 then
           return day - 7;
        else if day < 0 then
           return day + 7;
        else
           return day;
        end if;
        end if;
end;
' language 'plpgsql';

-- drop function delta_days(integer,varchar);
create function delta_days(integer,varchar) returns integer as '
declare
        day_in          alias for $1;
        pattern         alias for $2;
        v_day           integer;
        d_days          integer;
        v_pat           varchar;
begin
        select into v_pat substr(upper(pattern),1,3);
        select into v_day (case when v_pat = ''SUN'' then 0 else
                           case when v_pat = ''MON'' then 1 else
                           case when v_pat = ''TUE'' then 2 else
                           case when v_pat = ''WED'' then 3 else
                           case when v_pat = ''THU'' then 4 else
                           case when v_pat = ''FRI'' then 5 else
                           case when v_pat = ''SAT'' then 6
                           end end end end end end end);
                             
       d_days := mod_days((v_day - day_in) + 7);
       if d_days = 0 then 
          return 7;
       else
          return d_days;
       end if;
end;
' language 'plpgsql';


-- drop function next_day(datetime,varchar);
create function next_day(datetime,varchar) returns datetime as '
declare
       the_date      alias for $1;
       pattern       alias for $2;
       dow           integer;
begin
       dow    := date_part(''dow'',the_date);
       return the_date + delta_days(dow,pattern); 
end;
' language 'plpgsql';

create function trunc(timespan) returns varchar as '
declare
	inspan          alias for $1;
begin
        return date_part(''day'',inspan);
end;
' language 'plpgsql';

-- drop function julian_zero_date();
create function julian_zero_date() returns datetime as '
begin
	return ''4714-11-24 BC''::datetime;
end;
' language 'plpgsql';

-- drop function julian_date(datetime);
create function julian_date(datetime) returns varchar as '
declare
       indate    alias for $1;
begin
        return trunc(indate - julian_zero_date());
end;
' language 'plpgsql';

-- drop function date_from_julian(integer);
create function date_from_julian(integer) returns datetime as '
declare
                jdate   alias for $1;
begin
                return julian_zero_date() + jdate;
end;
' language 'plpgsql';



-- Populate all the status/type/url with the different types of 
-- data we are collecting
\i intranet-population.sql


-- drop function im_load_start_blocks(timestamp,integer,varchar);
create function im_load_start_blocks(timestamp,integer,varchar) 
returns integer as '
declare
        start_date      alias for $1;
        number_of_weeks alias for $2;
        comment         alias for $3;
        the_date        timestamp;
begin
        the_date := next_day(start_date,''SUNDAY'');
        for i in 0 .. number_of_weeks loop
            insert into im_start_blocks (start_block,note)
                                 values (the_date,comment);
            the_date := the_date + 7;
        end loop;            
        return 0;
end;
' language 'plpgsql';



-- Load the im_start_blocks.  This is setup to provide start blocks from 6 
-- months in the past to 100 years in the future.  Edit the start_date and 
-- the number_of_weeks if you want something different.  We're creating a Y2100
-- problem here, but unix has a Y2047? problem already, so we're safe :).

-- BMA: 100 years? That makes for ridiculous drop down boxes. I think having 2 years is plenty.
-- The start blocks can be recalculated every now and then.
-- select im_load_start_blocks(sysdate() - 26*7,52*100 + 26,'Week');

select im_load_start_blocks(sysdate() - 26*7,52*2 + 26,'Week');


-- More Pgsql for Postgres (BMA)
create function im_hours_group_id_valid_p(integer, integer, char, integer)
returns char
as '
DECLARE
	v_group_id alias for $1;
	v_user_id alias for $2;
	v_on_which_table alias for $3;
	v_julian_date alias for $4;
	v_check integer;
BEGIN
	select count(*) into v_check from user_group_map where user_id= v_user_id and group_id= v_group_id;

	if v_check > 0
	then return ''t'';
	end if;

	select count(*) into v_check from im_hours where 
	user_id= v_user_id and
	on_which_table= v_on_which_table and
	(im_hours.hours is not null or im_hours.note is not null) and day= date_from_julian(v_julian_date)
	and on_what_id = v_group_id;

	if v_check > 0
	then return ''t'';
	else return ''f'';
	end if;
END ;
' language 'plpgsql';
