-- This is a hack to allow a quick and dirty search implementation for OpenACS.
-- It's based on the PL/SQL function 'bboard_contains' originally used to search
-- photo.net's bulletin boards back in the pre-Context/Intermedia era.

-- The return value ranges from 0-100, which is computed according to the
-- algorithm du jour, currently just a simple keyword-type search.

-- This will be slow, requiring a sequential scan - making an index on a date
-- and restricting the search might be a good idea on big systems until we
-- get a better search.  At least Pg/Tcl is smart enough to save the 8.*
-- compiled bytes, which AOLserver is not for .tcl scripts!

-- If you want things like email, etc to be searched along with a message
-- then pass the concatenation to this routine.

-- Remember this shouldn't be called with NULL so exclude those in your
-- search query.

-- Note: you must have Pg/Tcl installed to run this.  If you've not
-- built Tcl support into Postgres, reconfigure with "--with-tcl" and
-- rebuild and reinstall.

-- Then enable Pg/Tcl for your database by running
-- "createlang pgtcl <your database name>"

-- The odd "increment by 10" code comes from the original PL/SQL, presumably
-- to allow one some flexibility in assigning weights if you want to 
-- customize this function.

create function rank_for_search(varchar, varchar) returns integer as '

    set search_words [split [string tolower $1] " "]
    set candidate_string [string tolower $2]

    set max_possible_score [expr [llength $search_words] * 10.0]
    set score 0
    foreach search_word $search_words {
        if { [string first $search_word $candidate_string] != -1 } {
            set score [expr $score+10]
        }
    }
    set score [expr int(($score/$max_possible_score) * 100.0)]
    return $score
' language 'pltcl';
