# order-view.tcl,v 1.2.2.2 2000/02/03 09:49:34 ron Exp
# we expect to get one argument:  order_id

set order_id [ns_queryget order_id]

validate_integer_or_null order_id $order_id

if { $order_id == "" } {
    ns_return 200 text/html "[ad_header "No order id"]

<h2>No order id</h2>

specified for [ad_system_name]

<hr>

Order id needed.  This request came in with no
order id.

[ad_footer]
"
    return
}

set db [ns_db gethandle]

set selection [ns_db 0or1row $db "select 
u.user_id,
a.activity_id, a.short_name, 
o.order_state, 
o.need_hotel_p, o.need_plane_p, o.need_car_p,
o.canceled_p,
to_char(o.confirmed_date,'YYYY-MM-DD HH24:MI:SS') as confirmed_date, 
u.first_names, u.last_name, u.email, uc.home_phone, 
uc.ha_line1, uc.ha_line2, 
uc.ha_city, uc.ha_state, 
uc.ha_postal_code, uc.ha_country_code,
o.attending_reason, o.where_heard, o.comments
from evreg_orders o, evreg_activities a, evreg_events e,
users u, users_contact uc
where order_id = $order_id
and a.activity_id = e.activity_id
and e.event_id = o.event_id
and u.user_id = o.user_id
and uc.user_id = o.user_id
"]

if { $selection == "" } {
     ns_return 200 text/html "[ad_header "Could not find order"]

<h2>Could not find order</h2>

in [ad_system_name]

<hr>

Order $order_id was not found in the database.

[ad_footer]
"
    return
}

# we have a valid order id

set_variables_after_query

if {$canceled_p == "t"} {
    set canceled_text "(canceled)"
} else {
    set canceled_text ""
}

set short_name [database_to_tcl_string $db "select short_name from evreg_activities where activity_id='$activity_id'"]

# we have all the description info from the RDBMS

ReturnHeaders

ns_write "[ad_header "Order # $order_id $canceled_text" ]

<h2>Order # $order_id $canceled_text</h2>
[ad_context_bar_ws [list "index.tcl" "Events Administration"] [list "order-history.tcl" "Order History"] "Order"]

<hr>

<table width=100%>
<tr><td valign=top>

<h3>Activity Information</h3>
<table>
<tr>
 <td>Activity #</td>
 <td>$activity_id ($short_name)</td>
"


ns_write "
</table>

<h3>Order Information</h3>
<table>
<tr>
 <td>Order State</td>
 <td>$order_state
<tr>
 <td>Confirmed Date</td>
 <td>$confirmed_date</td>
"

ns_write "
</table>

</td><td valign=top>

<h3>Customer Information</h3>

<table>
<tr>
 <td>Name
 <td><a href=\"order-same-person.tcl?[export_url_vars user_id]\">$first_names $last_name</a></td>
<tr>
 <td>Email
 <td><a href=\"mailto:$email\">$email</a>
<tr>
 <td>Telephone number
 <td>$home_phone
<tr>
  <td>Address
  <td>$ha_line1
"
if {$ha_line2 != ""} {
    ns_write "<tr><td>&nbsp;<td>$ha_line2"
}

ns_write "
<tr>
  <td>&nbsp;</td>
  <td>$ha_city, $ha_state $ha_postal_code"

if { [info exists ha_country_code] && $ha_country_code != "" && $ha_country_code != "us"} {
    set ha_country_name [database_to_tcl_string $db "select country_name from ad_country_codes where iso='$country_code'"]
    ns_write "&nbsp;&nbsp;&nbsp;&nbsp;$ha_country_name"
}

ns_write "
<tr>
  <td valign=top>Attending reason
  <td>$attending_reason
<tr>
  <td valign=top>Heard from
  <td>$where_heard
"

set needs_text "<tr><td valign=top>Needs<td><ul>"
if {$need_hotel_p == "t"} {
    append needs_text "<li>Needs Hotel"
}
if {$need_car_p == "t"} {
    append needs_text "<li>Needs a car"
}
if {$need_plane_p == "t"} {
    append needs_text "<li>Needs a plane ticket"
}

append needs_text "</ul></table>"

ns_write $needs_text

if { [info exists comments] && $comments != ""} {
    ns_write "
    <tr>
     <td colspan=2>
     <h3>Comments</h3>
     <pre>$comments</pre>
    "
}

ns_write "
</table>

<h2>Order Maintenance</h2>

<ul>
<li><a href=\"order-comments.tcl?order_id=$order_id\">Add/Edit Comments</a>
"
if {[empty_string_p $canceled_text]} {
    ns_write "<li>
    <a href=\"order-cancel.tcl?order_id=$order_id\">Cancel Order</a>"
}


ns_write "
</ul>

<hr>
<h2>All info in evreg_orders table regarding order # $order_id</h2>
This is probably only worth looking at if there's a problem with the order.
<p>
"

set selection [ns_db 0or1row $db "select order_id,
event_id, user_id, paid_p,
confirmed_date, ip_address, order_state, org, title_at_org,
attending_reason, where_heard,
need_hotel_p, need_car_p,
need_plane_p, comments, canceled_p
from evreg_orders
where order_id = $order_id"]

set_variables_after_query

ns_write "
<table>
<tr>
 <td>order_id
 <td>$order_id
<tr>
 <td>event_id
 <td>$event_id
<tr>
 <td>user_id
 <td>$user_id
<tr>
 <td>paid_p
 <td>$paid_p
<tr>
 <td>ip_address
 <td>$ip_address
<tr>
 <td>order_state
 <td>$order_state
<tr>
 <td>org
 <td>$org
<tr>
 <td>title_at_org
 <td>$title_at_org
<tr>
 <td>attending_reason
 <td>$attending_reason
<tr>
 <td>where_heard
 <td>$where_heard
<tr>
 <td>need_hotel_p
 <td>$need_hotel_p
<tr>
 <td>need_car_p
 <td>$need_car_p
<tr>
 <td>need_plane_p
 <td>$need_plane_p
<tr>
 <td>comments
 <td>$comments
<tr>
 <td>canceled_p
 <td>$canceled_p
</table>
[ad_footer]
"











