# group-add-2.tcl,v 1.3.4.1 2000/02/03 09:53:37 ron Exp
# UI for selecting a particular role once a group has been selected

ad_page_variables {
    permissions_id
    group_id
    {page_url {}}
    {page_name {}}
    {return_url {}}
    {return_name {}}
}

validate_integer permissions_id $permissions_id
validate_integer group_id $group_id

set local_user_id [ad_verify_and_get_user_id]
set local_group_id [ad_get_group_id]
set db [ns_db gethandle]

# check permissions
if { ![ad_g_owner_p $db $permissions_id $local_user_id $local_group_id] } {
    ad_return_error "Unauthorized" "You are not authorized to edit the permissions for this page.<p>"
    return
}

# first check to see if we need to do roles
set multi_role_p [database_to_tcl_string $db "select multi_role_p from user_groups
                                              where group_id = $group_id"]

if { $multi_role_p == "f" } {
    ns_returnredirect "group-add-3.tcl?[export_url_vars permissions_id page_url page_name return_url return_name group_id]"
}

# first, check to see if the "all" role  has been added
set null_role_p [database_to_tcl_string_or_null $db "select 1 from permissions_ug_map
                                                     where permissions_id = $permissions_id
                                                     and group_id = $group_id
                                                     and role is null"]

# get all roles for a particular group that aren't in pum
set selection [ns_db select $db "select * from user_group_roles ugr
                                 where group_id = $group_id
                                 and not exists (select 1 from permissions_ug_map pum
                                                 where permissions_id = $permissions_id
                                                 and ugr.role = pum.role
                                                 and pum.group_id = $group_id)"]

if { $selection == "" && $null_role_p == 1 } {
    ns_returnredirect "group-add-3.tcl?[export_url_vars permissions_id page_url page_name return_url return_name group_id]"
}

# show the UI. we have roles we want the user to choose.

set edit_perm_url edit-page-permissions.tcl?[export_url_vars permissions_id page_url page_name return_url return_name]
set group_edit_url group-add.tcl?[export_url_vars permissions_id page_url page_name return_url return_name]

if { [info exists return_name] && ![empty_string_p $return_name] && [info exists return_url] && ![empty_string_p $return_url] } {
    set navbar "[ad_context_bar_ws_or_index [list "$return_url" "$return_name"] [list "$edit_perm_url" "Edit Permissions"] [list "$group_edit_url" "Add a Group"] "Choose a Role"]"
} else { 
    set navbar "[ad_context_bar_ws_or_index [list "$page_url" "$page_name"] [list "$edit_perm_url" "Edit Permissions"] [list "$group_edit_url" "Add a Group"] "Choose a Role"]"
}

ReturnHeaders
ns_write "
[ad_header "Choose a Role"]

<h2>Choose a Role</h2>
$navbar
<hr>
Choose a role:
<p>

<table><tr><td>
<ul>
"

set count 0
while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    ns_write "<li><a href=\"group-add-3.tcl?[export_url_vars permissions_id group_id role page_url page_name return_url return_name]\">$role</a>\n"
    incr count
}

if { $count > 0 } {
    ns_write "
    </ul>
    </td>"
}

# if we want to give permissions to all roles, we don't pass role
if { $null_role_p == "" } {
    if { $count > 0 } {
	ns_write "<td><ul><li><a href=\"group-add-3.tcl?[export_url_vars permissions_id group_id page_url page_name return_url return_name]\">all</a></ul></td>\n"
    } else {
	ns_write "<li><a href=\"group-add-3.tcl?[export_url_vars permissions_id group_id page_url page_name return_url return_name]\">all</a>\n"
    }
}


ns_write "
</tr>
</table>
<p>
[ad_footer]"
