# search.tcl,v 1.2.4.1 2000/02/03 09:53:51 ron Exp
# (stolen from admin/users/search.tcl)

# Reusable page for searching for a user by email or last_name.
# Returns to "target" with user_id_from_search, first_names_from_search,
# last_name_from_search, and email_from_search, and passing along all
# form variables listed in "passthrough".

set_the_usual_form_variables

# email or last_name (search strings)
# also accept "keyword" (for looking through both)
# target (URL to return to)
# passthrough (form variables to pass along from caller)


# Check input.
set exception_count 0
set exception_text ""

# from one of the user pages
if { (![info exists email] || $email == "") && (![info exists last_name] || $last_name == "") } {
    incr exception_count
    append exception_text "<li>You must specify either an email address or last name to search for.\n"
}

if { [info exists email] && [info exists last_name] && $email != "" && $last_name != "" } {
    incr exception_count
    append exception_text "<li>You can only specify either email or last name, not both.\n"
}

if { ![info exists target] || $target == "" } {
    incr exception_count
    append exception_text "<li>Target was not specified. This shouldn't have happened,
    please contact the <a href=\"mailto:[ad_host_administrator]\">administrator</a>
    and let them know what happened.\n"
}

if { $exception_count != 00 } {
    ad_return_complaint $exception_count $exception_text
    return
}

if { [info exists keyword] } {
    set search_clause "lower(email) like '%[string tolower $keyword]%' or lower(first_names || ' ' || last_name) like '%[string tolower $keyword]%'"
    set search_text "name or email matching \"$keyword\""
} elseif { [info exists email] && $email != "" } {
    set search_text "email \"$email\""
    set search_clause "lower(email) like '%[string tolower $email]%'"
} else {
    set search_text "last name \"$last_name\""
    set search_clause "lower(last_name) like '%[string tolower $last_name]%'"
}

set edit_perm_url "edit-page-permissions.tcl?[export_url_vars permissions_id page_url page_name return_url return_name]"
set user_add_url "user-add.tcl?[export_url_vars permissions_id page_url page_name return_url return_name]"

if { [info exists return_name] && ![empty_string_p $return_name] && [info exists return_url] && ![empty_string_p $return_url] } {
    set navbar "[ad_context_bar_ws_or_index [list "$return_url" "$return_name"] [list "$edit_perm_url" "Edit Permissions"] [list "$user_add_url" "Add a User"] "User Search for $search_text"]"
} else { 
    set navbar "[ad_context_bar_ws_or_index [list "$page_url" "$page_name"] [list "$edit_perm_url" "Edit Permissions"] [list "$user_add_url" "Add a User"] "User Search for $search_text"]"
}

if { ![info exists passthrough] } {
    set passthrough_parameters ""
} else {
    set passthrough_parameters "&[export_entire_form_as_url_vars $passthrough]"
}

# append some sql to remove users already in this permission record
append search_clause " and not exists (select 1 from permissions_ug_map
                                       where permissions_id = $permissions_id
                                       and users.user_id = permissions_ug_map.user_id)"


set db [ns_db gethandle]
set selection [ns_db select $db "select user_id as user_id_from_search, 
  first_names as first_names_from_search, last_name as last_name_from_search,
  email as email_from_search, user_state
from users
where $search_clause"]

ReturnHeaders

ns_write "[ad_admin_header "User Search"]
<h2>User Search</h2>
$navbar
<hr>
<ul>
"

set i 0

set user_items ""

while { [ns_db getrow $db $selection] } {
    set_variables_after_query

    append user_items "<li><a href=\"$target?[export_url_vars user_id_from_search first_names_from_search last_name_from_search email_from_search]$passthrough_parameters\">$first_names_from_search $last_name_from_search ($email_from_search)</a>\n"
    incr i
    if { $user_state != "authorized" } {
	set user_finite_state_links  [ad_registration_finite_state_machine_admin_links $user_state $user_id_from_search]
	append user_items "<font color=red>$user_state</font> [join $user_finite_state_links " | "] \n"
    }
}

if { $i == 0 } {
    ns_write "<li>No users found that can be added to this permissions record.\n"
} else {
    ns_write $user_items
}

ns_write "</ul>
[ad_admin_footer]
"

