set_form_variables 0

ad_page_variables {
    {user_id ""}
    {project_id ""}
    {order_by "ticket_issues.msg_id"}
    {view_assignment all}
    {view_status open}
    {view_created all}
    {view_type all}
}

# form vars:
# 
# user_id - user_id this page is for (optional - defaults to logged in user)
# project_id - project_id for this user (optional - defaults to all tickets 
#              for the selected user)
#
# filter conditions
#

# logged_in_user_id corresponds to the user that is viewing this page
# and user_id is the user_id for the person whom we want to see a summary of 
# ticket issues.

if { [info exists user_id] && ![empty_string_p $user_id]} {
    # validate the user_id input and get the user_id of the person viewing this
    # page.
    validate_integer user_id $user_id
    set logged_in_user_id [ad_get_user_id]
} else {
    # a user_id was not passed to this page so just default to showing the user
    # his or her own tickets.
    set user_id [ad_get_user_id]
    set logged_in_user_id $user_id
}

if {![info exists project_id]} {
    set project_id ""
}

if {![empty_string_p $project_id]} {
    validate_integer project_id $project_id
    set original_project_id $project_id
} else {
    set original_project_id ""
}

set db [ticket_getdbhandle]

if {![empty_string_p $project_id]} {
    set project_title [database_to_tcl_string $db "select title from ticket_projects where project_id=$project_id"]
    set project_title_html "for project $project_title"
} else {
    set project_title_html "for all projects"
}

set list_of_projects [database_to_tcl_list_list $db "select project_id,title from ticket_projects where ticket_user_can_see_project_p($logged_in_user_id, project_id)='t'"]

set list_of_projects [concat [list [list "" "(all)"]] $list_of_projects]

set sql "select user_id, first_names || ' ' || last_name as name
from users where user_id in (select user_id from user_group_map where group_id in 
(select team_id from ticket_project_teams where ticket_user_can_see_project_p($logged_in_user_id, project_id)='t'))
order by upper(last_name)"

set list_of_users [database_to_tcl_list_list $db $sql]

# Status
# view_status  { open closed deferred created_by_you }
#
# Creation time
# view_created { last_24 last_week last_month all}
#
# Ticket Type
# view_type { all defects enhancements issues }
#
#
# order_by       column name to sort table by 

set ctrlvars {view_type view_assignment view_status view_created user_id project_id}

set name [database_to_tcl_string $db "select first_names || ' ' || last_name 
from users where user_id = $user_id"]

ReturnHeaders

ns_write "[ad_header "[ticket_system_name] Home"]

<h2>Tickets created by $name $project_title_html</h2>

[ad_context_bar_ws_or_index [list "index.tcl" "Ticket Tracker"] "One User"]

<hr>
"

# List of form vars used to select tickets to display

# Assignment filter conditions
#

set filter_vars {view_assignment view_status view_created view_type order_by project_id user_id}
set url "by-creation-user.tcl"
    
if {[string match "project*" $order_by]} {
    set order_by "project_title"
}

switch $view_assignment {
    "user" { 
	set assignment_filter "and (exists (select msg_id from ticket_issue_assignments 
	where ticket_issue_assignments.msg_id = ticket_issues.msg_id
	and ticket_issue_assignments.user_id = $user_id)) " 
    }
    
    "all" { set assignment_filter "" }
    
    "unassigned" {
	set assignment_filter "and (not exists (select msg_id from ticket_issue_assignments
	where ticket_issue_assignments.msg_id= ticket_issues.msg_id))"
    }
}

switch $view_status {
    "open" {
	set status_filter "and ((status <> 'closed') and (status <> 'deferred') and (status <> 'fixed waiting approval')) " 
    }
    "deferred" {
	set status_filter "and ((status <> 'closed') or (status = 'deferred')) " 
    }
    "closed" {
	set status_filter "" 
	set closed 1
	set view_closed 1
    }
}

switch $view_created {
    "last_24"   { set date_filter "and (posting_time > (sysdate() - '1 day'::reltime))" } 
    "last_week" { set date_filter "and (posting_time > (sysdate() - '7 days'::reltime)) " } 
    "last_month" { set date_filter "and (posting_time > (sysdate() - '30 days'::reltime)) " }
    "all" { set date_filter "" }
}

switch $view_type {
    "all" { set issue_table "ticket_issues" }
    "defects" { set issue_table "ticket_defects" }
    "enhancements" { set issue_table "ticket_enhancements" }
    "issues" { set issue_table "ticket_issues_issues" }
}

append page "<table border=0 cellspacing=0 cellpadding=0 width=100%>
<tr>
<th bgcolor=#ECECEC><font size=-1>Ticket Type</font></th>
<th bgcolor=#ECECEC><font size=-1>Ticket Assignment</font></th>
<th bgcolor=#ECECEC><font size=-1>Status</font></th>
<th bgcolor=#ECECEC><font size=-1>Creation Time</font></th></tr>"


#### Assignment flags
# Show assigned to you 
append page "<tr><td align=center><font size=-1><nobr>\["

# Issue type filter
append page [ticket_control_vars view_type all $filter_vars "ALL" "$url"]
append page " | "
append page [ticket_control_vars view_type defects $filter_vars "Defects" "$url"]
append page " | "
append page [ticket_control_vars view_type enhancements $filter_vars "Enhancements" "$url"]
append page " | "
append page [ticket_control_vars view_type issues $filter_vars "Issues" "$url"]

append page "\]</nobr></font></td><td align=center><nobr><font size=-1>\["

append page [ticket_control_vars view_assignment user $filter_vars "mine" "$url"]
append page " | "
# Show all tickets 
append page [ticket_control_vars view_assignment all $filter_vars "everyone's" "$url"]

# Depending on project_id
if {![empty_string_p $project_id]} {
    set unassigned_count [database_to_tcl_string $db "select count(*) from $issue_table where project_id=$project_id and ticket_n_assigned(msg_id)=0 $status_filter $date_filter"]
} else {
    set unassigned_count [database_to_tcl_string $db "select count(*) from $issue_table where ticket_user_can_see_project_p($user_id, project_id)='t' and ticket_n_assigned(msg_id)=0 $status_filter $date_filter"]
}

append page " | "

append page [ticket_control_vars view_assignment unassigned $filter_vars "unassigned" "$url"]

#### Status flags
append page "\]</nobr></font></td>\n<td align=center><font size=-1><nobr>\["

# Show open issues
append page [ticket_control_vars view_status open $filter_vars "active" "$url"]
append page " | "
# Show deferred issues
append page [ticket_control_vars view_status deferred $filter_vars "+deferred" "$url"]
append page " | "
# Show closed issues
append page [ticket_control_vars view_status closed $filter_vars "+closed" "$url"]

#### Creation time filter
append page "\]</nobr></font></td>\n<td align=center><font size=-1><nobr>\["

append page [ticket_control_vars view_created last_24 $filter_vars "last 24 hrs" "$url"]
append page " | "
append page [ticket_control_vars view_created last_week $filter_vars "last week" "$url"]
append page " | "
append page [ticket_control_vars view_created last_month $filter_vars "last month" "$url"]
append page " | "
append page [ticket_control_vars view_created all $filter_vars "all" "$url"]

append page "\]</nobr></font></td></tr></table><p>"

ns_write "$page"

# Project Filter
if {![empty_string_p $project_id]} {
    set project_filter " and ticket_projects.project_id=$project_id"
} else {
    set project_filter ""
}

###################################################################
#
# Now display the tickets pertaining the selected user and project
#
###################################################################

set query "select
 ticket_issues.msg_id,
 ticket_issues.ticket_type, 
 ticket_issues.one_line,
 ticket_issues.status,
 ticket_issues.severity,
 ticket_issues.posting_time,
 ticket_projects.title as project_title,
 ticket_projects.project_id,
 ticket_issues.priority, 
 to_char(ticket_issues.modification_time, 'MM/DD/YY') as modification_time_pretty,
 to_char(ticket_issues.posting_time, 'MM/DD/YY') as creation_time_pretty,
 to_char(ticket_issues.close_date, 'MM/DD/YY') as close_date_pretty,
 to_char(ticket_issues.close_date, 'MM/DD/YY') as close_date,
 to_char(ticket_issues.deadline, 'MM/DD/YY') as deadline_pretty,
 to_char(ticket_issues.deadline, 'MM/DD/YY') as deadline,
 sysdate() - deadline as pastdue,
 ticket_assignees(ticket_issues.msg_id) as ticket_assignees
from $issue_table ticket_issues, ticket_projects
where ticket_user_can_see_project_p($logged_in_user_id, ticket_projects.project_id)='t'
and ticket_issues.user_id= $user_id
and ticket_projects.project_id = ticket_issues.project_id
$project_filter
$assignment_filter
$status_filter
$date_filter
order by $order_by, ticket_issues.priority, ticket_issues.posting_time"

set this_url "by-creation-user.tcl"
append results [ticket_summary_display]

ns_write  "$results

<FORM method=GET action=by-creation-user.tcl>Project: 
[make_html_select project_id $list_of_projects $original_project_id] 
User: 
[make_html_select user_id $list_of_users $user_id]
<INPUT TYPE=submit value=go></FORM><p><p>

[ad_footer]
"

