# User is changing status this ticket.
# An explanatory comment is in message

set_the_usual_form_variables
# msg_id, message, status
# notify_creator_p

validate_integer msg_id $msg_id

set db [ticket_getdbhandle]

set user_id [ad_get_user_id]

# check perms
set project_id [database_to_tcl_string $db "select project_id from ticket_issues where msg_id=$msg_id"]

if {![ticket_user_can_edit_project_p $user_id $project_id $db]} {
    ticket_deny_access
    return
}

set exception_text ""
set exception_count 0


if {![empty_string_p [string trim $message]]} {
    if [catch { set n_previous [database_to_tcl_string $db "select count(*)
from ticket_issue_responses
where response_to = $msg_id
and user_id = $user_id
and dbms_lob.instr(message,'[bboard_convert_plaintext_to_html $message]') > 0"]} errmsg] {
       ns_log Notice "failed trying to look up previous posting: $errmsg"
    } else {
    # lookup succeeded
	if { $n_previous > 0 } {
	    incr exception_count
	    append exception_text "<li>There are already $n_previous responses from you with the same body.
Perhaps you already posted this?
If you are sure that you also want to add this issue, 
back up and change at least one character in the subject 
or message area, then resubmit."
    }
  }
}

if {$exception_count > 0} {
    ad_return_complaint $exception_count $exception_text
    return
}

# gets around clob driver problem with and empty string
if [empty_string_p $message] {
    set message " "
}

set selection [ns_db 1row $db "select one_line, title, 
ticket_issues.project_id, notify_p
from ticket_issues, ticket_projects
where ticket_issues.project_id = ticket_projects.project_id
and msg_id = $msg_id"]
set_variables_after_query

set message_in_html [bboard_convert_plaintext_to_html $message]

if {![info exists notify_creator_p]} {
    set notify_creator_p "f"
}

with_transaction $db {
    
    update_last_modified_info $db $msg_id 

    if {[string compare $status "closed"] == 0} {

	ns_db dml $db "update ticket_issues 
	set modification_time = sysdate(), close_date = sysdate(), closed_by = $user_id, status='closed'  where msg_id = $msg_id"
    } else {
	ns_db dml $db "update ticket_issues 
	set status='$status', modification_time = sysdate() where msg_id = $msg_id"
    }

    if {![empty_string_p [string trim $message]]} {
	ns_db dml $db "insert into ticket_issue_responses (response_id,response_to,user_id,message,posting_time) values (nextval('ticket_response_id_sequence'),$msg_id,$user_id,'[DoubleApos $message_in_html]',sysdate())"
    }

    # send notifcation email
    if { $notify_p == "t" } {
	    send_ticket_change_notification $db $msg_id $message $user_id $notify_creator_p
    }
} {
    # something went a bit wrong during the insert
    ns_return 200 text/html "[ad_header "Error Closing Issue"]
<h3>Ouch!!</h3>
<hr>
We encountered a problem closing your issue.
Here was the bad news from the database:
<pre>
$errmsg
</pre>
[ad_footer]
"
    return
}


ns_returnredirect "index.tcl"