# User is closing this ticket.
# An explanatory comment is in message

set_the_usual_form_variables
# msg_id, message, fixed_release_id
# notify_creator_p

validate_integer msg_id $msg_id
validate_integer_or_null fixed_release_id $fixed_release_id

set user_id [ad_get_user_id]
set db [ticket_getdbhandle]


# check perms
set project_id [database_to_tcl_string $db "select project_id from ticket_issues where msg_id=$msg_id"]

if {![ticket_user_can_close_issue_p $user_id $msg_id $db] && 
    ![ticket_user_can_admin_project_p $user_id $project_id $db]} {
    ticket_deny_access
    return
}

set exception_text ""
set exception_count 0

if [catch { set n_previous [database_to_tcl_string $db "select count(*)
from ticket_issue_responses
where response_to = $msg_id
and user_id = $user_id
and dbms_lob.instr(message,'[bboard_convert_plaintext_to_html $message]') > 0"]} errmsg] {
    ns_log Notice "failed trying to look up previous posting: $errmsg"
} else {
    # lookup succeeded
    if { $n_previous > 0 } {
	incr exception_count
	append exception_text "<li>There are already $n_previous responses from you with the same body.
Perhaps you already posted this?
If you are sure that you also want to add this issue, 
back up and change at least one character in the subject 
or message area, then resubmit."
    }
}

if {$exception_count > 0} {
    ad_return_complaint $exception_count $exception_text
    return
}

# gets around clob driver problem with and empty string
if [empty_string_p $message] {
    set message " "
}

set selection [ns_db 1row $db "select one_line, title, 
ticket_issues.project_id, notify_p
from ticket_issues, ticket_projects
where ticket_issues.project_id = ticket_projects.project_id
and msg_id = $msg_id"]
set_variables_after_query

set message_in_html [bboard_convert_plaintext_to_html $message]
set QQmessage_in_html [DoubleApos $message_in_html]

if {![info exists notify_creator_p]} {
    set notify_creator_p "f"
}

with_transaction $db {

    update_last_modified_info $db $msg_id 

    set response_id [db_sequence_nextval $db ticket_response_id_sequence]
    
    ns_db dml $db "insert into ticket_issue_responses (response_id,response_to,user_id,message,posting_time) values ($response_id,$msg_id,$user_id,'$QQmessage_in_html',sysdate())"

    database_to_tcl_string $db "select ticket_update_for_response($response_id)"

    ns_db dml $db "update ticket_issues 
    set close_date = sysdate(), closed_by = $user_id, status='closed', fixed_release_id='$fixed_release_id'  where msg_id = $msg_id"
} {
    # something went a bit wrong during the insert
    ns_return 200 text/html "[ad_header "Error Closing Issue"]
<h3>Ouch!!</h3>
<hr>
We encountered a problem closing your issue.
Here was the bad news from the database:
<pre>
$errmsg
</pre>
[ad_footer]
"
    return
}


ns_returnredirect "index.tcl"
