set_the_usual_form_variables

# project_id, one_line, notify_p, message, priority
#
# ticket_type
# 
# and any defined picklist field names
# 

validate_integer project_id $project_id

set url "[ns_conn location][ticket_admin_url_stub]"
set db [ticket_getdbhandle]

set user_id [ad_get_user_id]

# check perms
if {![ticket_user_can_edit_project_p $user_id $project_id $db]} {
    ticket_deny_access
    return
}


set formdata [ns_set copy [ns_conn form]]

if {[empty_string_p [set ColValue.deadline.month]] && [empty_string_p [set ColValue.deadline.day]]} {
    ns_set update $formdata ColValue.deadline.year ""
}

if [catch {ns_dbformvalue $formdata deadline date deadline} error] {
    ad_return_complaint 1 "<li>Error parsing the deadline date: $error
<br>Please specify a month, day, and four digit year for the deadline.\n"
    return
}

# check input
set exception_text ""
set exception_count 0

set selection [ns_db 1row $db "select email, first_names, last_name from users where user_id = $user_id"]

set_variables_after_query


set name "$first_names $last_name"
set QQemail [DoubleApos $email]
set QQname [DoubleApos $name]

if { ![info exists project_id] || $project_id == "" } {
    incr exception_count
    append exception_text "<li>You somehow got here without a project specified.\n"
}

if { ![info exists one_line] || $one_line == "" } {
    incr exception_count
    append exception_text "<li>You must enter a subject line.\n"
}

if { ![info exists notify_p] || $notify_p == "" } {
    incr exception_count
    append exception_text "<li>You somehow got here without specifying notification.\n"
}

if { ![info exists message] || $message == "" } {
    incr exception_count
    append exception_text "<li>You have to say something about the issue.\n"
}


if {[info exists preformat] && $preformat == "yes"} {
    set message_in_html "<pre>
[clean_up_html $message]
</pre>
" } else {
    set message_in_html [bboard_convert_plaintext_to_html $message]
}

set QQmessage_in_html [DoubleApos $message_in_html]

if [catch { set n_previous [database_to_tcl_string $db "select count(*) from ticket_issues
where one_line = '$QQone_line'
and project_id = $project_id
and dbms_lob.instr(message,'[bboard_convert_plaintext_to_html $QQmessage]') > 0"]} errmsg] {
    ns_log Notice "failed trying to look up previous posting: $errmsg"
} else {
    # lookup succeeded
    if { $n_previous > 0 } {
	incr exception_count
	append exception_text "<li>There are already $n_previous messages in the database with the same subject line and body.  Perhaps you already posted this?  Here are the messages:
<ul>
"
        set selection [ns_db select $db "select first_names || ' ' || last_name as name, email, posting_time
from ticket_issues, users
where one_line = '$QQone_line'
and project_id = $project_id
and dbms_lob.instr(message,'[DoubleApos $message]') > 0
and ticket_issues.user_id = users.user_id"]

        while {[ns_db getrow $db $selection]} {
	    set_variables_after_query
	    append exception_text "<li>$posting_time by $name ($email)\n"
	}
	append exception_text "</ul>
If you are sure that you also want to add this issue, 
back up and change at least one character in the subject 
or message area, then resubmit."
    }
}


if {$exception_count > 0} {
    ad_return_complaint $exception_count $exception_text
    return
}

# ReturnHeaders

set project_title [database_to_tcl_string $db "select title from ticket_projects where project_id = $project_id"]

#  ns_write "[ad_header "Inserting a New Issue"]

#  <h2>Inserting a New Issue</h2>

#  [ad_context_bar_ws_or_index [list "[ticket_admin_url_stub]/index.tcl" "Ticket Tracker"]  "Ticket Created"]
#  "

#  ns_write "<hr>\n"

# Collect up the picklist field values
set picklist_columns {}
set picklist_values {}
foreach entry [ticket_picklist_data] {
    set field_name [ticket_picklist_entry_field_name $entry]
    set column_name [ticket_picklist_entry_column_name $entry]
    lappend picklist_columns $column_name
    lappend picklist_values "'[DoubleApos [set $field_name]]'"
}

# Tickets default to "open" status when created.
set status "open"

# Find if there is a default assignee
set default_assignee [database_to_tcl_string_or_null $db "select default_assignee from
ticket_projects 
where project_id=$project_id"]

#  if {[llength $picklist_columns] > 0} {
#      set custom_values ", [join $picklist_values {,}]"
#      set custom_field_names ",  [join $picklist_columns {,}]"
#  } else {
#      set custom_values ""
#      set custom_field_names ""
#  }

with_transaction $db {
    set new_id [database_to_tcl_string $db "select ticket_issue_id_sequence.nextval from dual"]
    set indexed_stuff "$one_line $message $email $name $new_id"
    set QQindexed_stuff [DoubleApos $indexed_stuff]

    set deadline_sql [db_postgres_null_sql $deadline]

    ns_db dml $db "insert into ticket_issues
    (msg_id,project_id,user_id,one_line,message,indexed_stuff,posting_time, severity, notify_p, deadline, status, ticket_type, privacy, last_modified_by, release_id)
    values ($new_id,$project_id,$user_id,'$QQone_line','$QQmessage_in_html','$QQindexed_stuff',sysdate(),'$severity','$notify_p', $deadline_sql, '$status', '$ticket_type', [db_null_sql $privacy], $user_id, '$release_id')"

    # Insert the custom fields (new way of doing it per project - ben@mit.edu)
    foreach custom_field [ticket_custom_field_get_project_fields $project_id] {
	ns_db dml $db "insert into ticket_projects_field_vals (project_id, field_id, issue_id, field_val) values ($project_id, [lindex $custom_field 0], $new_id, '[DoubleApos [set [ticket_custom_field_varname [lindex $custom_field 1]]]]')"
    }

    if {[info exists default_assignee] && \
	    ![empty_string_p $default_assignee]} {
	ns_db dml $db "insert into ticket_issue_assignments (msg_id, user_id, active_p) VALUES  ($new_id, $default_assignee, 't')"
    }

    # Add the fact that this user is obviously interested in the issue
    if {[ticket_issue_author_is_interested_p]} {
	notification_set_user_interest $user_id [ticket_notification_class $db] $new_id $db
    }
    
} {
    # something went a bit wrong during the insert
    ns_return 200 text/html "<h3>Ouch!!</h3>
Here was the bad news from the database:
<pre>
$errmsg
</pre>
[ad_footer]
"
    return
}

set msg_id $new_id

ns_returnredirect issue-view.tcl?[export_url_vars msg_id]

#  ns_write "<h3>Success!!</h3>
#  A new issue for project 
#  <a href=\"project-top.tcl?project_id=$project_id\">$project_title</a>
#  has been entered in the database:

#  <br>

#  #$new_id: [clean_up_html $one_line]

#  <p>

#  <FORM METHOD=POST ENCTYPE=multipart/form-data ACTION=issue-add-attachment-2.tcl>
#  [export_form_vars msg_id]
#  Upload an attachment: <INPUT TYPE=file name=file><br>
#  Name of Attachment: <INPUT TYPE=text name=filename><br>
#  <INPUT TYPE=submit value=add>
#  </FORM>

#  <p>

#  You can:<br>
#  <ul>
#  <li><a href=\"issue-view.tcl?msg_id=$new_id\">View issue details and make assignments</a>
#  <li><a href=\"issue-new.tcl\">Add another issue</a>
#  <li><a href=\"project-top.tcl?[export_url_vars project_id]\">Go to project page $project_title</a>
#  <li><a href=\"index.tcl?[export_url_vars project_id]\">Return to main page</a>
#  </ul>
#  "

#send out the email

set ticket_email [ticket_reply_email_addr]
set extra_headers [ns_set create] 
ns_set update $extra_headers "Reply-to" $ticket_email

if { $notify_p == "t" } {

    set selection [ns_db select $db "select
email as notify_email
from users, ticket_assignments
where project_id = $project_id 
and users.user_id = ticket_assignments.user_id
"]

    while { [ns_db getrow $db $selection] } { 
	set_variables_after_query
	ns_sendmail $notify_email $email "New issue $one_line in project $project_title (TR#$new_id)" "Priority: $priority 
Submitted By: $name 
Description: $message

Please use $url/issue-view.tcl?msg_id=$new_id to manage this issue." $extra_headers
        # ns_write "<br> Emailed $notify_email"
   }
}

#  ns_write "
#  [ad_footer]
#  "
