# This page is for viewing an issue as a privileged user - 

# Most data fields are modifiable

set_the_usual_form_variables

# msg_id

validate_integer msg_id $msg_id

set db [ticket_getdbhandle]

set user_id [ad_get_user_id]

# permissions
if {![ticket_user_can_edit_issues_p $user_id $msg_id $db]} {
    ticket_deny_access
    return
}

set selection [ns_db 0or1row $db "select 
posting_time as posting_date,
case when deadline is null then null else deadline::date end as deadline, one_line, message, priority, ticket_type,  email, first_names, 
last_name, status, release_id
from ticket_issues, users
where msg_id = $msg_id
and users.user_id = ticket_issues.user_id"]

if { $selection == "" } {
    # message was probably deleted
    ad_return_complaint 1 "<li>Couldn't find message $msg_id."
    return
}

set_variables_after_query
set this_one_line $one_line
set name "$first_names $last_name"

set selection [ns_db 1row $db "select 
 title, ticket_issues.project_id, notify_p, public_p,
 ticket_issues.last_modified_by, 
fixed_release_id,
 ticket_issues.severity, 
 ticket_issues.privacy,
 modification_time,
 ticket_issues.contact_name,
 ticket_issues.contact_email,  ticket_issues.contact_info1,
 close_date, ticket_release_name(release_id) as release
from ticket_projects, ticket_issues
where ticket_projects.project_id = ticket_issues.project_id
and ticket_issues.msg_id = $msg_id
"]

set_variables_after_query


# get current user's email, to export as the "last modified by" value
set email [database_to_tcl_string $db "select email from users where user_id=[ad_get_user_id]"]


#if { [empty_string_p $deadline] } {
#    set deadline [ns_localsqltimestamp]
#}

#  if { $public_p == "t" } {
#      set PUBLIC_Y_CHECKED checked
#      set PUBLIC_N_CHECKED ""
#  } else {
#      set PUBLIC_N_CHECKED checked
#      set PUBLIC_Y_CHECKED ""
#  }

ReturnHeaders

if {[info exists deadline] && [empty_string_p $deadline]} {
    unset deadline
}

append page "[ad_header [clean_up_html $one_line]]

<h2>$ticket_type \#$msg_id: [clean_up_html $one_line]</h2>

[ad_context_bar_ws_or_index [list "[ticket_admin_url_stub]/index.tcl" "Ticket Tracker"] [list "project-top.tcl?project_id=$project_id" $title] "One Ticket"]

<hr>

<blockquote>
<table border=0><tr><td bgcolor=#f0f0f0>$message</td></tr></table>
<font size=-1>
"

set selection [ns_db select $db "select attachment_name from ticket_issues_attachments where msg_id=$msg_id"]

append page "<p>Attachments: "

while {[ns_db getrow $db $selection]} {
    set_variables_after_query

    append page "<a href=issue-download-attachment.tcl?[export_url_vars msg_id attachment_name]>$attachment_name</a> | "
}

append page "<a href=issue-add-attachment.tcl?[export_url_vars msg_id]>(new)</a> <p>"

if {[notification_user_is_interested_p $user_id [ticket_notification_class $db] $msg_id $db]} {
    append page "Notification <b>Enabled</b> (<A href=issue-remove-user-interest.tcl?[export_url_vars msg_id]>Disable</a>)"
} else {
    append page "Notification <b>Disabled</b> (<A href=issue-set-user-interest.tcl?[export_url_vars msg_id]>Enable</a>)"
}

append page "</font><p></blockquote>

<form action=issue-edit.tcl method=post>
<input type=hidden name=prev_status [export_form_value status]>
[export_form_vars email msg_id]

<blockquote>
<table border=0 cellspacing=3>
<tr>
  <th valign=top align=left>Subject:</th>
  <td><input size=64 maxsize=200 type=text name=one_line [export_form_value one_line]></td>
<tr>
<th valign=top align=left>Ticket Type:</th><td><select name=ticket_type>[ticket_html_select_ticket_type $ticket_type]</select>
"

if {[ticket_privacy_enabled_p]} {
    append page "&nbsp;&nbsp;&nbsp; Privacy Level: [make_html_select privacy {{"" "(not set)"} 1 2 3 4 5} $privacy]"
} else {
    append page "[philg_hidden_input privacy ""]"
}

append page "
</tr>"

# Status
append page "
<tr><td align=left><b>Status:</b><td>"

if {[string compare $status "closed"] == 0} {
    append page "Closed (<a href=\"issue-reopen.tcl?msg_id=$msg_id\">Reopen this issue</a>)\n [export_form_vars status]" 
} else {
    append page "<select name=status>"

    if {[string compare $status "fixed waiting approval"] == 0} {
	append page "<option selected> fixed waiting approval"
    }

append page "[ad_generic_optionlist [ticket_status_types] [ticket_status_types] $status]
</select>"

if {[string compare $status "fixed waiting approval"] != 0}  {
append page "(<a href=issue-fix.tcl?[export_url_vars msg_id]>fix issue</a>)\n"
}

if {[ticket_user_can_admin_issues_p $user_id [list $msg_id] $db] ||
    [ticket_user_can_close_issue_p $user_id $msg_id $db] } {
    append page "(<a href=issue-close.tcl?[export_url_vars msg_id]>close issue</a>)
    "
}
}

# Project
append page "</td></tr>
<tr><th align=left>Project:<td>$title
</td></tr>"

# Severity
append page "<tr><td align=left><b>Severity:</b><td>"
append page "<select name=severity>
[ad_generic_optionlist [ticket_severity_types] [ticket_severity_types] $severity]
</select></td></tr>
"

# Deadline
append page "
<tr><td align=left><b>Deadline:</b><td>[ticket_dateentrywidget_with_nulls deadline [export_var deadline  [database_to_tcl_string $db "select date_part('year', sysdate()) from dual"]]]
</tr>"

append page "<input type=hidden name=prev_severity [export_form_value severity]>"

set selection [ns_db select $db "select field_name, field_pretty_name, field_type, field_vals, field_val from ticket_projects_fields fields, ticket_projects_field_vals vals where fields.project_id=$project_id and vals.project_id=$project_id and vals.issue_id=$msg_id and fields.field_id=vals.field_id"]

while {[ns_db getrow $db $selection]} {
    set_variables_after_query

    append page "<tr><th align=left>$field_pretty_name</th><td>[ticket_custom_field_entry_form $field_type $field_name $field_vals $field_val]</td></tr>"
}

set return_url "issue-view.tcl?[export_url_vars msg_id]"
set return_pretty_name "Issue #$msg_id: $one_line"

set selection [ns_db 0or1row $db "select release as fixed_release from ticket_project_releases where release_id='$fixed_release_id'"]
if {$selection != ""} {
    set_variables_after_query
} else {
    set fixed_release ""
}

# Version
append page "<tr>
<th align=left>Project Release:</th>
<td>[ticket_release_select $project_id release_id $release_id $db] (<a href=releases.tcl?[export_url_vars project_id return_url return_pretty_name]>Manage Releases</a>)</td>
</tr>
<th align=left>Release Where Fixed:</th>
<td>$fixed_release</td>
</tr>\n"

# Show customizable picklist fields
#  foreach field [ticket_picklist_field_names] { 
#      set entry [ticket_picklist_field_info $field]
#      append page "<tr>[ticket_picklist_html_fragment $field [set [ticket_picklist_entry_column_name $entry]]]</tr>\n"
#  }

append page "
<tr><td align=left><b>Submitted By:</b><td>$name on $posting_date</tr>
<tr><td align=left><b>Last Modified By:</b><td>[database_to_tcl_string_or_null $db "select first_names || ' ' || last_name from users where user_id='$last_modified_by'"] on $modification_time</tr></tr>
"

append page "<tr><td colspan=2 align=center>
<input type=submit name=submit value=\"Update\">
</td></tr>

</form>
"

if { $close_date != "" } {
    append page "<tr><td align=left><b>Closed On:</b><td>$close_date</tr>\n"
}

append page "

</table>
</blockquote>
<p>
"


set selection [ns_db select $db "select 
 response_id, public_p,
 users.first_names || ' ' || users.last_name as name,
 posting_time as posting_date,
 message
from ticket_issue_responses, users
where ticket_issue_responses.user_id = users.user_id
and ticket_issue_responses.response_to = $msg_id
order by posting_time"] 

while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set text "<blockquote>
    $message
    </blockquote>
    Submitted by $name on $posting_date, "

    if {$public_p == "t"} {
	 append text "<i>Public?</i> Yes (<a href=\"set-response-visibility.tcl?response_id=$response_id&public_p=f&msg_id=$msg_id\">toggle</a>)"
    } else {
	append text  "<i>Public?</i> No (<a href=\"set-response-visibility.tcl?response_id=$response_id&public_p=t&msg_id=$msg_id\">toggle</a>)"
    }

    lappend responses $text

}

if { [info exists responses] } {
    append page "<b>Comments</b>
    [join $responses "<hr width=300>"]
    "
}

append page "<ul>"

if { $close_date == "" } {
    append page "<li><a href=\"issue-response.tcl?msg_id=$msg_id\">Add comment to \"[clean_up_html $one_line]\"</a>

"
} 


append page "
</ul>"

# List xrefs
append page "<b>Related Issues</b>
<br>
"
set selection [ns_db select $db "select to_ticket, one_line as xone_line, msg_id as xmsg_id
from ticket_xrefs, ticket_issues
where  to_ticket = ticket_issues.msg_id and 
from_ticket=$msg_id 
union
select to_ticket, one_line as xone_line, msg_id as xmsg_id
from ticket_xrefs, ticket_issues
where  from_ticket = ticket_issues.msg_id and 
to_ticket=$msg_id 
"]


# target for subroutine pages to return to this page
set target "issue-view.tcl"
append page "<ul>"
while {[ns_db getrow $db $selection]} {
    set_variables_after_query
    append page "<li><a href=\"issue-view.tcl?msg_id=$xmsg_id\">\[$xmsg_id\] $xone_line</a>&nbsp;&nbsp; <a href=\"unlink-xref.tcl?from_msg_id=$msg_id&to_msg_id=$xmsg_id&[export_url_vars target]\">(unlink)</a>"
}

append page "<p><li><a href=\"add-xref-search.tcl?[export_url_vars msg_id target]\">Add a cross-reference</a>"

append page "</ul>"

# View change log
append page "<p>
<ul>
<li><a href=issue-change-log.tcl?msg_id=$msg_id>View Change History</a>
<li><a href=\"issue-new.tcl?project_id=$project_id\">Add a new issue.</a>
</ul>


<p>
<b><a name=assignment>Assignment</a></b>
<table border=1 cellpadding=10>
<tr>
<th>Users assigned to this issue</th>
<th>Candidates for assignment</th>
</tr>
<tr valign=top><td><ul>
"

# query for the users assigned to this issue already
set selection [ns_db select $db "select case when first_names=email then '' else first_names end as first_names , case when last_name= email then '' else last_name end as last_name, users.user_id, email
from users, ticket_issue_assignments 
where users.user_id = ticket_issue_assignments.user_id 
AND msg_id=$msg_id
order by case when first_names=email then 'z' else lower(users.last_name) end, case when first_names=email then lower(email) else lower(users.first_names) end"]

while {[ns_db getrow $db $selection]} {
    set_variables_after_query
    if {$last_name == ""} {
	set name "$email"
    } else {
	set name "$last_name, $first_names ($email)"
    }
    
    append page "<li> \[<a href=\"issue-deassign-user.tcl?assignee_id=$user_id&msg_id=$msg_id\">deassign</a>\] $name \n"
}

append page  "</ul>
<br>
</td>

<td>
"

# query for the users on the relevant project 
# but not assigned to this issue already

set ticket_group [ticket_admin_group $db]

set selection [ns_db select $db "select users.user_id, case when first_names=email then '' else first_names end as first_names, case when last_name=email then '' else last_name end as last_name, email
from users 
where ticket_user_can_edit_project_p(user_id, $project_id)='t'
and users.user_id NOT IN 
    (SELECT ticket_issue_assignments.user_id 
     from ticket_issue_assignments, ticket_issues
     where ticket_issues.msg_id=$msg_id
     and ticket_issue_assignments.msg_id = ticket_issues.msg_id)
and users.email != 'system'
order by case when first_names=email then 'z' else lower(users.last_name) end, case when first_names=email then lower(email) else lower(users.first_names) end"]

while {[ns_db getrow $db $selection]}  {
    set_variables_after_query
    if {$last_name == ""} {
	set name "$email"
    } else {
	set name "$last_name, $first_names ($email)"
    }

    append page "<li> \[<a href=\"issue-assign-user.tcl?assignee_id=$user_id&msg_id=$msg_id\">assign</a>\]  $name\n"
}

append page "</ul>
</ul>
<p>
</form>
</td>
</tr>
</table>
"



append page "[ad_footer]"

ns_write $page



