# This page is for viewing an issue as a privileged user - 

# Most data fields are modifiable

set_the_usual_form_variables

# msg_id

validate_integer msg_id $msg_id

set db [ticket_getdbhandle]

set user_id [ad_get_user_id]

if {![ticket_user_can_see_issues_p $user_id $msg_id $db]} {
    ticket_deny_access
    return
}

# If user can edit issue, source the proper page
if {[ticket_user_can_edit_issues_p $user_id $msg_id $db]} {
    ns_db releasehandle $db
    util_source [ticket_url_stub]/issue-view-with-edit.tcl
    return
}

set selection [ns_db 0or1row $db "select 
posting_time as posting_date,
deadline, one_line, message, priority, ticket_type,  email, first_names, 
last_name, status
from ticket_issues, users
where msg_id = $msg_id
and users.user_id = ticket_issues.user_id"]

if { $selection == "" } {
    # message was probably deleted
    ad_return_complaint 1 "<li>Couldn't find message $msg_id."
    return
}

set_variables_after_query
set name "$first_names $last_name"
set this_one_line $one_line


set selection [ns_db 1row $db "select 
title,ticket_issues.project_id, notify_p, public_p,
fixed_release_id,
 ticket_issues.group_id  as ticket_group_id,
 ticket_issues.last_modified_by, 
 ticket_issues.severity, 
 modification_time,
 ticket_issues.contact_name,
 ticket_issues.contact_email,  ticket_issues.contact_info1,
 close_date,
 ticket_release_name(release_id) as release
from ticket_projects, ticket_issues
where ticket_projects.project_id = ticket_issues.project_id
and ticket_issues.msg_id = $msg_id"]

set_variables_after_query

# access control
if {![ticket_user_can_see_project_p $user_id $project_id $db]} {
    ticket_deny_access
    return
}

if {$public_p == "t"} {
    set public_pretty "Yes"
} else {
    set public_pretty "No"
}


append group_select_menu "<select name=ticket_group>"


# Get list of all user groups
set selection [ns_db select $db "select distinct group_id as group_id_x, group_name 
from user_groups"]

while {[ns_db getrow $db $selection]} {
    set_variables_after_query
    if {$group_id_x == $ticket_group_id} {
	append group_select_menu "<option selected value=\"$group_id_x\">$group_name</option>"
    } else {
	append group_select_menu "<option value=\"$group_id_x\">$group_name</option>"	
    }
} 

append group_select_menu "</select>"

# get current user's email, to export as the "last modified by" value
set email [database_to_tcl_string $db "select email from users where user_id=[ad_get_user_id]"]


#if { [empty_string_p $deadline] } {
#    set deadline [ns_localsqltimestamp]
#}

ReturnHeaders


append page "[ad_header [clean_up_html $one_line]]

<h2>$ticket_type \#$msg_id: [clean_up_html $one_line]</h2>

[ad_context_bar_ws_or_index [list "[ticket_url_stub]/index.tcl" "Ticket Tracker"] [list "project-top.tcl?project_id=$project_id" $title] "One Ticket"]

<hr>

<blockquote>
<table border=0><tr><td bgcolor=#f0f0f0>$message</td></tr></table>
<font size=-1>
"

set selection [ns_db select $db "select attachment_name from ticket_issues_attachments where msg_id=$msg_id"]

append page "<p>Attachments: "

while {[ns_db getrow $db $selection]} {
    set_variables_after_query

    append page "<a href=issue-download-attachment.tcl?[export_url_vars msg_id attachment_name]>$attachment_name</a> | "
}

append page "<p>"

if {[notification_user_is_interested_p $user_id [ticket_notification_class $db] $msg_id $db]} {
    append page "Notification <b>Enabled</b> (<A href=issue-remove-user-interest.tcl?[export_url_vars msg_id]>Disable</a>)"
} else {
    append page "Notification <b>Disabled</b> (<A href=issue-set-user-interest.tcl?[export_url_vars msg_id]>Enable</a>)"
}

append page "</font><p></blockquote>

<blockquote>
<table border=0 cellspacing=3>
<tr>
  <th valign=top align=left>Subject:</th>
  <td>[clean_up_html $one_line]</td>
<tr>
<th valign=top align=left>Ticket Type:</th><td>$ticket_type</select>
&nbsp;&nbsp;&nbsp;<b>Public?</b> $public_pretty

</tr>"

# Status
append page "
<tr><td align=left><b>Status:</b><td> $status
</td></tr>
"


# Project
append page "
<tr><th align=left>Project:<td>
$title
</td></tr>"

# Severity
append page "<tr><td align=left><b>Severity:</b><td>"
append page "
 $severity</td></tr>
"

# Deadline
append page "
<tr><td align=left><b>Deadline:</b><td> [util_AnsiDatetoPrettyDate $deadline]
</tr>"



# Show customizable picklist fields
#  foreach field [ticket_picklist_field_names] { 
#      set entry [ticket_picklist_field_info $field]
#      append page "<tr><th align=left>[ticket_picklist_entry_pretty_name $entry]</th><td>[set [ticket_picklist_entry_column_name $entry]]</td></tr>\n"
#  }

# Add real custom fields
set selection [ns_db select $db "select field_name, field_pretty_name, field_type, field_val from ticket_projects_fields fields, ticket_projects_field_vals vals where fields.project_id=$project_id and vals.project_id=$project_id and vals.issue_id=$msg_id and fields.field_id=vals.field_id"]

while {[ns_db getrow $db $selection]} {
    set_variables_after_query

    append page "<tr><th align=left>$field_pretty_name</th><td>[ticket_custom_field_display_field $field_type $field_val]</td></tr>\n"
}

set selection [ns_db 0or1row $db "select release as fixed_release from ticket_project_releases where release_id='$fixed_release_id'"]
if {$selection != ""} {
    set_variables_after_query
} else {
    set fixed_release ""
}

append page "
<tr><th align=left>Release</th><td>$release</td></tr>
<tr><th align=left>Release where Fixed</th><td>$fixed_release</td></tr>
<tr><td align=left><b>Submitted By:</b><td>$name on $posting_date</tr>
<tr><td align=left><b>Last Modified By:</b><td>[database_to_tcl_string_or_null $db "select first_names || ' ' || last_name from users where user_id='$last_modified_by'"] on $modification_time</tr></tr>
"

if { $close_date != "" } {
    append page "<tr><td align=left><b>Closed On:</b><td>$close_date</tr>\n"
}

append page "

</table>
</blockquote>
<p>
"


set selection [ns_db select $db "select 
 response_id, public_p,
 users.first_names || ' ' || users.last_name as name,
posting_time as posting_date,
 message
from ticket_issue_responses, users
where ticket_issue_responses.user_id = users.user_id
and ticket_issue_responses.response_to = $msg_id
and public_p='t'
order by posting_time"] 

while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set text "<blockquote>
    $message
    </blockquote>
    Submitted by $name on $posting_date. "

    lappend responses $text

}

if { [info exists responses] } {
    append page "<b>Comments</b>
    [join $responses "<hr width=300>"]
    "
}

append page "<ul>"



append page "
</ul>"

# List xrefs
append page "<b>Related Issues</b>
<br>
"
set selection [ns_db select $db "select to_ticket, one_line as xone_line, msg_id as xmsg_id
from ticket_xrefs, ticket_issues
where  to_ticket = ticket_issues.msg_id and 
from_ticket=$msg_id 
union
select to_ticket, one_line as xone_line, msg_id as xmsg_id
from ticket_xrefs, ticket_issues
where  from_ticket = ticket_issues.msg_id and 
to_ticket=$msg_id 
"]


# target for subroutine pages to return to this page
set target "issue-view.tcl"
append page "<ul>"
while {[ns_db getrow $db $selection]} {
    set_variables_after_query
    append page "<li><a href=\"issue-view.tcl?msg_id=$xmsg_id\">\[$xmsg_id\] $xone_line</a>&nbsp;&nbsp; <a href=\"unlink-xref.tcl?from_msg_id=$msg_id&to_msg_id=$xmsg_id&[export_url_vars target]\">(unlink)</a>"
}

append page "</ul>"

# View change log
append page "<p>
<ul>
<li><a href=issue-change-log.tcl?msg_id=$msg_id>View Change History</a>
<li><a href=\"issue-new.tcl?project_id=$project_id\">Add a new issue.</a>
</ul>


<p>
<b>Assignment</b>
<table border=1 cellpadding=10>
<tr>
<th>Users assigned to this issue</th>
</tr>
<tr valign=top><td><ul>
"

# query for the users assigned to this issue already
set selection [ns_db select $db "select first_names, last_name, users.user_id
from users, ticket_issue_assignments 
where users.user_id = ticket_issue_assignments.user_id 
AND msg_id=$msg_id
order by users.last_name, users.first_names"]

while {[ns_db getrow $db $selection]} {
    set_variables_after_query
    append page "<li> $first_names $last_name\n"
}

append page "
</td>
</tr>
</table>
"



append page "[ad_footer]"

ns_write $page



