set_form_variables 0


# form vars:
# 
# user_id - user_id this page is for (optional - defaults to logged in user)
# project_id - project_id for this user (optional - defaults to all tickets 
#              for the selected user)
#
# filter conditions
#

# logged_in_user_id corresponds to the user that is viewing this page
# and user_id is the user_id for the person whom we want to see a summary of 
# ticket issues.

if { [info exists user_id] } {
    # validate the user_id input and get the user_id of the person viewing this
    # page.
    validate_integer user_id $user_id
    set logged_in_user_id [ad_get_user_id]
} else {
    # a user_id was not passed to this page so just default to showing the user
    # his or her own tickets.
    set user_id [ad_get_user_id]
    set logged_in_user_id $user_id
}

if { [info exists project_id] } {
    validate_integer project_id $project_id
}

# Status
# view_status  { open closed deferred created_by_you }
#
# Creation time
# view_created { last_24 last_week last_month all}
#
# Ticket Type
# view_type { all defects enhancements issues }
#
#
# order_by       column name to sort table by 

set ctrlvars {view_type view_assignment view_status view_created user_id project_id}

set db [ticket_getdbhandle]

set name [database_to_tcl_string $db "select first_names || ' ' || last_name 
from users where user_id = $user_id"]

ReturnHeaders

ns_write "[ad_header "[ticket_system_name] Home"]

<h2>Tickets for $name</h2>

[ad_context_bar_ws_or_index [list "index.tcl" "Ticket Tracker"] "One User"]

<hr>
"

# List of form vars used to select tickets to display

# Assignment filter conditions
#

# default to show issues assigned to $user_id
if {![info exists view_type]} {
    set view_type all
}

# default to show issues assigned to $user_id
if {![info exists view_assignment]} {
    set view_assignment user
}

# default to only your own assigned issues
if {![info exists view_status]} {
    set view_status open
}

# default to all time
if {![info exists view_created]} {
    set view_created all
}

set assignment_filter ""

if { [info exists project_id] } {
    set project_filter "and ticket_projects.project_id = $project_id "
} else {
    set project_filter " "
}

switch $view_status {
    "open" {set status_filter "and ((status <> 'closed') and (status <> 'deferred')) " }
    "deferred" {set status_filter "and ((status <> 'closed') or (status = 'deferred')) " }
    "closed" {set status_filter "" } # shows everything
}

switch $view_created {
   "last_24"   { set date_filter "and (posting_time > (sysdate() - 1))" } 
   "last_week" { set date_filter "and (posting_time > (sysdate() - 7)) " } 
   "last_month" { set date_filter "and (posting_time > (sysdate() - 30)) " }
   "all" { set date_filter "" }
}

switch $view_type {
    "all" { set issue_table "ticket_issues" }
    "defects" { set issue_table "ticket_defects" }
    "enhancements" { set issue_table "ticket_enhancements" }
    "issues" { set issue_table "ticket_issues_issues" }
}

# Sort order of tickets by
if {![info exists order_by]} {
    set order_by "ticket_issues.msg_id"
} elseif {[string match "project*" $order_by]} {
    set order_by "project_title"
}


################################################################
# GUI ticket filter controls

# List of all the state vars we need to pass through these toggle switches
set filter_vars {view_type view_assignment view_status view_created order_by user_id project_id}


append results "<table border=0 cellspacing=0 cellpadding=0 width=100%>
<tr>
<th bgcolor=#ECECEC>Type</th>
<th bgcolor=#ECECEC>Status</th>
<th bgcolor=#ECECEC>Creation Time</th></tr>"

set url "one-user.tcl"

#### Assignment flags
# Show assigned to you 
append results "<tr>
<td align=center>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\["

# Issue type filter
append results [ticket_control_vars view_type all $filter_vars "ALL" "$url"]
append results " | "
append results [ticket_control_vars view_type defects $filter_vars "Defects" "$url"]
append results " | "
append results [ticket_control_vars view_type enhancements $filter_vars "Enhancements" "$url"]
append results " | "
append results [ticket_control_vars view_type issues $filter_vars "Issues" "$url"]
append results "\]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>\n<td align=center>\["

# Show open issues
append results [ticket_control_vars view_status open $filter_vars "active" "$url"]
append results " | "
# Show deferred issues
append results [ticket_control_vars view_status deferred $filter_vars "+deferred" "$url"]
append results " | "
# Show closed issues
append results [ticket_control_vars view_status closed $filter_vars "+closed" "$url"]


#### Creation time filter
append results "\]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>\n<td align=center>\["

append results [ticket_control_vars view_created last_24 $filter_vars "last 24 hrs" "$url"]
append results " | "
append results [ticket_control_vars view_created last_week $filter_vars "last week" "$url"]
append results " | "
append results [ticket_control_vars view_created last_month $filter_vars "last month" "$url"]
append results " | "
append results [ticket_control_vars view_created all $filter_vars "all" "$url"]

append results "\]</td></tr></table><p>"

###################################################################
#
# Now display the tickets pertaining the selected user and project
#
###################################################################

set query "select distinct
 ticket_issues.msg_id,
 ticket_issues.ticket_type, 
 ticket_issues.one_line,
 ticket_issues.status,
 ticket_issues.severity,
 ticket_issues.posting_time,
 users.email,
 ticket_projects.title as project_title,
 ticket_projects.project_id,
 ticket_issues.priority, 
 to_char(ticket_issues.modification_time, 'MM/DD/YY') as modification_time_pretty,
 to_char(ticket_issues.posting_time, 'MM/DD/YY') as creation_time_pretty,
 to_char(ticket_issues.close_date, 'MM/DD/YY') as close_date_pretty,
 to_char(ticket_issues.close_date, 'MM/DD/YY') as close_date,
 to_char(ticket_issues.deadline, 'MM/DD/YY') as deadline_pretty,
 to_char(ticket_issues.deadline, 'MM/DD/YY') as deadline,
 sysdate() - deadline as pastdue,
 ticket_issue_assignments.active_p as assigned_p,
 ticket_issues.public_p,
 ticket_assignees(ticket_issues.msg_id) as ticket_assignees
from $issue_table ticket_issues, ticket_projects, users, ticket_issue_assignments
where ticket_user_can_see_project_p($logged_in_user_id, ticket_projects.project_id)='t'
and users.user_id = ticket_issues.user_id
and ticket_projects.project_id = ticket_issues.project_id
$project_filter
and ticket_issues.msg_id = ticket_issue_assignments.msg_id
and ticket_issue_assignments.user_id= $user_id 
$assignment_filter
$status_filter
$date_filter
order by $order_by, ticket_issues.priority, ticket_issues.posting_time"

set this_url "one-user.tcl"
set display_project_p 0
append results [ticket_summary_display]


ns_write  "$results

<ul>
<li>Add new <a href=\"issue-new.tcl\">issue</a>
</ul>
[ad_footer]
"

