# Summarize projects status
#

set db [ns_db gethandle]

ReturnHeaders
ns_write "[ad_header "Project Status"]
<h2>Project Status Summary</h2>

[ad_context_bar_ws_or_index [list "[ticket_url_stub]/index.tcl" "Ticket Tracker"]  "Project Summaries"]

<hr>

<blockquote>
<table border=0>
<tr>
<th>Title</th>
<th>Assigned</th>
<th>Total </th>
<th>Active</th>
<th>Closed</th>
<th>Deferred</th>
<th>Last Mod</th>
<th>Oldest Active</th>
</tr>
"

set i 0

set user_id [ad_get_user_id]

set selection [ns_db select $db "
select 
  tp.project_id, 
  project_full_name(tp.project_id) as title, 
  count(msg_id) as total,
  sum(case when ticket_assignees(msg_id)='' then 0 else 1 end) as assigned,
  sum(case when status='closed' then 1 else 0 end) as closed,
  sum(case when status='closed' then 0 else case when status='deferred' then 0 else case when status is NULL then 0 else 1 end end end) as open,
  sum(case when status='deferred' then 1 else 0 end) as deferred,
  max(modification_time) as lastmod,
  min(posting_time) as oldest,
  sum(ticket_one_if_high_priority(priority, status)) as high_pri,
  sum(ticket_one_if_blocker(severity, status)) as blocker
from ticket_projects tp, ticket_issues ti
where tp.project_id = ti.project_id
and ticket_user_can_see_project_p($user_id, tp.project_id)='t'
group by tp.project_id, tp.title
order by upper(title)
"]

while { [ns_db getrow $db $selection] } {
    set_variables_after_query

    # show summaries of ticket stats
    # total #
    # open
    # closed
    # deferred
    # last modified (max)
    # 

    if {($i % 2) == 0} {
	set bgcolor "bgcolor=\#ECECEC"
    } else {
	set bgcolor ""
    }


    regsub "@.*" $assigned "" assigned

    ns_write "
<tr>
  <td $bgcolor><a href=\"project-top.tcl?project_id=$project_id\">$title
  <td $bgcolor align=left>$assigned&nbsp;
  <td $bgcolor align=right>[blank_zero $total]&nbsp;
  <td $bgcolor align=right>[blank_zero $open]&nbsp;
  <td $bgcolor align=right>[blank_zero $closed]&nbsp;
  <td $bgcolor align=right>[blank_zero $deferred]&nbsp;
  <td $bgcolor align=right nowrap>$lastmod&nbsp;
  <td $bgcolor align=right nowrap>$oldest&nbsp;
</tr>
"
   incr i
}

ns_write "</table>"

if {$i == 0} {
    ns_write "no projects available for your viewing"
}



ns_write "
</blockquote>


[ad_footer]
"
