# This is the target program for the form in room-add.tcl.

set_the_usual_form_variables 0

# purpose, title, user_classes, significant_new_capabilities, feature_list_complete, feature_list_ranking, dependencies, minimum_launchable_feature_set, promotion, name, email, phone1, phone2, phone3

# checks to make sure title, name, and email were filled out properly.
set exception_count 0
set exception_text ""

if {![info exists title] || [empty_string_p $title] } {
    incr exception_count
    append exception_text "<li>Please enter a proposal title."
}

if {![info exists name] || [empty_string_p $name] } {
    incr exception_count
    append exception_text "<li>Please enter a name."
}

if {![info exists email] || [empty_string_p $email]} {
    incr exception_count
    append exception_text "<li>Please enter an email address."
} elseif { ![philg_email_valid_p $email] } {
    incr exception_count
    append exception_text "<li>Your email address doesn't look right to us.  Here are examples
of full Internet email addresses:

<ul>
<li>joe123@aol.com
<li>bill.baguette@isp.fr
<li>123.456@compuserve.com
</ul>
"
}

if {![info exists phone1] || [empty_string_p $phone1]} {
    if {![empty_string_p $phone2]} {
	incr exception_count
	append exception_text "<li>Please make sure your phone information is filled in completely."
    } elseif {![empty_string_p $phone3]} {
	incr exception_count
	append exception_text "<li>Please make sure your phone information is filled in completely."
    }
} else {
    if {![info exists phone2] || [empty_string_p $phone2]} {
	incr exception_count
	append exception_text "<li>Please make sure your phone information is filled in completely."
    } elseif {![info exists phone3] || [empty_string_p $phone3]} {
	incr exception_count
	append exception_text "<li>Please make sure your phone information is filled in completely."
    }
}

if { $exception_count > 0 } {
    ad_return_complaint $exception_count $exception_text
    return
}

ReturnHeaders

ns_write "
[ad_header "Thank You For Submitting Your Proposal"]

<font size=2 face=\"verdana, arial, helvetica\">
<img src=\"/graphics/ad_logo_big.gif\" height=88 width=95 vspace=10> 
<br>
<a href=\"/index.adp\">Home</a>&nbsp;&nbsp;&gt;&nbsp;&nbsp;Proposal
<blockquote>
<h3>Thank You For Submitting Your Proposal</h3>

"

# does the actual insertion into the database table, proposals

set phone "${phone1}-${phone2}-${phone3}"

set db [ns_db gethandle]

ns_db dml $db "begin transaction"

if [catch {ns_ora clob_dml $db "
INSERT into proposals
(proposal_id, purpose, title, user_classes, significant_new_capabilities, feature_list_complete, feature_list_ranking, dependencies, minimum_launchable_feature_set, promotion, name, email, phone, date_submitted)
VALUES
($proposal_id, '$QQpurpose', '$QQtitle', empty_clob(), empty_clob(), empty_clob(), empty_clob(), empty_clob(), empty_clob(), empty_clob(), '$QQname', '$QQemail', '[DoubleApos $phone]', sysdate)
RETURNING
user_classes, significant_new_capabilities, feature_list_complete, feature_list_ranking, dependencies, minimum_launchable_feature_set, promotion
INTO
:1, :2, :3, :4, :5, :6, :7" $user_classes $significant_new_capabilities $feature_list_complete $feature_list_ranking $dependencies $minimum_launchable_feature_set $promotion} errmsg] {
	# if it was not a double click, produce an error
	if { [database_to_tcl_string $db "select count(proposal_id) from proposals where proposal_id = $proposal_id"] == 0 } {
	    ad_return_error "Insert Failed" "We were unable to create your proposal record in the database.  Here's what the error looked like:
<blockquote>
<pre>
$errmsg
</pre>
</blockquote>"
    }
}


ns_db dml $db "end transaction"

set date_submitted [database_to_tcl_string $db "select sysdate from dual"]

ns_db releasehandle $db

ns_write "


Your proposal has been received!
<p>

<b>So what happens next?</b>
<p>
Someone from ArsDigita will respond to your proposal within the next few days, either to the email address or phone number you provided. 
<p>

Currently, demand is outpacing our ability to provide development services, so we are able to be extremely selective about the projects we take on. Prospects are compared using a variety of criteria, reflecting our assessment of their organization, stability, fit with aD, and commitment to the project.
Based upon our initial evaluation, a prospective project can be:

<ol>
<li>Assigned a team leader to create a <i>project plan</i></li>
<li>Assigned a team leader to review as a possible \"smaller\" project or MIT
class project - <i>(Note that this is a rare outcome, since we currently
do not have the resources to fragment teams for consulting purposes.)</i></li>
<li>Deferred pending additional information</li>
<li>Referred elsewhere</li>
</ol>

The decision to create a project plan is not made lightly, as such a plan 
is labor-intensive and time-consuming. A team leader must review the information
obtained from the client, identify and resolve any ambiguities, and review
specifications on any outside systems or data sources. Concurrently,
staffing must be carefully weighed so we can be certain that we can honor
the start, development, and launch dates laid out in the plan. 
<p>

Once complete, the project plan serves as the roadmap for development and the
long term relationship with the client. It sets out dates for start
and launch and itemizes deliverables.
<p>

Prior to committing to create this document for the client,
we try to re-confirm:

<ul>
<li>They have budgeted at least enough to cover our average costs</li>
<li>They can come to a decision, sign a contract, and send a check within 5 business days of receipt</li>
</ul>

If the client cannot decide in that time period, we either re-confirm
that they understand that the dates presented on the project plan will
not be valid, or we (politely) inform them that we prefer to hold off creating
the project plan until they are ready to make a decision.
<p>

<b>In the meantime...</b>
<p>
Feel free to send email to <cite><a href=\"mailto:sales@arsdigita.com\"><b>sales@arsdigita.com</b></a></cite> regarding the status of your proposal.
<p>

</blockquote>
<hr><cite>&copy;1999-2000&nbsp;<a href=\"/pages/about.html\">ArsDigita Corporation</a></cite>
</body>
</html>

"

ns_sendmail sales@arsdigita.com "$email" "New proposal on intranet" "

Purpose: $purpose

Title: $title

User classes: $user_classes

Significant new capabilities; $significant_new_capabilities

Complete feature list: $feature_list_complete 

Feature list ranking: $feature_list_ranking 

Dependencies: $dependencies

Minimum launchable feature set: $minimum_launchable_feature_set

Promotion: $promotion

Name: $name

Email: $email

Phone: $phone

Date: $date_submitted

View this proposal at [ad_parameter SystemURL]/proposals/admin/one.tcl?[export_url_vars proposal_id]
"
