set_the_usual_form_variables
# module_id package_id

validate_integer package_id $package_id

set db [ns_db gethandle]

set_simple_user_information $db

if {![user_can_see_package_p $db $user_id $package_id]} {
    sdm_return_access_complaint
    return
}

set_simple_package_information $db $package_id

if {[info exists module_id]} {
    validate_integer module_id $module_id

    set module_name [database_to_tcl_string $db "select module_name from modules where module_id=$module_id"]
    set menu_bar [ad_context_bar_ws_or_index [sdm_home_context_bar_item] [sdm_package_context_bar_item $package_id $package_name] [sdm_module_context_bar_item $module_id $module_name] "Closed Bugs and Features"]
    set extra_sql "and module_id=$module_id"
} else {
    set menu_bar [ad_context_bar_ws_or_index [sdm_home_context_bar_item] [sdm_package_context_bar_item $package_id $package_name] "Closed Bugs and Features"]
    set extra_sql ""
}    

ReturnHeaders

ns_write "[sdm_header "Closed Bugs and Features"]
<h2>Closed Bugs and Features</h2>
$menu_bar
<hr><p>

You can check the <A href=open-bafs.tcl?[export_url_vars module_id package_id]>open bugs and features</a> for this package, instead.<p>

[html_section_title "Bugs"]
<p>
<blockquote>
<table noborder cellspacing=0 cellpadding=2>
<tr>
<th bgcolor=lightblue>Bug #</th><th bgcolor=lightblue>Rating</th><th bgcolor=lightblue># Interested</th><th bgcolor=lightblue>Severity</th><th bgcolor=lightblue>Module</th><th bgcolor=lightblue>Completion</th><th bgcolor=lightblue>Description</th>
</tr>
"

set selection [ns_db select $db "select baf_id, severity, substr(bugs.description,0,200) as short_description, baf_module(baf_id) as module_name, completion as fix_release_id, fetch_release_name(completion) as fix_release_name, baf_rating(baf_id) as rating, baf_n_interested(baf_id) as n_interested from closed_bugs bugs where bugs.package_id=$package_id $extra_sql order by rating desc, n_interested desc"]

set odd_row 0

while {[ns_db getrow $db $selection]} {
    set_variables_after_query
    
    # shading
    set odd_row [expr 1 - $odd_row]
    if {$odd_row} {
	set shading "bgcolor=#cccccc"
    } else {
	set shading ""
    }

    ns_write "<tr><td $shading><a href=one-bug.tcl?[export_url_vars baf_id]>$baf_id</a></td>
<td align=right $shading>&nbsp;$rating</td>
<td align=right $shading>&nbsp;$n_interested</td>
<td $shading>$severity</td>
<td $shading>&nbsp;$module_name</td>
<td $shading>$fix_release_name</td>
<td $shading>[ns_quotehtml $short_description] ...</td>
</tr>"
}

ns_write "</table></blockquote>

<p>
&nbsp;
<p>
[html_section_title "Features"]
<p>
<blockquote>
<table cellspacing=0 cellpadding=2>
<tr>
<th bgcolor=lightblue>Feature #</th><th bgcolor=lightblue>Rating</th><th bgcolor=lightblue># Interested</th><th bgcolor=lightblue>Severity</th><th bgcolor=lightblue>Completion</th><th bgcolor=lightblue>Description</th>
</tr>
"

set selection [ns_db select $db "select 
baf_id, severity,
baf_module(module_id) as module_name,
substr(features.description,0,200) as short_description, 
completion as fix_release_id, 
fetch_release_name(completion) as fix_release_name,
baf_rating(baf_id) as rating,
baf_n_interested(baf_id) as n_interested
from 
closed_features features
where features.package_id=$package_id $extra_sql order by severity desc"]

set odd_row 0

while {[ns_db getrow $db $selection]} {
    set_variables_after_query

    # shading
    set odd_row [expr 1 - $odd_row]
    if {$odd_row} {
	set shading "bgcolor=#cccccc"
    } else {
	set shading ""
    }

    ns_write "<tr><td $shading><a href=one-feature.tcl?[export_url_vars baf_id]>$baf_id</a></td>
<td align=right $shading>&nbsp;$rating</td>
<td align=right $shading>&nbsp;$n_interested</td>
<td $shading>$severity</td>
<td $shading>$fix_release_name</td>
<td $shading>[ns_quotehtml $short_description] ...</td>
</tr>"
}


ns_write "</table></blockquote>

<p>

[sdm_footer]
"