set_the_usual_form_variables
# baf_id

validate_integer baf_id $baf_id

set db [ns_db gethandle]

set selection [ns_db 1row $db "select module_id, package_id, baf_status.baf_status as baf_status, baf_status_id, baf_type, description, entered_by, severity, insertion_date, expected_completion, completion from bugs_and_features, baf_status where bugs_and_features.baf_status=baf_status.baf_status_id and baf_id=$baf_id"]

set_variables_after_query

set_simple_user_information $db

if {![user_can_see_package_p $db $user_id $package_id]} {
    sdm_return_access_complaint
    return
}

if {$module_id != "" && $module_id!=0 } {
    if {![user_can_see_module_p $db $user_id $module_id]} {
	sdm_return_access_complaint
	return
    }
    set_simple_module_information $db $module_id
} else {
    set module_name ""
}

set_simple_package_information $db $package_id

set selection [ns_db 1row $db "select first_names as entered_by_first_names, last_name as entered_by_last_name from users where user_id=$entered_by"]
set_variables_after_query

# Is user admin?
set admin_p [user_can_edit_baf_p $db $user_id $baf_id]

set has_audit_p [database_to_tcl_string $db "select case when count(*)>0 then 1 else 0 end from baf_audit where baf_id=$baf_id"]

set count [database_to_tcl_string $db "select count(*) from baf_comments where on_what_id=$baf_id"]

if {$count > 0} {
    if {$count > 1} {
	set comment_word "comments"
    } else {
	set comment_word "comment"
    }
    set comments_html "<a href=one-baf-comments.tcl?[export_url_vars baf_id]>$count $comment_word</a> | "
} else {
    set comments_html ""
}

if {$baf_status == "closed"} {
    set extra_menu [list "closed-bafs.tcl?[export_url_vars package_id]" "Closed Bugs and Features"]
} else {
    set extra_menu [list "open-bafs.tcl?[export_url_vars package_id]" "Open Bugs and Features"]
}

ReturnHeaders

ns_write "[sdm_header "a $baf_type"]
<h2>$baf_type #$baf_id</h2>
[ad_context_bar_ws_or_index [sdm_home_context_bar_item] [sdm_package_context_bar_item $package_id $package_name] $extra_menu "$baf_type #$baf_id"]
<hr><p>
<blockquote>
[util_convert_plaintext_to_html $description]
<p>
<font size=-1>
$comments_html <a href=baf-comment-add.tcl?[export_url_vars baf_id]>Add a Comment</a>
</font>
</blockquote><p>
<center>
<table noborder>
<tr>
<th bgcolor=lightgrey>Your Interests</th>
<th bgcolor=lightgrey>$baf_type information</th>
"

ns_write "
</tr>
<tr>
<td valign=top>
<center>
<table noborder width=80%>
<tr><td>
"

if {![sdm_user_has_rated_baf_p $user_id $baf_id $db]} {
    ns_write "<FORM METHOD=GET action=pvt/baf-rate.tcl>
[export_form_vars baf_id]
Rate this $baf_type<br>
(1 is least important, 9 is most important): [make_html_select rating {1 2 3 4 5 6 7 8 9} 5]
<INPUT TYPE=submit value=rate>
</FORM><p>
"
} else {
    ns_write "Rating:</td><td align=right><font size=+1><b>[sdm_baf_rating $baf_id $db]</b></font><p>"
}

ns_write "
</td>
</tr>
<tr>
<td colspan=2 align=center>
<FORM METHOD=GET ACTION=pvt/toggle-baf-interest.tcl>
[export_form_vars baf_id]
"

if {![sdm_baf_user_is_interested_p $baf_id $user_id $db]} {
    ns_write "<INPUT TYPE=hidden name=interest value=1>
<INPUT TYPE=submit value=\"I'm Interested\">"
} else {
    ns_write "<INPUT TYPE=hidden name=interest value=0>
<INPUT TYPE=submit value=\"I'm not Interested\">"
}

ns_write "
<br>
([sdm_baf_n_interested_users $baf_id $db] interested users)
</FORM>
</td>
</tr>
</table>
</td>
"

ns_write "
<td valign=top>
"

if {$has_audit_p} {
    ns_write "<div align=right>\[<a href=baf-audit.tcl?[export_url_vars baf_id]>audit</a>\]</div><br>"
}

if {!$admin_p || $baf_status_id==[sdm_closed_baf_status_id] || $baf_status_id==[sdm_reopened_baf_status_id]} {
ns_write "
<table>
<tr>
<td><i>Module</i></td>
<td>$module_name</td>
</tr>
<tr>
<td><i>Discovered</i></td>
<td>[util_AnsiDatetoPrettyDate $insertion_date]</td>
</tr>
<tr>
<td><i>Severity</i></td>
<td>$severity</td>
</tr>
<tr>
<td><i>Status</i></td>
<td>$baf_status"

# If we're the admins, but the baf is closed,
# we can reopen it.
if {$admin_p && $baf_status_id==[sdm_closed_baf_status_id]} {
    ns_write " <font size=-1>(<a href=pvt/baf-reopen.tcl?[export_url_vars baf_id]>reopen</a>)</font>"
}

if {$admin_p && $baf_status_id==[sdm_reopened_baf_status_id]} {
    set new_baf_id [database_to_tcl_string $db "select baf_id from bugs_and_features where old_baf_id=$baf_id"]
    ns_write " <font size=-1><a href=one-baf.tcl?baf_id=$new_baf_id>reopened...</a></font>"
}

ns_write "</td>
</tr>
<tr>
<td><i>Entered by</i></td>
<td><a href=one-user.tcl?user_id=$entered_by>$entered_by_first_names $entered_by_last_name</a>
</td>
</tr>
<tr>
<td><i>Assigned to</i></td>
<td>
<a href=baf-assignments.tcl?[export_url_vars baf_id]>[database_to_tcl_string $db "select sdm_baf_assigned($baf_id)"]</a>
</td>
</tr>
<tr>
<td><i>Expected Completion</i></td>
<td><a href=one-package-release.tcl?release_id=$expected_completion&package_id=$package_id>[database_to_tcl_string_or_null $db "select release_name(major_version,minor_version,patch_version,beta_version) from package_releases where release_id=[db_postgres_null_sql $expected_completion] and package_id=$package_id"]</a></td>
</tr>
<tr>
<td><i>Actual Completion</i></td>
<td><a href=one-package-release.tcl?release_id=$completion&package_id=$package_id>[database_to_tcl_string_or_null $db "select release_name(major_version,minor_version,patch_version,beta_version) from package_releases where release_id=[db_postgres_null_sql $completion] and package_id=$package_id"]</a></td>
</tr>
"


if {$baf_type == "bug"} {
    ns_write "<tr><td><i>Releases Affected</i></td>
<td>"

set selection [ns_db select $db "select release_name(major_version,minor_version,patch_version,beta_version) as release_name, release_id, package_id from package_releases where release_id in (select release_id from bug_release_map where bug_id=$baf_id)"]

while {[ns_db getrow $db $selection]} {
    set_variables_after_query

    ns_write "<a href=one-package-release.tcl?[export_url_vars release_id package_id]>$release_name</a> "
}

ns_write "</td></tr>"
}

} else {
    set list_of_modules [database_to_tcl_list_list $db "select module_id, module_name from modules where package_id=$package_id"]
    lappend list_of_modules [list "" "(No Module)"]
## Admin
set html_form "
<FORM METHOD=POST action=pvt/baf-edit.tcl>
[export_form_vars baf_id]
<table>
<tr>
<td><i>Module</i></td>
<td>[make_html_select module_id $list_of_modules $module_id]</td>
</tr>
<tr>
<td><i>Type</it></td>
<td>[make_html_select baf_type [sdm_list_of_baf_types] $baf_type]</td>
</tr>
<tr>
<td><i>Discovered</i></td>
<td>[util_AnsiDatetoPrettyDate $insertion_date]</td>
</tr>
<tr>
<td><i>Severity</i></td>
<td>[make_html_select severity [sdm_list_of_severities] $severity]</td>
</tr>
<tr>
<td><i>Status</i></td>
<td>[make_html_select baf_status [database_to_tcl_list_list $db "select baf_status_id, baf_status from baf_status"] $baf_status_id]</td>
</tr>
<tr>
<td><i>Entered by</i></td>
<td><a href=one-user.tcl?user_id=$entered_by>$entered_by_first_names $entered_by_last_name</a>
</td>
</tr>
<tr>
<td><i>Assigned to</i></td>
<td>
<a href=baf-assignments.tcl?[export_url_vars baf_id]>[database_to_tcl_string $db "select sdm_baf_assigned($baf_id)"]</a>
</td>
</tr>
<tr>
<td><i>Expected Completion</i></td>
"

set list_of_release_values [database_to_tcl_list_list $db "select release_id, release_name(major_version,minor_version,patch_version,beta_version) as release_name from package_releases where package_id=$package_id"]

lappend list_of_release_values [list "" "(Not Yet)"]

if {$completion == 0} {
    set completion ""
}

if {$completion != ""} {
    append html_form "
<td><a href=one-package-release.tcl?release_id=$expected_completion&package_id=$package_id>[database_to_tcl_string_or_null $db "select release_name(major_version,minor_version,patch_version,beta_version) from package_releases where release_id=[db_postgres_null_sql $expected_completion] and package_id=$package_id"]</a></td>"
} else {
    append html_form "
<td>[make_html_select expected_completion $list_of_release_values $expected_completion]</td>
"
}

append html_form "</tr>
<tr>
<td><i>Actual Completion</i></td>
<td>"

if {$completion != ""} {
append html_form "<A href=one-package-release.tcl?release_id=$completion>[database_to_tcl_string $db "select fetch_release_name($completion)"]</a>" 
}

append html_form "</td>
</tr>
"

if {$baf_type == "bug"} {
    append html_form "<tr><td><i>Releases Affected</i></td>
<td>"

set selection [ns_db select $db "select release_name(major_version,minor_version,patch_version,beta_version) as release_name, release_id, package_id from package_releases where release_id in (select release_id from bug_release_map where bug_id=$baf_id)"]

while {[ns_db getrow $db $selection]} {
    set_variables_after_query

    append html_form "<a href=one-package-release.tcl?[export_url_vars release_id package_id]>$release_name</a> "
}

}

ns_write "$html_form</td>
</tr>
<tr>
<td colspan=2 align=center>
<INPUT TYPE=submit value=edit>
</FORM>
</td></tr>"

}


ns_write "</table></td>
</tr>
</table>
</center>
<p>
"

ns_write "<p>
</blockquote>
[sdm_footer]
"