set_the_usual_form_variables
# release_id

validate_integer release_id $release_id

set db [ns_db gethandle]

set_simple_user_information $db

set package_id [database_to_tcl_string $db "select package_id from package_releases where release_id=$release_id"]

if {![user_can_see_package_p $db $user_id $package_id]} {
    sdm_return_access_complaint
    return
}

set_simple_package_information $db $package_id

# Check if the user is the manager for this release
set release_manager_user_id [database_to_tcl_string $db "select manager from package_releases where package_id=$package_id and release_id=$release_id"]

if {$user_id == $release_manager_user_id} {
    set user_is_manager_p 1
} else {
    set user_is_manager_p 0
}

set selection [ns_db 1row $db "select release_name(major_version,minor_version,patch_version,beta_version) as release_name, general_description, release_filename, anticipated_release_date, release_date, user_id as manager_user_id, email as manager_email, first_names as manager_first_names, last_name as manager_last_name, supported_platforms from package_releases, users where package_id=$package_id and release_id=$release_id and manager=user_id"]
set_variables_after_query

ReturnHeaders

ns_write "[sdm_header "A release"]
<h2>$package_name: Release $release_name</h2>
[ad_context_bar_ws_or_index [sdm_home_context_bar_item] [sdm_package_context_bar_item $package_id $package_name] $release_name]
<hr><p>
"

ns_write "
<center>
<table border=1 cellpadding=4>
<tr>
<td><i>Manager</i></td>
<td><A href=one-user.tcl?user_id=$release_manager_user_id>$manager_first_names $manager_last_name</a> ($manager_email)</td>
</tr>
<tr>
<td><i>Anticipated Release Date</i></td>
<td>[util_AnsiDatetoPrettyDate $anticipated_release_date]</td>
</tr>
<tr>
<td><i>Release Date</i></td>
<td>[util_AnsiDatetoPrettyDate $release_date]</td>
</tr>
<tr>
<td><i>Supported Platforms</i></td>
<td>$supported_platforms</td>
</tr>
</table>
</center>
<p>
<blockquote>
$general_description
</blockquote><p>
"

if {$release_date != "" && $release_filename!=""} {
    ns_write "
<center><form method=get action=download-package-release.tcl>
[export_form_vars release_id package_id]
<input type=submit value=download>
</FORM>
[database_to_tcl_string $db "select count(*) from package_release_downloads where package_id=$package_id and release_id=$release_id"] downloads<br>
(<a href=package-release-notes.tcl?[export_url_vars package_id release_id]>Release Notes</a>)
</center>
<p>
"
}

# ns_write "<p><hr width=70%><p>"

# Administration Section
if {$user_is_manager_p} {
    ns_write "[html_section_title "Administration"]
<ul>
<li> <a href=pvt/package-release-notes-edit.tcl?[export_url_vars package_id release_id]>Edit Release Notes</a>
"
    if {$release_date == ""} {
	ns_write "
<FORM method=post action=pvt/package-release-anticipated-release-date.tcl>
<li> <a href=pvt/package-release-edit.tcl?[export_url_vars release_id package_id]>edit</a> release
<li> Set Anticipated Release Date:
[export_form_vars release_id package_id]
[philg_dateentrywidget anticipated_release_date [database_to_tcl_string $db "select sysdate()::date"]]
<INPUT TYPE=submit value=set>
</FORM>
<li> <A href=pvt/package-release-make-live.tcl?[export_url_vars package_id release_id]>Make this release live</a>
"
} else {
    set active_release_id [database_to_tcl_string $db "select current_release from packages where package_id=$package_id"]

    if {$active_release_id != $release_id} {
	ns_write "
<li> <a href=pvt/package-release-choose-live.tcl?[export_url_vars release_id]>make this release the live one</a>.\n"
}
    
    set selection [ns_db select $db "select release_id as new_release_id, release_name(major_version, minor_version, patch_version, beta_version) as new_release_name from package_releases where release_date is NOT NULL and package_id=$package_id and release_id <> $release_id"]

    set release_select_html "<SELECT name=new_release_id>
    <OPTION SELECTED VALUE=\"\"> No current release
    "
    
    while {[ns_db getrow $db $selection]} {
	set_variables_after_query

	append release_select_html "<OPTION VALUE=$new_release_id> $new_release_name\n"
    }

    append release_select_html "</SELECT>"
    
    ns_write "
<FORM METHOD=POST action=pvt/package-release-make-unlive.tcl>
<li> If you want to take back this release so that it not be live anymore, you can choose to do so by picking a past release to replace this one as the live one:
[export_form_vars package_id release_id]
$release_select_html
<INPUT TYPE=submit value=go>
</FORM>
"
}
ns_write "
</ul>
<p>
"
}

set selection [ns_db 1row $db "select major_version,minor_version,patch_version,beta_version from package_releases where package_id=$package_id and release_id=$release_id"]
set_variables_after_query

# Get the features that were just implemented in this release.
set selection [ns_db select $db "select baf_id, substr(description,0,100) as short_description from features, package_releases where features.completion=package_releases.release_id and package_releases.release_id=$release_id"]

set flag 0

while {[ns_db getrow $db $selection]} {
    if {!$flag} {
	ns_write "
	<h3>Implemented Features</h3>
	<ul>
	"
	set flag 1
    }

    set_variables_after_query

    ns_write "<li> <A href=one-feature.tcl?[export_url_vars baf_id]>Feature #$baf_id</a>: $short_description ...\n"
}

ns_write "</ul><P>"

# Get the bugs that are fixed in this release
set selection [ns_db select $db "select baf_id, substr(description,0,100) as short_description from bugs, package_releases where bugs.completion=package_releases.release_id and package_releases.release_id=$release_id"]

set flag 0

while {[ns_db getrow $db $selection]} {
    if {!$flag} {
	ns_write "
	<h3>Fixed Bugs</h3>
	<ul>
	"
	set flag 1
    }

    set_variables_after_query

    ns_write "<li> <A href=one-bug.tcl?[export_url_vars baf_id]>Bug #$baf_id</a>: $short_description ...\n"
}

ns_write "</ul><P>"



set selection [ns_db select $db "select baf_id, substr(description,0,100) as short_description from bugs, package_releases, bug_release_map where bugs.baf_id=bug_release_map.bug_id and package_releases.release_id=$release_id and package_releases.package_id=$package_id and package_releases.release_id=bug_release_map.release_id"]

set flag 0

while {[ns_db getrow $db $selection]} {
    if {!$flag} {
	ns_write "
	<h3>Known Bugs</h3>
	<ul>
	"
	set flag 1
    }

    set_variables_after_query

    ns_write "<li> <A href=one-bug.tcl?[export_url_vars baf_id]>Bug #$baf_id</a>: $short_description ...\n"
}

ns_write "</ul>

<p>

[sdm_footer]
"
