set_the_usual_form_variables
# package_id
# possibly full_entry_path

validate_integer package_id $package_id

set db [ns_db gethandle]

set_simple_user_information $db


if {![user_can_see_package_p $db $user_id $package_id]} {
    sdm_return_access_complaint
    return
}

if {![sdm_sourcecode_package_has_repository_p $db $package_id]} {
    ad_return_complaint 1 "Package does not have a repository"
    return
}

set_simple_package_information $db $package_id

set_simple_repository_information $db $package_id

if {![info exists current_entry]} {
    set current_entry [sdm_sourcecode_make_entry DIRECTORY {} $repository_name]
}

if {![sdm_sourcecode_entry_safe $current_entry $repository_name]} {
    ns_return 200 text/html "$current_entry"
    return
}

ReturnHeaders

ns_write "[sdm_header "Repository: $package_name"]
<h2>Source Repository</h2>
[ad_context_bar_ws_or_index [sdm_home_context_bar_item] [sdm_package_context_bar_item $package_id $package_name] "Source Code Repository"]
<hr><p>
"

if {[user_can_edit_package_p $db $user_id $package_id]} {
    ns_write "<ul>
<li> <a href=pvt/package-repository-edit.tcl?[export_url_vars package_id]>edit repository properties</a>
<li> <a href=pvt/package-download-source.tcl?[export_url_vars package_id]>get new source from CVS</a>
</ul><p>"
}

ns_write "<h3>"



if {[sdm_sourcecode_entry_has_parent $current_entry]} {
    set parent_entry [sdm_sourcecode_entry_parent_entry $current_entry]
    ns_write "<a href=package-repository.tcl?current_entry=[ns_urlencode $parent_entry]&[export_url_vars package_id]>[sdm_sourcecode_entry_path $parent_entry]/[sdm_sourcecode_entry_name $parent_entry]</a>"
}

ns_write "/[sdm_sourcecode_entry_name $current_entry]</h3>"

if {[sdm_sourcecode_entry_type $current_entry] == "DIRECTORY"} {
    
    # ns_write "<ul>"
    ns_write "
    <table boder=0>
    <tr><th>File</th><th>current</th><th>Last Updated</th><th>Author</th><th>Comments</th></tr>
    "

    set light_background "eeeeee"
    set dark_background "cccccc"
    set bgs [list $light_background $dark_background]

    set counter 0

    # list that directory
    set list_of_entries [sdm_sourcecode_list_entries $current_entry $file_glob_patterns]
    
    foreach entry $list_of_entries {
	#ns_write "<li> "
	set counter [expr "1 - $counter"]
	
	ns_write "<tr bgcolor=#[lindex $bgs $counter]>"
	set current_entry $entry

	if {[sdm_sourcecode_entry_type $entry] == "DIRECTORY"} {
	    ns_write " <td><a href=\"package-repository.tcl?[export_url_vars current_entry package_id]\">[sdm_sourcecode_entry_name $entry]</a>/</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>"
	} else {
	    ns_write " <td><a href=\"package-repository.tcl?[export_url_vars current_entry package_id]\">[sdm_sourcecode_entry_name $entry]</a></td>"

	    set cvsroot [cvs_get_cvsroot $package_id $db]

	    set history [cvs_file_get_history [sdm_source_root][sdm_sourcecode_entry_path $entry] [sdm_sourcecode_entry_name $entry] $cvsroot]

	    set hist [lindex $history 0]
	    
	    ns_write "<td>[cvs_history_get_revision $hist]</td><td>[cvs_history_get_date $hist]</td><td>[cvs_history_get_author $hist]</td><td>[cvs_history_get_comments $hist]</td>"
	}

	ns_write "</tr>"

    }
    
    ns_write "</table>"
} else {
    set cvsroot [cvs_get_cvsroot $package_id $db]

    set history [cvs_file_get_history [sdm_source_root][sdm_sourcecode_entry_path $current_entry] [sdm_sourcecode_entry_name $current_entry] $cvsroot]

    if {![info exists tag]} {
	set tag [cvs_history_get_revision [lindex $history 0]]
    }
	
    set version_html ""
    set list_of_other_tags [list]

    foreach hist $history {
	if {[cvs_history_get_revision $hist] == $tag} {
	    set current_hist $hist
	    
	    append version_html "<b>[cvs_history_get_revision $hist]</b> &nbsp "
	} else {
	    append version_html "<a href=package-repository.tcl?[export_url_vars package_id current_entry]&tag=[cvs_history_get_revision $hist]>[cvs_history_get_revision $hist]</a> &nbsp "
	    lappend list_of_other_tags [cvs_history_get_revision $hist]
	}
    }

    set new_release $tag

    ns_write "
<table border=0>
<tr><td>Revision</td><td>$version_html</td></tr>
<tr><td>Author</td><td>[cvs_history_get_author $current_hist]</td></tr>
<tr><td valign=top>Comments</td><td>[cvs_history_get_comments $current_hist]</td></tr>
</table>
<p>

<FORM method=get action=entry-history-diff.tcl>
[export_form_vars current_entry new_release package_id]
Diff this with: [make_html_select old_release $list_of_other_tags]<br>
<INPUT TYPE=submit value=diff>
</FORM>
<p>
<a href=one-file-annotate.tcl?[export_url_vars package_id current_entry]>annotate this file</a>
<p>
"

    # This should be abstracted
    set raw_contents [ns_quotehtml [cvs_checkout_to_string [sdm_sourcecode_entry_fullpath $current_entry] $cvsroot $tag]]
    set colorized_contents [sdm_sourcecode_colorize [sdm_sourcecode_entry_name $current_entry] $raw_contents]
    set colorized_contents_with_line_numbers [sdm_sourcecode_add_lines_numbers $colorized_contents]

    ns_write "<blockquote><pre>
$colorized_contents_with_line_numbers
</pre>
</blockquote>
"
}
	
ns_write "[sdm_footer]"