set_the_usual_form_variables
# package_id, module_id, baf_type
# possibly old_baf_id

validate_integer package_id $package_id

if {![info exists module_id]} {
    set module_id ""
}

validate_integer_or_null module_id $module_id

set db [ns_db gethandle]

# If we're reopening something, let's pull some data out
if {[info exists old_baf_id]} {
    validate_integer old_baf_id $old_baf_id
    set selection [ns_db 1row $db "select package_id, module_id, baf_type from bugs_and_features where baf_id=$old_baf_id"]
    set_variables_after_query
}

set_simple_user_information $db

if {![user_can_see_package_p $db $user_id $package_id]} {
    sdm_return_access_complaint
    return
}

if {$module_id != "" && ![user_can_see_module_p $db $user_id $module_id]} {
    sdm_return_access_complaint
    return
}

set_simple_package_information $db $package_id

if {$module_id != ""} {
    set_simple_module_information $db $module_id
}

ReturnHeaders

ns_write "[sdm_header "Enter a new $baf_type"]
<h2>Enter a new $baf_type</h2>
[ad_context_bar [sdm_home_context_bar_item] [sdm_package_context_bar_item $package_id $package_name] "new $baf_type"]
<hr><p>
"

set baf_id [database_to_tcl_string $db "select baf_id_sequence.nextval from dual"]

set list_of_modules [database_to_tcl_list_list $db "select module_id, module_name from modules where package_id=$package_id order by module_name"]

lappend list_of_modules [list "" "(Module Unknown)"]

if {$baf_type == "bug"} {

    set list_of_releases [database_to_tcl_list_list $db "select release_id, release_name(major_version,minor_version,patch_version,beta_version) as release_name from package_releases where package_id=$package_id order by release_name"]

    set selected_release_name [database_to_tcl_string_or_null $db "select release_id from package_releases where release_date is not null order by release_date desc limit 1"]

    set list_of_inputs [list \
	    [list release_id {Release where bug discovered} select $list_of_releases $selected_release_name] \
	    [list module_id {Module} select $list_of_modules $module_id] \
	    [list description {Description of bug} textarea 80 20 soft]]
} else {
    set list_of_inputs [list \
	    [list module_id {Module} select $list_of_modules $module_id] \
	    [list description {Description of feature} textarea 60 5 soft]]
}

lappend list_of_inputs [list baf_id "" hidden $baf_id]
lappend list_of_inputs [list baf_type "" hidden $baf_type]
lappend list_of_inputs [list package_id "" hidden $package_id]

if {[info exists old_baf_id]} {
    lappend list_of_inputs [list old_baf_id "" hidden $old_baf_id]
}

ns_write "
[make_html_form POST new-baf-2.tcl $list_of_inputs]
"

ns_write "<p>

[sdm_footer]"