# Ticket tracker admin user home page
#
# index.tcl by hqm@arsdigita.com June 1999
# 
set_form_variables 0

# form vars:
# 
# (these are all optional args which have defaults)
#
# filter conditions
#
# Assignments:
# view_assignment    { user unassigned all }

# Status
# view_status  { open closed deferred created_by_you }
#
# Creation time
# view_created { last_24 last_week last_month all}
#
#
# order_by       column name to sort table by 

set ctrlvars {view_assignment view_status view_created}

set db [ticket_getdbhandle]

set user_id [ad_get_user_id]

ReturnHeaders

ns_write "[ad_header "[ticket_system_name] Home"]

<h2>[ticket_system_name]</h2>

[ad_context_bar_ws_or_index "Ticket Tracker"]

<hr>
"

# List of form vars used to select tickets to display

# Assignment filter conditions
#

if {![info exists view_assignment]} {
    # default to show issues assigned to $user_id
    set view_assignment user
}

# default to only your own assigned issues
if {![info exists view_status]} {
    set view_status open
}

# default to all time
if {![info exists view_created]} {
    set view_created all
}


switch $view_assignment {
    "user" { 
	set assignment_filter "and (exists (select msg_id from ticket_issue_assignments 
where ticket_issue_assignments.msg_id = ticket_issues.msg_id
and ticket_issue_assignments.user_id = $user_id
)) " }

     "all" { set assignment_filter "" }
}

switch $view_status {
    "open" {set status_filter "and ((status <> 'closed') and (status <> 'deferred')) " }
    "deferred" {set status_filter "and ((status <> 'closed') or (status = 'deferred')) " }
    "closed" {set status_filter "" } # shows everything
}

switch $view_created {
   "last_24"   { set date_filter "and (posting_time > (sysdate - 1))" } 
   "last_week" { set date_filter "and (posting_time > (sysdate - 7)) " } 
   "last_month" { set date_filter "and (posting_time > (sysdate - 30)) " }
   "all" { set date_filter "" }
}


# Sort order of tickets by
if {![info exists order_by]} {
    set order_by "ticket_issues.msg_id"
    set sql_order_clause $order_by
} elseif {[string match "project*" $order_by]} {
    set sql_order_clause "project_title"
} elseif {[string compare "severity" $order_by] == 0} {
    set sql_order_clause "decode(severity[severity_decode_list])"
} elseif {[string compare "severity desc" $order_by] == 0} {
    set sql_order_clause "decode(severity[severity_decode_list]) desc"
} elseif {[string match "assigned_p desc" $order_by]} {
    set sql_order_clause "upper(assigned_user_email)"
} elseif {[string match "assigned_p" $order_by]} {
    set sql_order_clause "upper(assigned_user_email) desc"
} else {
    set sql_order_clause $order_by
}


################################################################
# GUI ticket filter controls

# List of all the state vars we need to pass through these toggle switches
set filter_vars {view_assignment view_status view_created order_by}


append results "<table border=0 cellspacing=0 cellpadding=0 width=100%>
<tr><th bgcolor=#ECECEC>Ticket Assignment</th>
<th bgcolor=#ECECEC>Status</th>
<th bgcolor=#ECECEC>Creation Time</th></tr>"


#### Assignment flags
# Show assigned to you 
append results "<tr><td align=center>\["

append results [ticket_control_vars view_assignment user $filter_vars "mine"]
append results " | "
# Show all tickets 
append results [ticket_control_vars view_assignment all $filter_vars "everyone's"]

#### Status flags
append results "\]</td>\n<td align=center>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\["

# Show open issues
append results [ticket_control_vars view_status open $filter_vars "active"]
append results " | "
# Show deferred issues
append results [ticket_control_vars view_status deferred $filter_vars "+deferred"]
append results " | "
# Show closed issues
append results [ticket_control_vars view_status closed $filter_vars "+closed"]


#### Creation time filter
append results "\]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>\n<td align=center>\["

append results [ticket_control_vars view_created last_24 $filter_vars "last 24 hrs"]
append results " | "
append results [ticket_control_vars view_created last_week $filter_vars "last week"]
append results " | "
append results [ticket_control_vars view_created last_month $filter_vars "last month"]
append results " | "
append results [ticket_control_vars view_created all $filter_vars "all"]

append results "\]</td></tr></table><p>"

################################################################



set last_priority "starting"

set count 0

append results "<table border=0>
<tr>
<th align=left><a href=\"index.tcl?order_by=[toggle_order ticket_issues.msg_id $order_by]&[eval export_url_vars $ctrlvars]\">ID#</a></th>
<th align=left><a href=\"index.tcl?order_by=[toggle_order priority $order_by]&[eval export_url_vars $ctrlvars]\">Pri</a></th>
<th align=left><a href=\"index.tcl?order_by=[toggle_order ticket_type $order_by]&[eval export_url_vars $ctrlvars]\">Typ</a></th>
<th align=left><a href=\"index.tcl?order_by=[toggle_order email $order_by]&[eval export_url_vars $ctrlvars]\">Creator</a></th>
<th align=left><a href=\"index.tcl?order_by=[toggle_order status $order_by]&[eval export_url_vars $ctrlvars]\">Stat</a></th>
<th align=left><a href=\"index.tcl?order_by=[toggle_order severity $order_by]&[eval export_url_vars $ctrlvars]\">Sever</a></th>
<th align=left><a href=\"index.tcl?order_by=[toggle_order posting_time $order_by]&[eval export_url_vars $ctrlvars]\">Creat</a></th>
<th align=left><a href=\"index.tcl?order_by=[toggle_order modification_time $order_by]&[eval export_url_vars $ctrlvars]\">Mod</a></th>
<th align=left><a href=\"index.tcl?order_by=[toggle_order assigned_p $order_by]&[eval export_url_vars $ctrlvars]\">Asgn?</a></th>"

if { $view_status == "closed" } {
    append results "<th align=left><a href=\"index.tcl?order_by=[toggle_order close_date $order_by]&[eval export_url_vars $ctrlvars]\">Closed</a></th>"
} else {
    append results "<th align=left><a href=\"index.tcl?order_by=[toggle_order deadline $order_by]&[eval export_url_vars $ctrlvars]\">Deadline</a></th>"
}

append results "<th align=left><a href=\"index.tcl?order_by=[toggle_order one_line $order_by]&[eval export_url_vars $ctrlvars]\">Subject</a></th>
"
if {![string match "project_title" $order_by]} {
    append results "<th align=left><a href=\"index.tcl?order_by=[toggle_order project_id $order_by]&[eval export_url_vars $ctrlvars]\">Project</a></th>
"    
}
append results "</tr>\n"

set last_project_title ""

set query "select 
 ticket_issues.msg_id,
 ticket_issues.ticket_type, 
 ticket_issues.one_line,
 ticket_issues.status,
 ticket_issues.severity,
 ticket_issues.posting_time,
 users.email,
 assigned_users.email as assigned_user_email,
 ticket_projects.title as project_title,
 ticket_projects.project_id,
 ticket_issues.priority, 
 to_char(ticket_issues.modification_time, 'mm/dd/yy') as modification_time_pretty,
 to_char(ticket_issues.posting_time, 'mm/dd/yy') as creation_time_pretty,
 to_char(ticket_issues.close_date, 'mm/dd/yy') as close_date_pretty,
 to_char(ticket_issues.deadline, 'mm/dd/yy') as deadline_pretty,
 to_char(sysdate - deadline) as pastdue,
 ticket_issue_assignments.active_p as assigned_p,
 ticket_issue_assignments.user_id as assigned_user_id,
 ticket_issues.public_p,
 ticket_priorities.name as priority_name
from ticket_issues, ticket_priorities, ticket_projects, users, ticket_issue_assignments, users assigned_users
where ticket_priorities.priority = ticket_issues.priority
and users.user_id = ticket_issues.user_id
and ticket_projects.project_id = ticket_issues.project_id
and ticket_issues.msg_id = ticket_issue_assignments.msg_id(+)
and ticket_issue_assignments.user_id = assigned_users.user_id(+)
$assignment_filter
$status_filter
$date_filter
order by $sql_order_clause, ticket_issues.priority, ticket_issues.posting_time"

set selection [ns_db select $db $query]

set last_msg_id ""
set msg_ids {}

while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set cols {}
    if {$msg_id == $last_msg_id} {
	set assign $assigned_user_email
	regsub "@.*" $assign "" assign
	set cols [list "" "" "" "" "" "" "" "" "$assign" "" "" ""]
		
    } else {
	set last_msg_id $msg_id
	lappend msg_ids $msg_id
	
	lappend cols "<a href=\"issue-view.tcl?msg_id=$msg_id\">$msg_id</a>"
	lappend cols "P$priority"
	lappend cols "[string range $ticket_type 0 3]"
	regsub "@.*" $email "" email
	lappend cols "$email"
	if { [string compare $status "fixed waiting approval"] == 0 } {
	    lappend cols "<font color=#00ff00>(w approv)</font>"
	} elseif { [string compare $status "need clarification"] == 0 } {
	    lappend cols "<font color=#ff0000>nd clar</font>"
	} else {
	    lappend cols $status
	}
	lappend cols "$severity"
	lappend cols "$creation_time_pretty"
	lappend cols "$modification_time_pretty"
	
	if {$assigned_p == "t"} {
	    set assign $assigned_user_email
	    regsub "@.*" $assign "" assign
	    lappend cols "$assign"
	} else {
	    lappend cols ""
	}

	
	if {[info exists closed] && $view_closed == 1} {
	    lappend cols "$close_date_pretty" 
	} else {
	    if {$pastdue > 0} {
		lappend cols "<font color=red>$deadline_pretty</font>"
	    } else {
		lappend cols "$deadline_pretty"
	    }
	}
	
	lappend cols "<a href=\"issue-view.tcl?msg_id=$msg_id\">[clean_up_html $one_line]</a>"
	
	# show project title if we are not sorting by project
	if {![string match "project_title" $order_by]} {
	    lappend cols "<a href=project-top.tcl?project_id=$project_id>[string range $project_title 0 12]</a>"	
	}
	
	if {[string match "project_title" $order_by] && $last_project_title != $project_title} {
	    append results "<tr><th colspan=10 align=left><a href=project-top.tcl?project_id=$project_id>$project_title</a></th></tr>\n"
	    set last_project_title $project_title
	}
	
	incr count
	if {($count % 2) == 0} {
	    set bgcolor "bgcolor=\#ECECEC"
	} else {
	    set bgcolor ""
	}
	
    }
    append results "<tr $bgcolor>"
    foreach col $cols {
	append results "<td>$col&nbsp;</td>\n"
    }
    append results "</tr>"    
}

if { $count == 0 } {
    append results "<tr><td colspan=10 align=center>-- No issues --</td></tr>"   
}

append results "</table>\n<p>"


# added to keep the script from breaking when there are no tickets to display.
if { ![info exists project_id] } {
    set project_id ""
}




ns_write  "
$results
<ul>
<li>Summarize by 
    <a href=\"project-summary.tcl\">project</a> |
    <a href=\"user-summary.tcl\">user</a>

<li>View by 
   <a href=\"index.tcl?order_by=[toggle_order project_id $order_by]&[eval export_url_vars $ctrlvars]\">project</a> |
   <a href=\"project-view-assignments.tcl\">user</a>
   
<li><a href=\"issue-details.tcl?[export_url_vars msg_ids]&report_type=msgs_and_responses\">View issues above as full report</a>
<li><a href=\"issue-details.tcl?[export_url_vars msg_ids]&report_type=msgs\">View issues above as summary</a>

<p>

<li>Add new 
   <a href=\"project-new.tcl\">project</a> |
   <a href=\"issue-new.tcl\">issue</a>

<p>

<li>
<form action=ticket-search.tcl method=GET>
Quick Search:
<input type=text maxlength=100 name=query_string_1>
</form>

<li><form action=project-top.tcl method=get>
View Project: <select name=project_id>
    [ad_db_optionlist $db  "select title,
    project_id from ticket_projects
    order by title asc" $project_id]
    </select>
<input type=submit value=Go>
</form>



</ul>



<h3>Advanced Search</h3>

<blockquote>
<form action=ticket-search.tcl method=post>
[ticket_search_fragments]
<p>
<center>
<input type=submit name=submit value=\"Search\">
</center>

</form>
</blockquote>

[ad_footer]
"
