# This page is for viewing an issue as a privileged user - 

# Most data fields are modifiable

set_the_usual_form_variables

# msg_id

set db [ticket_getdbhandle]

set user_id [ad_get_user_id]

set selection [ns_db 0or1row $db "select 
to_char(posting_time, [ticket_date_format]) as posting_date,
deadline, one_line, message, priority, ticket_type,  email name, status
from ticket_issues, users
where msg_id = $msg_id
and users.user_id = ticket_issues.user_id"]

if { $selection == "" } {
    # message was probably deleted
    ad_return_complaint 1 "<li>Couldn't find message $msg_id.  Probably it was deleted by the forum maintainer."
    return
}

set_variables_after_query
set this_one_line $one_line


set selection [ns_db 1row $db "select 
 title, ticket_issues.project_id, notify_p, public_p,
 ticket_issues.last_modified_by, 
 ticket_issues.severity, 
 ticket_issues.data1, 
 ticket_issues.data2, 
 ticket_issues.data3, 
 ticket_issues.data4, 
 ticket_issues.data5, 
 to_char(ticket_issues.modification_time, 'Month dd, yyyy hh24:mi:ss') as modification_time,
 ticket_issues.contact_name,
 ticket_issues.contact_email,  ticket_issues.contact_info1,
 to_char(close_date, [ticket_date_format]) close_date,  ticket_priorities.name  as priority_name, ticket_priorities.priority
from ticket_projects, ticket_issues, users, ticket_priorities
where ticket_projects.project_id = ticket_issues.project_id
and ticket_issues.msg_id = $msg_id
and users.user_id(+) = closed_by
and ticket_priorities.priority = ticket_issues.priority"]

set_variables_after_query


# get current user's email, to export as the "last modified by" value
set email [database_to_tcl_string $db "select email from users where user_id=[ad_get_user_id]"]


#if { [empty_string_p $deadline] } {
#    set deadline [ns_localsqltimestamp]
#}

if { $public_p == "t" } {
    set PUBLIC_Y_CHECKED checked
    set PUBLIC_N_CHECKED ""
} else {
    set PUBLIC_N_CHECKED checked
    set PUBLIC_Y_CHECKED ""
}

ReturnHeaders

if {[info exists deadline] && [empty_string_p $deadline]} {
    unset deadline
}

append page "[ad_header [clean_up_html $one_line]]

<h2>Ticket #$msg_id</h2>

[ad_context_bar_ws_or_index [list "/ticket/admin/index.tcl" "Ticket Tracker"] [list "project-top.tcl?project_id=$project_id" $title] "One Ticket"]

<hr>

<h2>$ticket_type \#$msg_id: [clean_up_html $one_line]</h2>
<blockquote>
<table border=0><tr><td bgcolor=#f0f0f0>$message</td></tr></table>
</blockquote>

<form action=issue-edit.tcl method=post>
<input type=hidden name=prev_status [export_form_value status]>
[export_form_vars email msg_id]

<blockquote>
<table border=0 cellspacing=3>
<tr>
  <th valign=top align=left>Subject:</th>
  <td><input size=64 maxsize=200 type=text name=one_line [export_form_value one_line]></td>
<tr>
<th valign=top align=left>Ticket Type:</th><td><select name=ticket_type>[ticket_html_select_ticket_type $ticket_type]</select>
&nbsp;&nbsp;&nbsp;<b>Public?</b>
<input type=radio name=public_p value=t $PUBLIC_Y_CHECKED> Yes
<input type=radio name=public_p value=f $PUBLIC_N_CHECKED> No
</tr>"

# Status
append page "
<tr><td align=left><b>Status:</b><td>"

if {[string compare $status "closed"] == 0} {
    append page "Closed (<a href=\"issue-reopen.tcl?msg_id=$msg_id\">Reopen this issue</a>)\n [export_form_vars status]" 
} else {
    append page "<select name=status>
[ad_generic_optionlist [ticket_status_types] [ticket_status_types] $status]
</select>
"
}

# Project
append page "</td></tr>
<tr><th align=left>Project:<td><select name=project_id>
[ad_db_optionlist $db  "select title,
project_id from ticket_projects
order by title asc" $project_id]
</select>
</td></tr>"

# Severity
append page "<tr><td align=left><b>Severity:</b><td>"
append page "<select name=severity>
[ad_generic_optionlist [ticket_severity_types] [ticket_severity_types] $severity]
</select></td></tr>
"

# Deadline
append page "
<tr><td align=left><b>Deadline:</b><td>[ticket_dateentrywidget_with_nulls deadline [export_var deadline  [database_to_tcl_string $db "select to_char(sysdate, 'yyyy') from dual"]]]
</tr>"

# Priority
append page "<tr><td align=left><b>Priority:</b><td><select name=priority>
[ad_db_optionlist $db  "select name, priority from ticket_priorities order by priority" $priority]
</select>
<input type=hidden name=prev_priority [export_form_value priority]>
</tr>"



append page "<input type=hidden name=prev_severity [export_form_value severity]>"

append page "<tr><th align=left>Contact Name</th><td><input type=text name=contact_name [export_form_value contact_name] size=50></tr>"

append page "<tr><th align=left>Contact Email</th><td><input type=text name=contact_email [export_form_value contact_email] size=50></tr>"

append page "<tr><th align=left>Contact Info</th><td><textarea rows=3 cols=40 name=contact_info>$contact_info1</textarea></tr>"


# Show customizable picklist fields
foreach field [ticket_picklist_field_names] { 
    set entry [ticket_picklist_field_info $field]
    append page "<tr>[ticket_picklist_html_fragment $field [set [ticket_picklist_entry_column_name $entry]]]</tr>\n"
}

append page "</td></tr>
<tr><td align=left valign=top><b>Notify project<br>members via email:</b>
"

if {$notify_p == "t" } {
    append page "<td valign=top>Yes (<a href=\"notification-set.tcl?msg_id=$msg_id&notify=f&return_url=[ns_urlencode "issue-view.tcl?msg_id=$msg_id"]\">toggle</a>)</tr>\n"
} else {
    append page "<td valign=top>No (<a href=\"notification-set.tcl?msg_id=$msg_id&notify=t&return_url=[ns_urlencode "issue-view.tcl?msg_id=$msg_id"]\">toggle</a>)</tr>\n"
}

append page "
<tr><td align=left><b>Submitted By:</b><td>$name on $posting_date</tr>
<tr><td align=left><b>Last Modified By:</b><td>$last_modified_by on $modification_time</tr></tr>
"

append page "<tr><td colspan=2 align=center>
<input type=submit name=submit value=\"Update\">
</td></tr>

</form>
"

if { $close_date != "" } {
    append page "<tr><td align=left><b>Closed On:</b><td>$close_date</tr>\n"
}

append page "

</table>
</blockquote>
<p>
"


set selection [ns_db select $db "select 
 response_id, public_p,
 users.first_names || ' ' || users.last_name as name,
 to_char(posting_time, [ticket_date_format]) as posting_date,
 message
from ticket_issue_responses, users
where ticket_issue_responses.user_id = users.user_id
and ticket_issue_responses.response_to = $msg_id
order by posting_time"] 

while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set text "<blockquote>
    $message
    </blockquote>
    Submitted by $name on $posting_date, "

    if {$public_p == "t"} {
	 append text "<i>Public?</i> Yes (<a href=\"set-response-visibility.tcl?response_id=$response_id&public_p=f&msg_id=$msg_id\">toggle</a>)"
    } else {
	append text  "<i>Public?</i> No (<a href=\"set-response-visibility.tcl?response_id=$response_id&public_p=t&msg_id=$msg_id\">toggle</a>)"
    }

    lappend responses $text

}

if { [info exists responses] } {
    append page "<b>Comments</b>
    [join $responses "<hr width=300>"]
    "
}

append page "<ul>"

if { $close_date == "" } {
    append page "<li><a href=\"issue-response.tcl?msg_id=$msg_id\">Add comment to \"[clean_up_html $one_line]\"</a>
<li><a href=\"issue-close.tcl?[export_url_vars msg_id ticket_type]\">Close this issue</a>
<li><a href=\"issue-fix.tcl?[export_url_vars msg_id ticket_type]\">Fix this issue</a>

"
} 


append page "
</ul>"

# List xrefs
append page "<b>Related Issues</b>
<br>
"
set selection [ns_db select $db "select to_ticket, one_line as xone_line, msg_id as xmsg_id
from ticket_xrefs, ticket_issues
where  to_ticket = ticket_issues.msg_id and 
from_ticket=$msg_id 
union
select to_ticket, one_line xone_line, msg_id xmsg_id
from ticket_xrefs, ticket_issues
where  from_ticket = ticket_issues.msg_id and 
to_ticket=$msg_id 
"]


# target for subroutine pages to return to this page
set target "issue-view.tcl"
append page "<ul>"
while {[ns_db getrow $db $selection]} {
    set_variables_after_query
    append page "<li><a href=\"issue-view.tcl?msg_id=$xmsg_id\">\[$xmsg_id\] $xone_line</a>&nbsp;&nbsp; <a href=\"unlink-xref.tcl?from_msg_id=$msg_id&to_msg_id=$xmsg_id&[export_url_vars target]\">(unlink)</a>"
}

append page "<p><li><a href=\"add-xref-search.tcl?[export_url_vars target msg_id one_line]\">Add a cross-reference</a>"

append page "</ul>"

# View change log
append page "<p>
<ul>
<li><a href=issue-change-log.tcl?msg_id=$msg_id>View Change History</a>
<li><a href=\"issue-new.tcl?project_id=$project_id\">Add a new issue.</a>
</ul>


<p>
<b>Assignment</b>
<table border=1 cellpadding=10>
<tr>
<th>Users assigned to this issue</th>
<th>Candidates for assignment</th>
</tr>
<tr valign=top><td><ul>
"

# query for the users assigned to this issue already
set selection [ns_db select $db "select first_names, last_name, email, users.user_id
from users, ticket_issue_assignments 
where users.user_id = ticket_issue_assignments.user_id 
AND msg_id=$msg_id"]

append page "
<form method=get action=issue-deassign-user.tcl>
[export_form_vars msg_id project_id]

<select name=assignee_id>"

while {[ns_db getrow $db $selection]} {
    set_variables_after_query
    append page "<option value=$user_id>$last_name, $first_names ($email)</option>
"
}


append page "</select>
<input type=submit value=\"Deassign User\">
</form>"


append page  "</ul>
<br>
</td>

<td>
"

# query for the users on the relevant project 
# but not assigned to this issue already

set ticket_group [ticket_admin_group $db]

set selection [ns_db select $db "select users.user_id, first_names, last_name, email
from ticket_assignments, users_active users 
where ticket_assignments.user_id = users.user_id 
and ticket_assignments.project_id = $project_id
and ticket_assignments.user_id NOT IN 
    (SELECT ticket_issue_assignments.user_id 
     from ticket_issue_assignments, ticket_issues
     where ticket_issues.msg_id=$msg_id
     and ticket_issue_assignments.msg_id = ticket_issues.msg_id)
union
select users.user_id, first_names, last_name, email
from user_group_map, users_active users where 
user_group_map.user_id = users.user_id 
and user_group_map.group_id = $ticket_group
and users.user_id NOT IN 
    (SELECT ticket_issue_assignments.user_id 
     from ticket_issue_assignments, ticket_issues
     where ticket_issues.msg_id=$msg_id
     and ticket_issue_assignments.msg_id = ticket_issues.msg_id)
order by last_name
"]

append page "
<form method=get action=issue-assign-user.tcl>
[export_form_vars msg_id project_id]

<select name=assignee_id>"

while {[ns_db getrow $db $selection]}  {
    set_variables_after_query
    append page "<option value=$user_id>$last_name, $first_names ($email)</option>
"
}

append page "</select>
<input type=submit value=\"Assign User\">
</form>

</ul>
<p>
</form>
</td>
</tr>
</table>
"



append page "[ad_footer]"

ns_write $page



