set_form_variables 0

# optional: closed, assigned_to, created_by, unassigned, project_id, order_by deferred

# assigned_to and created_by can be user IDs to filter by.
# By default, only non-closed issues are shown; closed indicates
# that closed issues should also be shown.


set db [ticket_getdbhandle]

set user_id [ad_get_user_id]
set where_clause_list [list "ticket_priorities.priority = ticket_issues.priority"]

if { [info exists closed] && $closed == 1 } {
    set view "All Issues"
} else {
    lappend where_clause_list "close_date is null"
    set view "Open Issues"
    set closed 0
}

if {[info exists deferred] && $deferred == 1} {
    # 
} else {
    lappend where_clause_list "status <> 'deferred'"
    set deferred 0
}

if { [info exists created_by] } {
    lappend where_clause_list "ticket_issues.user_id = $user_id"
    append view " created by you"
}


if { [info exists project_id] && $project_id != "" } {
    lappend where_clause_list "ticket_issues.project_id = $project_id"
    set project_title [database_to_tcl_string $db "select title
from ticket_projects where project_id = $project_id"]
    append title " in project $project_title"
} else {
   set project_title "View All Projects"
}

ReturnHeaders

ns_write "[ad_header $project_title]
<h2>$project_title</h2>"

ns_write "[ad_context_bar_ws_or_index [list "/ticket/admin/index.tcl" "Ticket Tracker"] $project_title]"


ns_write "
<hr>
"

if {![info exists order_by]} {
    set order_by "msg_id"
}

if {[string match "project*" $order_by]} {
    set order_by "project_title"
}


set selection [ns_db select $db "select 
ticket_issues.msg_id,
ticket_issues.ticket_type, 
ticket_issues.one_line,
ticket_issues.status,
ticket_issues.severity,
users.email,
ticket_projects.title as project_title,
ticket_projects.project_id,
ticket_issues.priority, 
to_char(ticket_issues.modification_time, 'mm/dd/yy') as modification_time,
to_char(ticket_issues.posting_time, 'mm/dd/yy') as creation_time,
to_char(ticket_issues.close_date, 'mm/dd/yy') as close_date,
to_char(ticket_issues.deadline, 'mm/dd/yy') as deadline,
to_char(sysdate - deadline) as pastdue,
ticket_issues.public_p,
ticket_priorities.name as priority_name 
from ticket_issues, ticket_priorities, ticket_projects, users
where [join $where_clause_list " and "]
and users.user_id = ticket_issues.user_id
and ticket_projects.project_id = ticket_issues.project_id
order by $order_by, ticket_priorities.priority, posting_time"]

set last_priority "starting"

set count 0

ns_write "<table border=0>
<tr>
<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order msg_id $order_by]&[export_url_vars project_id closed deferred]\">ID#</a></th>
<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order priority $order_by]&[export_url_vars project_id closed deferred]\">Pri</a></th>
<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order ticket_type $order_by]&[export_url_vars project_id closed deferred]\">Typ</a></th>
<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order email $order_by]&[export_url_vars project_id closed deferred]\">Owner</a></th>
<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order status $order_by]&[export_url_vars project_id closed deferred]\">Stat</a></th>
<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order severity $order_by]&[export_url_vars project_id closed deferred]\">Sever</a></th>
<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order posting_time $order_by]&[export_url_vars project_id closed deferred]\">Creat</a></th>
<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order modification_time $order_by]&[export_url_vars project_id closed deferred]\">Mod</a></th>
"

if {[info exists closed] &&  $closed == 1} {
    ns_write "<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order close_date $order_by]&[export_url_vars project_id closed deferred]\">Closed</a></th>"
} else {
    ns_write "<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order deadline $order_by]&[export_url_vars project_id closed deferred]\">Deadline</a></th>"
}

ns_write "<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order one_line $order_by]&[export_url_vars project_id closed deferred]\">Synopsis</a></th>
"
if {![string match "project_title" $order_by]} {
    ns_write "<th align=left><a href=\"project-view-issues.tcl?order_by=[toggle_order project_id $order_by]&[export_url_vars project_id closed deferred]\">Project</a></th>
"    
}
ns_write "</tr>\n"

set last_project_title ""

while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    set cols {}
    lappend cols "<a href=\"issue-view.tcl?msg_id=$msg_id\">$msg_id</a>"
    lappend cols "P$priority"
    lappend cols "[string range $ticket_type 0 3]"
    regsub "@.*" $email "" email
    lappend cols "$email"
    if { [string compare $status "fixed waiting approval"] == 0 } {
	lappend cols "<font color=#00ff00>(w approv)</font>"
    } elseif { [string compare $status "need clarification"] == 0 } {
	lappend cols "<font color=#ff0000>nd clar</font>"
    } else {
	lappend cols $status
    }
    lappend cols "$severity"
    lappend cols "$creation_time"
    lappend cols "$modification_time"

    if {[info exists closed] && $closed == 1} {
	lappend cols "$close_date" 
    } else {
	if {$pastdue > 0} {
	    lappend cols "<font color=red>$deadline</font>"
	} else {
	    lappend cols "$deadline"
	}
    }

    lappend cols "<a href=\"issue-view.tcl?msg_id=$msg_id\">[clean_up_html $one_line]</a>"

    # show project title if we are not sorting by project
    if {![string match "project_title" $order_by]} {
	lappend cols "<a href=project-top.tcl?project_id=$project_id>[string range $project_title 0 12]</a>"	
    }

    if {[string match "project_title" $order_by] && $last_project_title != $project_title} {
	ns_write "<tr><th colspan=10 align=left><a href=project-top.tcl?project_id=$project_id>$project_title</a></th></tr>\n"
	set last_project_title $project_title
    }

    incr count
    if {($count % 2) == 0} {
	set bgcolor "bgcolor=\#ECECEC"
    } else {
	set bgcolor ""
    }

    ns_write "<tr $bgcolor>"
    foreach col $cols {
	ns_write "<td>$col&nbsp;</td>\n"
    }
    ns_write "</tr>"    
}

if { $count == 0 } {
    ns_write "<tr><td colspan=10 align=center>-- No issues --</td></tr>"   
}  else { 
    ns_write "<tr><td colspan=10>$count issues found</td></tr>"
}

ns_write "</table>\n<p>"



if { $closed == 0 } {
    ns_write "<a href=\"project-view-issues.tcl?[export_url_vars assigned_to created_by unassigned deferred order_by]&closed=1\">view closed issues as well</a> 
"
} else {
    ns_write "<a href=\"project-view-issues.tcl?[export_url_vars assigned_to created_by unassigned deferred order_by]\">view open issues</a>
"
}

if { $deferred == 0 } {
    ns_write "<br><a href=\"project-view-issues.tcl?[export_url_vars assigned_to created_by unassigned closed order_by]&deferred=1\">view deferred issues as well</a> 
"
} else {
    ns_write "<br><a href=\"project-view-issues.tcl?[export_url_vars assigned_to created_by unassigned order_by]\">view only open issues</a>
    "}


set count 0

ns_write "<p>"

if {[info exists project_id]} {
    ns_write "<A HREF=\"issue-new.tcl?project_id=$project_id\">add a new issue</a>"
} else {
    ns_write "<A HREF=\"issue-new.tcl\">add a new issue</a>"
}

ns_write "[ad_footer]"
