# View assigned tickets, by user
#
# project-view-assignments.tcl by hqm@arsdigita.com June 1999
# 
set_form_variables 0

# form vars:
# 
# filter conditions
#
# project_id (blank for all projects)

set db [ticket_getdbhandle]

set user_id [ad_get_user_id]

ReturnHeaders

ns_write "[ad_header "[ticket_system_name] View Ticket Assignments"]

<h2>[ticket_system_name]</h2>

[ad_context_bar_ws_or_index  [list "/ticket/index.tcl" "Ticket Tracker"] "View Assignments"]

<hr>
"


if {[info exists project_id] && ![empty_string_p $project_id]} {
    set project_filter "and ticket_issues.project_id = $project_id "
} else {
    set project_filter ""
}



set query "select 
 ticket_issues.msg_id,
 ticket_issues.ticket_type, 
 ticket_issues.one_line,
 ticket_issues.status,
 ticket_issues.severity,
 ticket_issues.posting_time,
 users.email as assignee_email,
 ticket_projects.title as project_title,
 ticket_projects.project_id,
 ticket_issues.priority, 
 to_char(ticket_issues.modification_time, 'mm/dd/yy') as modification_time,
 to_char(ticket_issues.posting_time, 'mm/dd/yy') as creation_time,
 to_char(ticket_issues.close_date, 'mm/dd/yy') as close_date,
 to_char(ticket_issues.deadline, 'mm/dd/yy') as deadline,
 to_char(sysdate - deadline) as pastdue,
 ticket_issue_assignments.user_id as assignee_user_id,
 ticket_issues.public_p,
 ticket_priorities.name as priority_name
 from ticket_issues, ticket_priorities, ticket_projects, users, ticket_issue_assignments
 where ticket_priorities.priority = ticket_issues.priority
 and users.user_id = ticket_issue_assignments.user_id
 and ticket_projects.project_id = ticket_issues.project_id
 and ticket_issues.msg_id = ticket_issue_assignments.msg_id
 and ticket_issues.status <> 'closed'
 $project_filter
 order by assignee_user_id, project_title, msg_id
"

set selection [ns_db select $db $query]

set results ""
append results "<table border=0>
<tr><th align=left>User</th>
<th align=left>ID#</th>
<th align=left>Pri</th>
<th align=left>Typ</th>
<th align=left>Stat</th>
<th align=left>Sever</th>
<th align=left>Creat</th>
<th align=left>Mod</th>
<th align=left>Synopsis</th>
<th align=left>Project</th>
</tr>
"

set count 0
set last_email ""

while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    if {$last_email != $assignee_email} {
	append results "<tr><td colspan=10 bgcolor=#cccccc>&nbsp;</tr>"
    }
    set cols {}
    regsub "@.*" $assignee_email "" aemail

    # only print  user email when it changes
    if {$last_email != $assignee_email} {
	lappend cols $aemail
    } else {
	lappend cols ""
    }
    set last_email $assignee_email

    lappend cols "<a href=\"issue-view.tcl?msg_id=$msg_id\">$msg_id</a>"
    lappend cols "P$priority"
    lappend cols "[string range $ticket_type 0 3]"
    if { [string compare $status "fixed waiting approval"] == 0 } {
	lappend cols "<font color=#00ff00>(w approv)</font>"
    } elseif { [string compare $status "need clarification"] == 0 } {
	lappend cols "<font color=#ff0000>nd clar</font>"
    } else {
	lappend cols $status
    }
    lappend cols "$severity"
    lappend cols "$creation_time"
    lappend cols "$modification_time"
    
    lappend cols "<a href=\"issue-view.tcl?msg_id=$msg_id\">[clean_up_html $one_line]</a>"

    # show project title if we are not sorting by project
    lappend cols "<a href=project-top.tcl?project_id=$project_id>[string range $project_title 0 12]</a>"	
    
    incr count
    if {($count % 2) == 0} {
	set bgcolor "bgcolor=\#ECECEC"
    } else {
	set bgcolor ""
    }
    
    append results "<tr $bgcolor>"
    foreach col $cols {
	append results "<td>$col&nbsp;</td>\n"
    }
    append results "</tr>"    
}

if { $count == 0 } {
    append results "<tr><td colspan=10 align=center>-- No issues --</td></tr>"   
}

append results "</table>\n<p>"


append results  "
<ul>
<li><a href=\"issue-new.tcl\">add a new issue</a>
</ul>"

append results "[ad_footer]"
ns_write $results
