set_the_usual_form_variables

set db [ns_db gethandle]

# creator_fname, creator_lname
# contact_fname, contact_lname
# ticket_id
# ticket_type
# creation_start, creation_end 
# project_id, priority

# Check input.

set exception_text ""
set exception_count 0

ticket_search_combine_and_build_error_list

if { $exception_count > 0 } {
    ad_return_complaint $exception_count $exception_text
    return
}

# This looks at a bunch of form vars, and builds a list of search
# clauses in search_clause_list
ticket_search_build_where_clause_and_description

if {![info exists search_description_items] || [llength $search_description_items] == 0} {
    ad_return_complaint 1 "<li>Please specify at least some search criteria.\n"
    return
}


if { [info exists assigned_fname] && [info exists assigned_lname] && (![empty_string_p $assigned_fname] || ![empty_string_p $assigned_lname]) } {
    set assigned_user_conditions [list]
    if {![empty_string_p $assigned_fname]} {
	lappend assigned_user_conditions "(lower(email) like  '[string tolower [DoubleApos $assigned_fname]]%' or  lower(first_names) like '[string tolower [DoubleApos $assigned_fname]]%')"
    } 
    if {  ![empty_string_p $assigned_lname] } {
	lappend assigned_user_conditions "lower(last_name) like '[string tolower [DoubleApos $assigned_lname]]%'"
    }

    lappend search_clause_list " msg_id in (select msg_id from ticket_issue_assignments ta, users where ta.user_id = users.user_id and [join $assigned_user_conditions { or }] ) "

}

if {[llength $search_clause_list] > 0} {
    set search_clause "and [join $search_clause_list " and "]"
} else {
    set search_clause ""
}

set display_title "Ticket search"

set query "select ticket_issues.*, ticket_priorities.name, 
ticket_projects.title as project_title, 
users.first_names,
users.last_name,
to_char(posting_time, 'mm/dd/yyyy') as creation_date
from ticket_issues, ticket_projects, ticket_priorities, users, users closer
where
closer.user_id(+) = ticket_issues.closed_by
and users.user_id = ticket_issues.user_id 
and ticket_issues.priority = ticket_priorities.priority
and ticket_projects.project_id = ticket_issues.project_id
$search_clause
order by ticket_issues.project_id, msg_id"

ReturnHeaders

set selection [ns_db select $db $query]

append pagebody "[ad_header $display_title]
<h2>$display_title</h2>
in <a href=\"\">[ad_system_name]</a>
<hr>

Search criteria:

<ul>
<li>[join $search_description_items "<li>"]
</ul>

<p>

Search results:

<form method=post action=\"list-issues.tcl\">

<input type=hidden name=search_items [export_form_value search_description_items]>
"

set i 0
set ppcount  0
set msgs_displayed_already [list]

set last_title ""

while { [ns_db getrow $db $selection] } {
    set_variables_after_query
    if { [lsearch $msgs_displayed_already msg_id] != -1 } {
	continue
    } else {
	lappend msgs_displayed_already $msg_id
    }
    set detail_list ""

    if {[string compare $project_title $last_title] != 0} {
	if {$ppcount != 0} {
	    append pagebody "<br>$ppcount issues"
	    set ppcount 0
	}
	append pagebody "</ul><b>$project_title</b><ul>"
    }

    set last_title $project_title

    set item_list_list { {"creation_date" "Creation Date:"} {"priority" "Priority:"} {"source" "Source:"} {"data1" ""} {"data2" ""} }

    foreach item_list $item_list_list {
	set variable [lindex $item_list 0]
	set annotation [lindex $item_list 1]
	if ![empty_string_p [set $variable]] {
	    lappend detail_list "$annotation [set $variable]"
	}
    }
    
    append pagebody "<br><input type=checkbox name=msg_ids value=\"$msg_id\" checked>
<a href=\"/ticket/admin/issue-view.tcl?[export_url_vars msg_id]\">\#$msg_id [clean_up_html $one_line]</a>"
   if {[string tolower $status] == "closed"} {
       append pagebody " <font color=green>Status: closed</font> "
   } else {
       append pagebody " Status: $status "
   }
   append pagebody [join $detail_list " ; "]

   incr i
   incr ppcount
}

if {$ppcount != 0} {
    append pagebody "<br>$ppcount issues"
    set ppcount 0
}

if { $i == 0 } {
    append pagebody "No tickets found.\n"
} else {
    append pagebody "
</ul>
<p>
<center>
<input type=submit name=report_type value=\"Ticket Summaries\">
<input type=submit name=report_type value=\"Complete Ticket Reports\">
</center>"
}

append pagebody "</form>
<p>
<ul>
<li><a href=\"issue-new.tcl\">Add a new ticket</a>
</ul>
[ad_footer]
"
ns_write $pagebody
