# slide-attach-2.tcl,v 3.0 2000/02/06 03:55:31 ron Exp
# File:        slide-attach-2.tcl
# Date:        28 Nov 1999
# Author:      Jon Salz <jsalz@mit.edu>
# Description: Adds an attachment.
# Inputs:      slide_id, attachment (file), inline_image_p, display

ad_page_variables {
    slide_id
    attachment
    inline_image_p
    display
}

set db [ns_db gethandle]
set user_id [ad_maybe_redirect_for_registration]

set selection [ns_db 1row $db "select * from wp_slides where slide_id = $slide_id"]
set_variables_after_query
wp_check_authorization $db $presentation_id $user_id "write"

set tmp_filename [ns_queryget attachment.tmpfile]
set guessed_file_type [ns_guesstype $attachment]
set n_bytes [file size $tmp_filename]

# strip off the C:\directories... crud and just get the file name
if ![regexp {([^/\\]+)$} $attachment match client_filename] {
    set client_filename $attachment
}

set exception_count 0
set exception_text ""

if { $n_bytes == 0 } {
    append exception_text "<li>You haven't uploaded a file.\n"
    incr exception_count
}

if { ![empty_string_p [ad_parameter MaxAttachmentSize "comments"]] && $n_bytes > [ad_parameter MaxAttachmentSize "comments"] } {
    append exception_text "<li>Your file is too large.  The publisher of [ad_system_name] has chosen to limit attachments to [util_commify_number [ad_parameter MaxAttachmentSize "comments"]] bytes.\n"
    incr exception_count
}

if { $exception_count > 0 } {
    ad_return_complaint $exception_count $exception_text
    return
}

if { $inline_image_p == "f" } {
    set QQdisplay ""
}

set attachment_id [database_to_tcl_string $db "select empty_lob()"]

ns_db dml $db "begin"
ns_db dml $db "
    insert into wp_attachments(attach_id, slide_id, lob, file_size, file_name, mime_type, display)
    values(nextval('wp_ids'), $slide_id, $attachment_id, $n_bytes, '[DoubleApos $client_filename]', '$guessed_file_type', [db_postgres_null_sql $QQdisplay])"

ns_pg blob_dml_file $db $attachment_id $tmp_filename
ns_db dml $db "end"

ns_returnredirect "slide-attach.tcl?slide_id=$slide_id"
